<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007, 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

/****************************************************
 * Install
 ****************************************************/

class install {

	public  $template;
	private $print;
	private $conf_vars = array ();
	private $silent    = false;  // running in silent mode?
	private $baseurl   = "";    // cmd line argument if running in silent mode
	private $conndbdir = "";  // cmd line argument if running in silent mode
	private $conndb    = "";  // connection to the connections.db database.

	function __construct()
	{
		$informixcontime = 20;
		$informixconretry = 3;
		
		require_once("lib/output.php");
		$this->print = new output();

		$this->load_template("install_template");
		$this->conf_vars = array (
        "LANG"          => array ("default" => "en_US", "desc" => "Default Language to use")
		,"CONNDBDIR"    => array ("default" => "",   "desc" => "the directory for connections database")
		,"BASEURL"      => array ("default" => "",   "desc" => "the BASEURL for links")
		,"HOMEDIR"      => array ("default" => "", "desc" => "where everything is installed")
		,"GOOGLEMAPKEY" => array ("default" => "" , "desc" => "The API key from googlemaps , http://www.google.com/apis/maps/signup.html" )
		,"PINGINTERVAL" => array ("default" => "300", "desc" => "How often to check servers status (seconds)")
		,"SECURESQL"	=> array ("default" => "on", "desc" => "Require login credentials when using SQLToolbox")
		,"INFORMIXCONTIME"	=> array ("default" => "{$informixcontime}", "desc" => "Connect Time: The number of seconds that OAT attempts to connect to the database server before returning an error. ")
		,"INFORMIXCONRETRY"	=> array ("default" => "{$informixconretry}", "desc" => "Connect Retry: The number of times that OAT attempts to connect to the database server during the connect time specified by INFORMIXCONTIME.")
		);
	}
//		,"USEREDIRECT"  => array ("default" => "0" ,"desc" => "if non zero then the # of seconds to pause with a redirect ")

	// set install to run in silent mode
	function setSilentMode($baseurl,$conndbdir)
	{
		$this->silent=true;
		$this->baseurl=$baseurl;
		$this->conndbdir=$conndbdir;
	}

	function run()
	{

		if ( file_exists("install.lock") )
		{
			$this->error("install.lock exists..<br/> This file needs to be removed before install/upgrade can continue",true);
		}
		$option = isset($_GET['do']) ? $_GET['do'] : "unknown";
			
		if ( strlen(stristr("unknown check", $option)) <= 0 ) {
			if ( !file_exists("99") ) {
				$this->error("You need to accept the license agreement first." ,true);
			}
		}
			
		if ($this->silent)
		{
			$option="silent";
		}

		switch ($option)
		{
			case "start":
				$this->start();
				break;
			case "check":
				$this->check();
				break;
			case "config":
				$this->config();
				break;
			case "saveconfig":
				$this->saveConfig();
				break;
			case "dbs":
				$this->dbs();
				break;
			case "dodbs":
				$this->dodbs();
				break;
			case "plugins":
			    $this->plugins();
			    break;
			case "installplugins":
			    $this->installplugins();
			    break;
			case "final":
				$this->fin();
				break;
			case "silent":
				$this->doSilentInstall();
				break;
			default:
				$this->start();
				break;
		}

		if ( ! $this->silent )
		{
			$this->print->render();
		}

	} // end run

	function start()
	{
		$this->print->setPageTitle("Install OAT");
		$this->print->add_to_output($this->template->start());
	}// end start

	function error($str, $fatal=false)
	{
		if ( $this->silent )
		{
			// errors for silent install should be sent to the console
			if ($fatal)
			{
				echo "\nFATAL ERROR:\n" . $str . "\n";
				exit(1);
			} else {
				echo "\nERROR:\n" . $str . "\n";
			}
		} else {

			$this->print->add_to_output($this->template->error($str));
			if ( $fatal )
			{
				$this->print->render();
				die();
			}
		}
	}  // error

	function warning($str)
	{
		$this->print->add_to_output($this->template->warning($str));
	}  // warning

	function check()
	{
		// License Agreement first ...
		if( !file_exists("99") ) {
			if (isset($_POST['install_start']) && $_POST['Lic_CB']) {
				fopen("99","w");
			} else {
				$this->error("You need to accept the license agreement first to continue.",true);
			}
		}

		$show_next = true;
		$show_warning = false;
		$this->print->setPageTitle("Install OAT - Check Phase");
		/* Perform some sanity checks */
		$needed = array("PDO" => "true"
		,"pdo_informix" => "true"
		,"pdo_sqlite" => "true"
		,"soap" => "true"
		,);
		$recommended = array("gd" => "true" 
		, "zip" => "true"
		,);

		foreach ($needed as $k => $v)
		{
			$needed[$k] = $this->check_module($k);
			if ( $needed[$k] === false || $needed[$k] == "")
			$show_next = false;
		}
		foreach ($recommended as $k => $v)
		{
			$recommended[$k] = $this->check_module($k);
			if ( $recommended[$k] === false || $recommended[$k] == "")
			$show_warning = true;
		}

		if ( $this->silent )
		{
			if ( ! $show_next )
			{
				$errmsg = "Not all required PHP modules are loaded. \n";
				$errmsg .= "Missing module(s) : \n";
				foreach ($needed as $module => $value)
				{
					if (!$value)
					{
						$errmsg .= " -  $module \n";
					}
				}
				$this->error($errmsg,true);
			}

			if ( $show_warning )
			{
				$warn_msg = "\nNot all recommended PHP modules are loaded.  " .
                "While the recommended modules are not required to complete " .
                "the installation or start using the tool, you will not be " .
                "able to utilize OpenAdmin Tool's full functionality " .
                "without these modules. \n" .
                "Missing recommended module(s) : \n";
				foreach ($recommended as $module => $value)
				{
					if (!$value)
					{
						$warn_msg .= " -  $module \n";
					}
				}
				echo ($warn_msg);
			}

		} else {

			$this->print->add_to_output($this->template->show_required($needed));

			$this->print->add_to_output($this->template->show_recommended($recommended));

			$this->print->add_to_output($this->template->show_next($show_next,"config"));
		}

	} // end check

	function check_module($module)
	{
		$extensions = get_loaded_extensions();
		if (in_array($module, $extensions ) )
		{
			return true;
		}
		return false; // return false returns a blank string ! ?
	} // end check_module

	function config($config = array(), $warn_dir = '')
	{
		$homedir = str_replace( '\\', '/', getcwd() ) . '/';
		$homedir = substr($homedir, 0, -8);

		if (isset($config['CONNDBDIR'])){
			$conndbdir = $config['CONNDBDIR'];
		}else{
                	$conndbdir = str_replace( '\\', '/', getcwd() ) . '/';
	                $conndbdir = substr($conndbdir, 0, -8) . 'conf/';
		}

		// If conf/config.php does not exist, copy the default
		if (!file_exists($conndbdir."/config.php") && file_exists($conndbdir."/config.default.php"))
		{
			copy($conndbdir."/config.default.php", $conndbdir."/config.php");
		}

		if (isset($config['BASEURL'])){
			$baseurl = $config['BASEURL'];
		}else{
			$baseurl = substr($_SERVER['HTTP_REFERER'],0, -27);
		}

		$lang = isset($config['LANG']) ? $_POST['LANG'] : "en_US";
		$googlemapkey = isset($config['GOOGLEMAPKEY']) ? $_POST['GOOGLEMAPKEY'] : "";
		$pinginterval = isset($config['PINGINTERVAL']) ? $_POST['PINGINTERVAL'] : "300";
		if (!empty($config)&&!isset($config['SECURESQL'])){
			$secureSQL = "";
		}else{
			$secureSQL = "on";
		}
		$informixcontime = isset($config['INFORMIXCONTIME']) ? $_POST['INFORMIXCONTIME']:20;
		$informixconretry = isset($config['INFORMIXCONRETRY']) ? $_POST['INFORMIXCONRETRY']:3;

		$this->conf_vars = array (
        "LANG"          => array ("default" => "{$lang}",        "desc" => "Default Language to use")
		,"CONNDBDIR"    => array ("default" => "{$conndbdir}",   "desc" => "the directory for connections database")
		,"BASEURL"      => array ("default" => "{$baseurl}",     "desc" => "the BASEURL for links")
		,"HOMEDIR"      => array ("default" => "{$homedir}",     "desc" => "where everything is installed")
		,"GOOGLEMAPKEY" => array ("default" => "{$googlemapkey}","desc" => "The API key from googlemaps , http://www.google.com/apis/maps/signup.html" )
		,"PINGINTERVAL" => array ("default" => "{$pinginterval}","desc" => "How often to check servers status (seconds)")
		,"SECURESQL"	=> array ("default" => "{$secureSQL}", "desc" => "Require login credentials when using SQLToolbox")
		,"INFORMIXCONTIME"	=> array ("default" => "{$informixcontime}", "desc" => "Connect Time: The number of seconds that OAT attempts to connect to the database server before returning an error. ")
		,"INFORMIXCONRETRY"	=> array ("default" => "{$informixconretry}", "desc" => "Connect Retry: The number of times that OAT attempts to connect to the database server during the connect time specified by INFORMIXCONTIME.")
		);

		$this->print->setPageTitle("Install OAT - Create Config");
		$this->print->add_to_output($this->template->config_start());
		$this->print->add_to_output($this->template->config_row($this->conf_vars));
		$this->print->add_to_output($this->template->config_warning($warn_dir));
		$this->print->add_to_output($this->template->config_end());
	} // end config

	function saveConfig()
	{

		if ( ! isset ( $_POST['LANG']) )
		{
			$this->error("Missing value LANG for config");
			return $this->config($_POST);
		}

		if ( ! isset ( $_POST['BASEURL']) )
		{
			$this->error("Missing value BASEURL for config");
			return $this->config($_POST);
		}

		if ( ! isset ( $_POST['HOMEDIR']) )
		{
			$this->error("Missing value HOMEDIR for config");
			return $this->config($_POST);
		}


		if ( ! isset ( $_POST['GOOGLEMAPKEY']) )
		{
			$this->error("Missing value GOOGLEMAPKEY for config");
			return $this->config($_POST);
		}

		if ( ! isset ( $_POST['PINGINTERVAL']) )
		{
			$this->error("Missing value PINGINTERVAL for config");
			return $this->config($_POST);
		}
		
		if ( ! isset ( $_POST['INFORMIXCONTIME']) )
		{
			$this->error("Missing value INFORMIXCONTIME for config");
			return $this->config($_POST);
		}
		
		if ( ! isset ( $_POST['INFORMIXCONRETRY']) )
		{
			$this->error("Missing value INFORMIXCONRETRY for config");
			return $this->config($_POST);
		}
		
		if ( ! is_dir($_POST['HOMEDIR']) )
		{
			$this->error("HOMEDIR is not valid");
			return $this->config($_POST);
		}

		if ( ! isset ( $_POST['CONNDBDIR']) )
		{
			$this->error("Missing value CONNDBDIR for config");
			return $this->config($_POST);
		}

		if ( ! is_dir($_POST['CONNDBDIR']) || ( is_writable($_POST['CONNDBDIR']) === false ) )
		{
			$this->error("CONNDBDIR - {$_POST['CONNDBDIR']} -  is not valid. Please check it exists and is writable.");
			return $this->config($_POST);
		}

		$cwd = realpath(getcwd());
		$request_uri = dirname(getenv('REQUEST_URI'));
		$l = strlen($cwd) - strlen($request_uri);
		$doc_root = substr($cwd,0,$l);
		$conndbdir = realpath($_POST['CONNDBDIR']);

		if (!stristr($conndbdir,$doc_root)===false)
		{
			//On windows, "\" signs are changed to "\\" in the post data
			if (substr(PHP_OS,0,3) === "WIN")
			{
				$warned_dir = str_replace("\\\\","\\",$_POST['warned_dir']);
			}
			else
			{
				$warned_dir = $_POST['warned_dir'];
			}

			if ((strcasecmp('',$warned_dir)==0)||(strcasecmp($conndbdir,$warned_dir)!=0))
			{
				$this->warning("The connections database location ({$conndbdir}) is under the webserver's document root. This may expose OAT to security risks. While it is highly recommended that you change the CONNDBDIR to a secure location, you may choose to ignore this warning by clicking 'Save' again.");
				return $this->config($_POST,$conndbdir);
			}
		}

		$homedir = $_POST['HOMEDIR']."/conf";
		if ( ( is_dir($homedir) === false )
		|| ( is_writable($homedir) === false )  )
		{
			$this->error("{$homedir} is not accessible. Check it exists and is writable.");
			return $this->config($_POST);
		}

		$conffile=$homedir."/config.php";
		if ( is_writable($homedir."/config.php") === false )
		{
			$this->error("{$homedir}/config.php is not accessible. Check it exists and is writable.");
			return $this->config($_POST);
		}

		// write out the config file ..
		$fd = fopen($conffile,'w+');

		fputs($fd,"<?php \n");
		foreach ($this->conf_vars as $k => $v)
		{
			$out = "\$CONF['{$k}']=\"{$_POST[$k]}\";#{$v['desc']}\n";
			fputs($fd,$out);
		}
		fputs($fd,"?>\n");
		fclose($fd);

		// redirect to next stage which is dbs..
		header("Location: index.php?do=dbs");
		die();
	} // end SaveConfig

	/**
	 *
	 * For silent install: save default values in config file
	 */
	function saveDefaultConfig()
	{
		$homedir = str_replace( '\\', '/', getcwd() ) . '/';
		$homedir = substr($homedir, 0, -8);

		$conndbdir = $this->conndbdir;

		if(!is_dir($this->conndbdir))
		{
			$chk = exec('mkdir "'.$this->conndbdir.'"');
			if($chk!=0)
			{
				$this->error("Failed to create {$this->conndbdir}. Make sure location is writable.",true);
			}
		}

		// If conf/config.php does not exist, copy the default
		if (!file_exists($homedir."/conf/config.php") && file_exists($homedir."/conf/config.default.php"))
		{
			copy($homedir."/conf/config.default.php", $homedir."/conf/config.php");
		}

		// default config values
		$lang         = "en_US";
		$useredirect  = "0";
		$googlemapkey = "";
		$pinginterval = "300";
		$secureSQL    = "on";
		$informixcontime = 20;
		$informixconretry = 3;

		$this->conf_vars = array (
         "LANG"         => array ("default" => "{$lang}"          ,"desc" => "Default Language to use")
		,"CONNDBDIR"    => array ("default" => "{$conndbdir}"     ,"desc" => "the directory for connections database")
		,"BASEURL"      => array ("default" => "{$this->baseurl}" ,"desc" => "the BASEURL for links")
		,"HOMEDIR"      => array ("default" => "{$homedir}"       ,"desc" => "where everything is installed")
		,"GOOGLEMAPKEY" => array ("default" => "{$googlemapkey}"  ,"desc" => "The API key from googlemaps , http://www.google.com/apis/maps/signup.html" )
		,"PINGINTERVAL" => array ("default" => "{$pinginterval}"  ,"desc" => "How often to check servers status (seconds)")
		,"SECURESQL"	=> array ("default" => "{$secureSQL}"     ,"desc" => "Require login credentials when using SQLToolbox")
		,"INFORMIXCONTIME"	=> array ("default" => "{$informixcontime}", "desc" => "Connect Time: The number of seconds that OAT attempts to connect to the database server before returning an error. ")
		,"INFORMIXCONRETRY"	=> array ("default" => "{$informixconretry}", "desc" => "Connect Retry: The number of times that OAT attempts to connect to the database server during the connect time specified by INFORMIXCONTIME.")
		);

//		,"USEREDIRECT"  => array ("default" => "{$useredirect}"   ,"desc" => "if non zero then the # of seconds to pause with a redirect ")
		if ( ! is_dir($homedir) )
		{
			$this->error("HOMEDIR is not valid",true);
		}

		//TODO: Nice to warn user if the directory entered is still web accessible .. !!

		if ( ! is_dir($conndbdir) || ( is_writable($conndbdir) === false ) )
		{
			$this->error("CONNDBDIR - $conndbdir -  is not valid. Please check it exists and is writable.",true);
		}

		$homedir = $homedir . "/conf";
		if ( ( is_dir($homedir) === false )
		|| ( is_writable($homedir) === false )  )
		{
			$this->error("{$homedir} is not accessible. Check it exists and is writable.",true);
		}

		$conffile=$homedir."/config.php";
		if ( is_writable($homedir."/config.php") === false )
		{
			$this->error("{$homedir}/config.php is not accessible. Check it exists and is writable.",true);
		}

		// write out the config file ..
		$fd = fopen($conffile,'w+');

		# write out the conf
		fputs($fd,"<?php \n");
		foreach ($this->conf_vars as $k => $v)
		{
			$out = "\$CONF['{$k}']=\"{$v['default']}\";#{$v['desc']}\n";
			fputs($fd,$out);
		}
		fputs($fd,"?>\n");
		fclose($fd);

	} // end saveDefaultConfig


        /**
         * Language in config file must have format "en_US".
         * Earlier versions of OAT just used "en", so if we find that, we need to update it.
         */
        function checkConfigLang($config_file)
        {
		if ( ! file_exists($config_file))
                {
                    return;
                }

                $CONF=array();
                require($config_file);
			
                // If the config file's lang is "en", change it to "en_US"
                if (isset($CONF['LANG']) && $CONF['LANG'] == "en")
                {
                        $CONF['LANG'] = "en_US";

                        // Re-write config file
                        $fd = fopen($config_file,'w+');

                        fputs($fd,"<?php \n");
                        foreach ($this->conf_vars as $k => $v)
                        {
                            $out = "\$CONF['{$k}']=\"{$CONF[$k]}\";#{$v['desc']}\n";
                            fputs($fd,$out);
                        }
                        fputs($fd,"?>\n");
                        fclose($fd);
                }
	}

	function dbs()
	{
		$vernum = "0";

		require_once("../conf/config.php");

		if ( ! isset($CONF['CONNDBDIR']) )
		{
			$this->error("Please check config.php param CONNDBDIR - it doesnt seem to be set.");
			return false;
		}

		$file="{$CONF['CONNDBDIR']}/connections.db";
		if ( ! is_dir($CONF['CONNDBDIR']) )
		{
			$this->error("Please check config.php param CONNDBDIR - it doesnt seem to be set to a directory.");
			return false;
		}

		unset($CONF);

		$this->conndb = new PDO("sqlite:{$file}");

		$stmt = $this->conndb->query("SELECT * FROM version");
		if ( $this->conndb->errorCode() == "00000" )
		{
			$version  = $stmt->fetch();
			$stmt->closeCursor();
			/* we got a version so lets run an upgrade */
			$vernum = $version['version_num'];
		}

		$this->print->setPageTitle("Install OAT - connections database");
		if ( $vernum == "0")
		{
			$this->print->add_to_output($this->template->createDatabases());
			$this->print->add_to_output($this->template->show_next(true,"dodbs"));
		}
		else
		{
			$this->print->add_to_output($this->template->updateDatabase());
			$this->print->add_to_output($this->template->show_next(true,"dodbs&amp;version={$vernum}"));
		}
	} // end dbs


	/**
	 *  dodbs :  create / upgrade the connections.db database.
	 *
	 * @return boolean
	 *
	 * Returns false if there was an error.
	 * Returns true if creation of connections database succeeded.
	 */
	function dodbs()
	{

		$chk = $this->open_connectionsdb();
		if($chk === false){
			return false;
		}
        // if we are being called with a version then we need to upgrade the connections.db.
		if ( isset( $_GET['version'] ) )
		{
			return $this->dodbs_upgrade( $_GET['version'] );
		}

        //create the default 220 database tables.
		$this->conndb->query("DROP TABLE groups;");
		$this->conndb->query("CREATE TABLE groups ( group_num INTEGER PRIMARY KEY AUTOINCREMENT , group_name VARCHAR(40) , password VARCHAR(20) , readonly CHAR(1) )");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the GROUPs table.");
			return false;
		}

		$this->conndb->query("DROP TABLE clusters;");
		$this->conndb->query("CREATE TABLE clusters (cluster_id INTEGER PRIMARY KEY AUTOINCREMENT , cluster_name VARCHAR(50))");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the clusters table.");
			return false;
		}

		$this->conndb->query("DROP TABLE connections;");

		$this->conndb->query("CREATE TABLE connections ( conn_num INTEGER PRIMARY KEY AUTOINCREMENT , group_num INTEGER, nickname VARCHAR(40), host VARCHAR(200) , port VARCHAR(30) , server VARCHAR(128) , lat float , lon float , username varchar(128), password varchar(20), lastpingtime int , laststatus int , laststatusmsg int, lastonline int , cluster_id int , last_type int , cwd varchar(256) );");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the connections table.");
			return false;
		}

		/**
		 * Create conn_envvars table.
		 * Used to let users specified env variables on their connections.
		 */
		$this->conndb->query("DROP TABLE conn_envvars;");
		$this->conndb->query("CREATE TABLE conn_envvars ( conn_num INTEGER, envvar_name VARCHAR(128) , envvar_value VARCHAR(512) )");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the CONN_ENVVARS table.");
			return false;
		}

		/**
		 * Create the Dashboard tables
		 */
		$this->conndb->query("DROP TABLE dashboards;");
		$this->conndb->query("CREATE TABLE dashboards ( dashboard_id INTEGER PRIMARY KEY AUTOINCREMENT , dashboard_refresh INT, dashboard_name VARCHAR(100), dashboard_description TEXT );");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the dashboards table.");
			return false;
		}
		$this->conndb->query("INSERT INTO dashboards  (dashboard_refresh,dashboard_name,dashboard_description) VALUES (10,'Default','An example dashboard that shows Memory and Transactions')");
		$this->conndb->query("INSERT INTO dashboards  (dashboard_refresh,dashboard_name,dashboard_description) VALUES (10,'Space','An example dashboard that shows Space')");

		/**
		 * Create the panels tables.
		 */
		$this->conndb->query("DROP TABLE panels;");
		$this->conndb->query("CREATE TABLE panels ( panel_id INTEGER PRIMARY KEY AUTOINCREMENT ,  panel_short_name VARCHAR(20) , panel_title VARCHAR(100), panel_description TEXT  );");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the panels table.");
			return false;
		}

		$this->conndb->query("INSERT INTO panels VALUES (1,'memory','Overall Shared Memory Usage','This panel shows overall shared memory usage')");
		$this->conndb->query("INSERT INTO panels VALUES (2,'transactions','Overall Number of Transactions','This panel shows the overall number of transactions')");
		$this->conndb->query("INSERT INTO panels VALUES (3,'space','Used / Free Space for DBSpaces','This panel shows used/free space per dbspace')");
		$this->conndb->query("INSERT INTO panels VALUES (4,'locks','Overall Number of Locks','This panel shows the overall number of locks')");


		/**
		 * Create the DashPanel table - this table describes what panels are on a dashboard and at what position.
		 */
		$this->conndb->query("DROP TABLE dashpanels;");
		$this->conndb->query("CREATE TABLE dashpanels ( dashid INTEGER , panelid INTEGER , pos INTEGER );");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the dashpanels table.");
			return false;
		}

		$this->conndb->query("INSERT INTO dashpanels VALUES (1,1,1)");
		$this->conndb->query("INSERT INTO dashpanels VALUES (1,2,2)");

		$this->conndb->query("INSERT INTO dashpanels VALUES (2,3,1)");
		$this->conndb->query("INSERT INTO dashpanels VALUES (2,4,2)");

		/**
		 * Create the pinger info table.
		 */
		$this->conndb->query("CREATE TABLE 'pingerinfo' ('lastrun' INTEGER, 'isrunning' INTEGER, 'result' TEXT)");
		$this->conndb->query("INSERT INTO pingerinfo VALUES (0,0,'')");


		/*
		 * Create the idsd table. It links a connection with
		 * the IDS Daemon (idsd) controlling it.
		 */

		$this->conndb->query ( "CREATE TABLE idsd ( cid INTEGER PRIMARY KEY, host VARCHAR, port INTEGER ) ");
		$this->conndb->query ( "CREATE TRIGGER idsd_del DELETE ON connections BEGIN DELETE FROM idsd WHERE cid = old.conn_num; END" );

		/*
		 * Create the env and env_link tables. They store the environment
		 * variables for a server.
		 */

		$this->conndb->query ( "CREATE TABLE env ( eid INTEGER, name VARCHAR, value VARCHAR )" );
		$this->conndb->query ( "CREATE TABLE env_link ( eid INTEGER PRIMARY KEY AUTOINCREMENT, cid INTEGER, stamp INTEGER )" );
		$this->conndb->query ( "CREATE UNIQUE INDEX env_link_idx ON env_link ( cid )" );
		$this->conndb->query ( "CREATE TRIGGER env_link_del DELETE ON connections BEGIN DELETE FROM env_link WHERE cid = old.conn_num; END" );
		$this->conndb->query ( "CREATE TRIGGER env_del DELETE on env_link WHERE eid = old.eid; END" );
			
		/*
		 * Insert some default values into the necessary tables..
		 */
		$this->conndb->query("INSERT INTO groups VALUES (NULL,'Default','','')");

		/*
		 * Create the necessary Triggers
		 */
		$this->conndb->query("CREATE TRIGGER group_del DELETE on groups BEGIN delete from connections where connections.group_num = old.group_num;END");
		$this->conndb->query("CREATE TRIGGER cluster_del DELETE on clusters BEGIN update connections set cluster_id=0 where connections.cluster_id = old.cluster_id;END");
		$this->conndb->query("CREATE UNIQUE index i_clust_name ON clusters(cluter_name)");
		$this->conndb->query("CREATE TRIGGER dashboard_del DELETE on dashboards BEGIN delete from dashpanels where dashid = old.dashboard_id;END");

		/**
		 * drop and recreate the version table
		 * the version table is used to perform upgrades
		 * to the connections db
		 */
		$this->conndb->query("DROP TABLE version");
		$this->conndb->query("CREATE TABLE version ( version_num INTEGER )");
		if ( $this->conndb->errorCode() != "00000" )
		{
			$err = $this->conndb->errorInfo();
			$this->error("The following error: {$err[2]} occurred creating the version table.");
			return false;
		}

		/**
		 * Insert the current version .
		 */
		$this->conndb->query("INSERT INTO version  (version_num) VALUES (220)");

		/**
		 * we now have a connections.db @ version 2.20 - we must now run the necessary upgrade scripts.
		 */

		return $this->dodbs_upgrade("220");


		//TODO: move this to dbdbs_upgrade -
		/* move this to dodbs_upgrade
		 if (!$this->silent)
		 {
			$this->print->add_to_output($this->template->databasesDone());
			$this->print->add_to_output($this->template->show_next(true,"final"));
			}

			return true;  // to indicate success
			*/

	} // end dodbs

	/**
	 * dodbs_upgrade - run the upgrade scripts ..
	 *
	 */

	function open_connectionsdb()
	{
		require("../conf/config.php");

		if ( ! isset($CONF['CONNDBDIR']) )
		{
			$this->error("Please check config.php param CONNDBDIR - it doesnt seem to be set.");
			return false;
		}

		$file="{$CONF['CONNDBDIR']}/connections.db";
		if ( ! is_dir($CONF['CONNDBDIR']) )
		{
			$this->error("Please check config.php param CONNDBDIR - it doesnt seem to be set to a directory.");
			return false;
		}

		unset($CONF);

		$this->conndb = new PDO("sqlite:{$file}");
	}

	function dodbs_upgrade($version)
	{
		$updfilelist = array();   // list of 'upgrade files' that need to be processed

		$path = "./upgrade/";

		$dir = @opendir($path);

		/* read the upgrade directory */
		while ( false !== ( $file = readdir($dir) ) )
		{
			/* lets see if this is a php file */
			$fname = preg_split("/\./",$file);
			if ( strtolower( $fname[1] ) == "php" )
			{
				/* do we need to add this file ? */
				if ( $fname[0] > $version)
				{
					/* add the file to our list */
					$updfilelist[] = $file;
				}
			}
		}
		closedir($dir);
		sort($updfilelist);
		foreach ($updfilelist as $pos => $name )
		{
			/* reset the sql array */
			$sql = array();
			/* load the upgrade file */
			include_once("upgrade/$name");
			/* process each $sql array element as a seperate statment */
			foreach ( $sql as $n => $qry )
			{
				if ( ! $this->silent )
				{
					$file = "upgrade/$name";
					// only print upgrade statements that have a name ..
					if ( ! is_numeric($n) )
					{
						$this->print->add_to_output("Processing: {$file} - {$n}<br/>");
					}
				}
				
				$stmt = $this->conndb->query($qry);
				if ( $this->conndb->errorCode() != "00000" )
				{
					$err = $this->conndb->errorInfo();
					$this->error("The following error: {$err[2]} occurred upgrading: {$file} - part {$n}.");
					return false;
				}

			}

			//update the version number in the connections.db ..
			$ver = preg_split("/\./",$name);
		
			$this->conndb->query("UPDATE version  SET version_num = {$ver[0]} ; ");
			if ( $this->conndb->errorCode() != "00000" )
			{
				$err = $this->conndb->errorInfo();
				$this->error("The following error: {$err[2]} occurred upgrading: {$file} - part {$n}.");
				return false;
			}

		} // end foreach of the list of files to process.

		if ( ! $this->silent )
		{
			$this->print->add_to_output($this->template->databasesDone());
			$this->print->add_to_output($this->template->show_next(true,"plugins"));
		}

		return true;  // to indicate success

	}
	
	function plugins()
	{
	    $this->print->setPageTitle("OAT Install - Plug-ins");
	    
	    // Check for PHP zip extension
	    $php_ext = get_loaded_extensions();
		if (!in_array("zip", $php_ext ) )
		{
			unset($php_ext);
			$HTML .= "<table width='100%'><tr>"
            	   . "<td class='tblheader'>Choose Plug-ins to Install</td>"
            	   . "</tr><tr><td>The php zip extension is required to install plug-ins.</td></table>";
            $this->print->add_to_output($HTML);
            $this->print->add_to_output($this->template->show_next(true,"final"));
            return;
		}
		unset($php_ext);
		
		// Check that tmp directory is writable
	    if ( is_writable( "../tmp" ) === false )
	    {
	        $HTML .= "<table width='100%'><tr>"
            	   . "<td class='tblheader'>Choose Plug-ins to Install</td>"
            	   . "</tr><tr><td>Cannot write to the OAT tmp directory. Please check permissions "
            	   . "and then use the Plug-in Manager to install plug-ins.</td></table>";
            $this->print->add_to_output($HTML);
            $this->print->add_to_output($this->template->show_next(true,"final"));
            return;
	    }

		// Display list of available plugins
	    $plugins = $this->getAvailablePlugins();
    	if (count($plugins) == 0)
        {
        	error_reporting(E_ALL ^ E_NOTICE);//suppressing wrong notice that PHP 5.3 displays
            $HTML .= "<table width='100%'><tr>"
            	   . "<td class='tblheader'>Choose Plug-ins to Install</td>"
            	   . "</tr><tr><td>There are no plug-ins to install.</td></table>";
            $this->print->add_to_output($HTML);
			error_reporting(E_ALL);
			
            $this->print->add_to_output($this->template->show_next(true,"final"));
            return;    		
	    }
		$this->print->add_to_output($this->template->showPlugins($plugins));
	}
	

	function installplugins()
	{
	    $this->print->setPageTitle("OAT Install - Plug-ins");
	    $this->print->add_to_output("<table width='100%'><tr><td class='tblheader'>Plug-in Install</td></tr>");
	    
	    // Determine which plugins should be installed
	    $plugins_to_install = array();
	    foreach ($_POST as $key => $value)
	    {
	        if (substr($key,0,7) == "plugin_")
	        {
	            if ($value == "on")
    	        {
    	            $plugins_to_install[] = $_POST['file_name_' . $key];
    	        }
	        } else if (substr($key,0,12) == "license_file")
	        {
	            // remove license files saved in temp
	            if (file_exists($_POST[$key]))
	            {
	                unlink($_POST[$key]);
	            }
	        }
	    }
	    // If no plugins to install, print message and return
	    if (count($plugins_to_install) == 0)
	    {
	        $this->print->add_to_output("<tr><td>No plug-ins selected</td></tr></table>");
	        $this->print->add_to_output($this->template->show_next(true,"final")); 
	        return;
	    }
	    
	    // Use the plugin manager to install each of the plugins selected
	    require_once("../admin/modules/pluginmgr.php");
	    $this->open_connectionsdb();
	    $pluginmgr = new pluginManager();
	    $pluginmgr->setConndb($this->conndb);
	    foreach ($plugins_to_install as $plugin_to_install)
	    {
	        $result_msg = $pluginmgr->doInstallPlugin("","",$plugin_to_install);
	        $this->print->add_to_output("<tr><td>" . $result_msg . "</td></tr>");
	    }
	    		    
	    $this->print->add_to_output("</table>");
	    $this->print->add_to_output($this->template->show_next(true,"final")); 
	}
	
	function getAvailablePlugins()
	{
	    $plugins = array();
		$dir = opendir( "../plugin_install" );
		while ( ( $file = readdir( $dir ) ) !== false )
		{
			// only interested in zip files.
			$plugin = explode( '.',$file );
			if ( strtolower( $plugin[count($plugin) -1] ) == "zip" )
			{
				$zip = new ZipArchive;
				if ($zip->open("../plugin_install/{$file}") !== TRUE) {
					continue;
				}

				if ( $zip->locateName("plugin.xml") !== FALSE )
				{
					$zip->extractTo("../tmp/","plugin.xml");
					$pluginData = file_get_contents("../tmp/plugin.xml");
					$plugin = $this->pluginInfoFromXML($pluginData,$file);
					if ($plugin->plugin_license)
					{
					    $zip->extractTo("../tmp/",$plugin->plugin_license);
					}
					$plugins[] = $plugin;
					unlink("../tmp/plugin.xml");
				}
				$zip->close();
			}
		}
		closedir($dir);
		
		return $plugins;
	}
	
	/**
	 * get the plugin info from the plugin plugin.xml file
	 *
	 * @param  $xml
	 * @return plugin object.
	 */
	function pluginInfoFromXML(&$xml,$file)
	{
		$sxml = new SimpleXMLElement($xml,NULL,false);
		require_once("../admin/lib/plugin.php");

		$plugin = new plugin($this->idsadmin);
		$pluginInfo = $sxml->plugin_info;
		$plugin->init(0,$file,$pluginInfo->plugin_name,$pluginInfo->plugin_desc
		,$pluginInfo->plugin_author,$pluginInfo->plugin_version
		,$pluginInfo->plugin_server_version , $pluginInfo->plugin_upgrade_url
		,false,$pluginInfo->plugin_enabled , "" , $pluginInfo->plugin_license
		);

		return $plugin;
	}
	
	function fin()
	{
		$this->print->setPageTitle("Install OAT - DONE");
		$this->print->add_to_output($this->template->fin());
		$fd = fopen("install.lock","w");
		fclose($fd);
	} // end fin

	function load_template($template)
	{
		require_once("templates/{$template}.php");
		$this->template = new $template($this->print);
	} // end load_template

	/* Silent Install */
	function doSilentInstall()
	{
		// For silent install, the user has already accepted the license
		// agreement, as indicated by the -accept_license command line
		// argument.  Therefore, we will automatically create the '99' file
		// that represent license agreement acceptance.
		fopen("99","w");

		// check modules
		$this->check();

		// in silent mode, if conf/config.php already exist, use and settings in the existing config.php
		// else create default config.php file
		$homedir = str_replace( '\\', '/', getcwd() ) . '/';
		$homedir = substr($homedir, 0, -8);
		$confdir = $homedir.'/conf/';
		if (!file_exists($confdir."/config.php"))
		{
			$this->saveDefaultConfig();
		} else {
			$this->checkConfigLang($confdir."/config.php");
		} 
		
		// in silent, if connections.db does not exist, run do dbs() to create it.
		// else, use existing connections.db
		require("../conf/config.php");
                if(!is_dir($CONF['CONNDBDIR']))
                {
                        $chk = exec('mkdir "'.$CONF['CONNDBDIR'].'"');
                        if($chk!=0)
                        {
                                $this->error("Failed to create {$CONF['CONNDBDIR']}. Make sure location is writable.",true);
                        }
                }
		
		$file="{$CONF['CONNDBDIR']}/connections.db";

		if(!file_exists($file)){
			// create connections database
			if (! ($this->dodbs()) )
			{
				// in silent install, failure in creation of connections database is a fatal error
				$this->error("Creation / Upgrade of connections database failed.", true);
			}			
		}else{
			$chk = $this->open_connectionsdb();
			if($chk === false){
				$this->error("Unable to preserve existing connections.db ({$file}). Delete existing connections.db and run the OAT automated installer again",true);
			}
			
			$stmt = $this->conndb->query("SELECT version_num FROM version;");
			if ( $this->conndb->errorCode() != "00000" )
			{
				$err = $this->conndb->errorInfo();
				$this->error("Error Occurred: {$err[2]}. Unable to preserve existing connections.db ({$file}). Delete existing connections.db and run the OAT automated installer again",true);
			}
			
			$res = $stmt->fetch();
			$chk = $this->dodbs_upgrade($res["version_num"]);
			if($chk === false){
				$this->error("Unable to preserve existing connections.db ({$file}). Delete existing connections.db and run the OAT automated installer again",true);
			}
		}
		
		// If we got to this point in the silent install without error,
		// the install succeeded.  Write "install.lock" file.
		fopen("install.lock","w");
		echo "\nOpenAdmin Tool for IDS installed successfully!\n";
	}// end doSilentInstall

} // end class


/**
 * setup and run the install
 */
$install = new install();

if ( version_compare(PHP_VERSION, "5.2.4", "<") )
{
	$install->error("Minimum PHP Version Required is 5.2.4 version found: ".PHP_VERSION , true);
}


/**
 * Check if running the install in silent mode off the command line.
 * If so, it will have been called with three arguments:
 * index.php -accept_license -baseurl="<base URL for OpenAdmin Tool>"
 *            -conndbdir="<the directory for connections database>";
 * If these three arguments do not exist, default to web install mode.
 */
if ( @$_SERVER['argc'] == 4
&& strcasecmp(@$_SERVER['argv'][1], "-accept_license") == 0
&& strcasecmp(substr(@$_SERVER['argv'][2],0,9), "-baseurl=") == 0
&& strcasecmp(substr(@$_SERVER['argv'][3],0,11), "-conndbdir=") == 0 )
{
	echo "Starting silent install of OpenAdmin Tool for IDS...\n";
	$baseurl = substr(@$_SERVER['argv'][2],9);
	$conndbdir = addslashes(substr(@$_SERVER['argv'][3],11));
	$install->setSilentMode($baseurl,$conndbdir);
}

// start installation
$install->run();
?>
