<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007, 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

class install_template {

	private $print;

	function __construct(&$print)
	{
		$this->print = &$print;
	}

	function start()
	{
		$HTML = "";
		$HTML .= <<< EOF
<form method="post" action="index.php?do=check">
<table width='100%'>
	<tr>
		<td>
			<p>Welcome to Open Admin Tool for IDS install. </p><br/>
	</td>
	</tr>
	<tr>
		<td>
			<p><a href='../LICENSE'>LICENSE AGREEMENT</a></p><br/>
			<input type='checkbox' name='Lic_CB'> I accept the terms in the license agreement. </>
		</td>
	</tr>
	<tr>
		<td colspan="3" class="formsubtitle" align="right">
			<input name="install_start" class="button" type="submit" value="Next"/>
		</td>
	</tr>
</table>
</form>
EOF;
		return $HTML;
	} // end start.


	function show_required($required)
	{
		$HTML = "";
		 
		$HTML .= "<table width='100%'>";

		$HTML .= <<< EOF
		<tr>
			<td class='tblheader' colspan="2" align="center">Required PHP Modules</td>
		</tr>
		<tr>
			<td colspan="2" class="formsubtitle">
				<p>The following PHP modules are required to run OAT.</p> 
                <p>A <img src='../images/cross.png' border='0' alt='X' /> indicates the module could not be found. Please check your php installation.</p>
			</td>
		</tr>
EOF;
		foreach ($required as $k => $v)
		{
			$img = "<img src='../images/check.png' border='0' alt='OK' />";
			$HTML .= "<tr><td class='formleft'>{$k}</td>";

			if ($v == 0)
			{
				$show_next = false;
				$img = "<img src='../images/cross.png' border='0' alt='X' />";
			}
			$HTML .= "<td class='formright'>{$img}</td></tr>";
		}
		$HTML .= "<tr><td>&nbsp;</td></tr></table>";
		return $HTML;
	} // end show_required


	function show_recommended($recommended)
	{
		$HTML = "";

		$HTML .= "<table width='100%'>";

		$HTML .= <<< EOF
                <tr>
                        <td class='tblheader' colspan="2" align="center">Recommended PHP Modules</td>
                </tr>
                <tr>
                        <td colspan="2" class="formsubtitle">
                                <p>The following PHP modules are recommended to run OAT.</p>
                                <p>* While these modules are not required to complete the installation or start 
                                   using the tool, you will not be able to utilize OpenAdmin Tool's full 
                                   functionality without these modules.</p>
                        </td>
                </tr>
EOF;
		foreach ($recommended as $k => $v)
		{
			$img = "<img src='../images/check.png' border='0' alt='OK' />";
			$HTML .= "<tr><td class='formleft'>{$k}</td>";

			if ($v == 0)
			{
				$img = "<img src='../images/cross.png' border='0' alt='X' />";
			}
			$HTML .= "<td class='formright'>{$img}</td></tr>";
		}
		$HTML .= "</table>";
		return $HTML;
	} // end show_recommended

	function config_start()
	{
		$HTML="";
		$HTML .= <<< EOF
		<form method="post" action="index.php?do=saveconfig">
		<table width="100%">
			<tr>
				<td colspan="3" class="tblheader">Configuration Params</td>
			</tr>
EOF;
		return $HTML;
	}

	function config_row($what)
	{
		$HTML = "";
		foreach ($what as $k => $v)
		{
			$HTML.=<<<EOF
			<tr>
			<td class="formleft">{$k}</td>
			<td class="formright">{$v['desc']}</td>
EOF;
			
			if ($k == "SECURESQL")
			{
				$SECURESQL_SELECTED = (strcasecmp($v['default'],"on")==0) ? "CHECKED":"";
				$HTML .= <<<EOF
				<td class="formright"><input type=checkbox name="{$k}" {$SECURESQL_SELECTED}/></td>
EOF;
			} 
			else if ($k == "LANG")
			{
			    $HTML .= <<<EOF
				<td class="formright"><select name="{$k}">{$this->option_lang($v['default'])}</select></td>
EOF;
			}
			else
			{
				$HTML .= <<<EOF
				<td class="formright"><input name="{$k}" value="{$v['default']}"/></td>
EOF;
			}
			$HTML .= <<<EOF
			</tr>
EOF;
		}
		return $HTML;
	}

	function config_warning($warned_dir)
	{
		$HTML ="";
		$HTML .= "<input type='hidden' name='warned_dir' value='{$warned_dir}'/>";
		return $HTML;
	}

	function config_end()
	{
		$HTML = "";

		$HTML .= <<< EOF
			<tr>
				<td colspan="3" class="formsubtitle" align="center">
					<input type="submit" class="button" value="Save"/>
				</td>
			</tr>
		</table>
		</form>
EOF;
		return $HTML;
	}
	
    /**
     * Determines all languages supported by OAT and creates the drop-down options 
     * to include each language.
     * 
     * @param $defLang default language that should be selected
     * @return $HTML options for drop-down control
    */
    function option_lang($defLang="en_US")
    {
        // Load the English lang_language file for all language names
        $lang = array();
        $fname = "../lang/en_US/lang_language.xml";
        if ( file_exists($fname) )
        {
            $xml = simplexml_load_file( $fname );
        }
        if (! is_null($xml))
        {
            foreach ( $xml as $k )
            {
                $name = (string)$k->getName();
                $lang[$name]=(string)$xml->$name;
            }
        } else {
            // If xml is null (or we couldn't find the file), just add English
            // (This is sanity check.  We should never get here.)
            $lang['en_US'] = "English";
        }

        // Find all of the languages OAT supports
        $HTML = "";
        $data = array();
        $dir = opendir("../lang/");
        while (false !== ($file = readdir($dir)))
        {
            if (substr($file,0,1) == "." || $file == ".." )
            {
                continue;
            }
            if (file_exists("../lang/".$file."/language.php"))
            {
                $data[$file]['lang']=$lang[$file];   
            }
        }
        asort($data);  // Sort languages alphabetically

        // Create options for the drop-down of each language OAT supports
        foreach ( $data as $k => $v )
        {
            if ( $k == $defLang)
            {
                $HTML .= "<option value='{$k}' SELECTED>{$v['lang']}</option>";
            }
            else
            {
                $HTML .= "<option value='{$k}'>{$v['lang']}</option>";
            }
        }
        return $HTML;
    }

	function show_next($show_next,$what)
	{
		$HTML = "";
		if ( $show_next == true )
		{
			$HTML .= <<< EOF
			<form method="post" action="index.php?do={$what}">
            <table width="100%">
                <tr>
                   <td>&nbsp;</td>
                </tr>
                <tr>
                   <td class="formsubtitle" align="right">
                   <input name="next" class="button" type="submit" value="Next"/>
                   </td>
                </tr>
            </table>
            </form>
EOF;
		}
		return $HTML;
	} // end show_next

	function createDatabases()
	{
		$HTML = "";
		$HTML .= <<< EOF
		<table width="100%">
			<tr>
				<td class="tblheader">
					Create connections database.
				</td>
			</tr>
			<tr>
				<td>
					<p>Click next below to create the 'connections' database.
					</p>
					<p>
						<b>NOTE:</b> This will remove any existing connection info defined.
					</p>
				</td>
			</tr>
		</table>
EOF;
		return $HTML;
	} // end createDatabases

	
	function updateDatabase()
	{
		$HTML = "";
		$HTML .= <<< EOF
		<table width="100%">
			<tr>
				<td class="tblheader">
					Upgrade connections database.
				</td>
			</tr>
			<tr>
				<td>
					<p>Click next below to upgrade the 'connections' database.
					</p>
				</td>
			</tr>
		</table>
EOF;
		return $HTML;
	} // end updateDatabase

	function databasesDone()
	{
		$HTML = "";
		$HTML .= <<< EOF
		<table width="100%">
			<tr>
				<td align="center">Modifications to database <i>connections</i> complete.</td>
			</tr>
		</table>
EOF;
		return $HTML;
	} // end databasesDone
	
	
	function showPlugins($plugins)
	{
	    $HTML = "";
	    $HTML .= <<< EOF
		<script type="text/javascript">
			function plugin_click(plugin_name, checkbox)
			{
				// This function only applies if the checkbox was checked
				// (not if it was unchecked).
				if (checkbox.checked == false)
				{
					return;
				}
				
				c = confirm("Do you accept the license aggreement for the " + plugin_name + " plug-in?");
				if (c)
				{
					checkbox.checked = true;
				} else {
					checkbox.checked = false;
				}
			}
		</script>
		<form method="post" action="index.php?do=installplugins">
		<table width="100%">
			<tr>
				<td class="tblheader">Choose Plug-ins to Install</td>
			</tr>
		</table>
		<table width="100%" border="0" cellpadding="5">
			<tr class="formsubtitle">
				<td>Name</td>
				<td>Description</td>
				<td>Author</td>
				<td>Version</td>
				<td>Server Version</td>
				<td>License</td>
				<td>Install?</td>
			</tr>		
EOF;

		$index = 0;
		foreach ($plugins as $p)
		{
		    $HTML .= "<tr><td>" . $p->plugin_name . "</td>" 
		          .  "<td>" . $p->plugin_desc . "</td>"
		          .  "<td>" . $p->plugin_author . "</td>"
		          .  "<td>" . $p->plugin_version . "</td>"
		          .  "<td>" . $p->plugin_server_version . "</td>"
		          .  "<td> <a href='../tmp/{$p->plugin_license}'>License Agreement</a> </td>"
		          .  "<td><input name='plugin_{$index}' type='checkbox' onclick='plugin_click(\"{$p->plugin_name}\", this)'/></td></tr>"
		          .  "<input type='hidden' name='file_name_plugin_{$index}' value='{$p->plugin_file_name}'/>"
		          .  "<input type='hidden' name='license_file_plugin_{$index}' value='../tmp/{$p->plugin_license}'/>";
		    $index++;
		}
		
		$HTML .= <<< EOF
        </table>
        <table width="100%">
                <tr>
                   <td>&nbsp;</td>
                </tr>
                <tr>
                   <td class="formsubtitle" align="right">
                   <input name="next" class="button" type="submit" value="Next"/>
                   </td>
                </tr>
            </table>
            </form>
EOF;
		return $HTML;
	}
	
	
	function fin()
	{
		$HTML = "";
		$HTML .= <<< EOF
		<table width="100%">
			<tr>
				<td align="center">Open Admin Tool for IDS install complete.</td>
			</tr>
			<tr>
				<td align="center">Click <a href="../index.php" name="OAT">HERE</a> to goto Open Admin Tool for IDS</td>
			</td>
		</table>
EOF;
		return $HTML;
	} // end fin

	function error($err_string="")
	{
		$HTML = "";
		$HTML .= <<<EOF
		<fieldset>
		<legend>An Error occurred</legend>
		<div class="borderwrapred">
		<table width="100%" style="background: #E3C0C0;" >
		<tr>
		<td>{$err_string}</td>
  </tr>
</table>
</div>
</fieldset>
EOF;
		return $HTML;
	}

	function warning($warning_string="")
	{
		$HTML = "";
		$HTML .= <<<EOF
		<fieldset>
		<legend>Warning</legend>
		<div class="borderwrapred">
		<table width="100%" style="background: #E3C0C0;" >
		<tr>
		<td>{$warning_string}</td>
  </tr>
</table>
</div>
</fieldset>
EOF;
		return $HTML;
	}

} // end class
?>
