<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007, 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */



class Charts {

    public  $idsadmin;
    private $select;
    private $dbname;
    private $type;
    private $legendDir;
    private $title;
    private $dataTitles;
    private $width="100%";
    private $height="100%";
    private $data = array();
    private $id = "gengraph";

    function __construct(&$idsadmin)
    {
        $this->idsadmin = $idsadmin;
        $this->setId("genGraph");
    }

    function setDbname($dbname="")
    {
        $this->dbname=$dbname;
    }

    /*
     * set the select statement
     */
    function setSelect($sel="")
    {
        $this->select = $sel;
    } // end setSelect

    /*
     * Set the type of Graph we want..
     * Valid values are: PIE / LINE
     */
    function setType($type="")
    {
        $this->type=$type;
    } // end setType

    /*
     * Set the direction of legend.
     * Valid values are: horizontal / vertical
     */
    function setLegendDir($dir="horizontal")
    {
        $this->legendDir = $dir ;
    } // end setLengendDir

    function setTitle($title="")
    {
        $this->title=$title;
    } // end setTitle

    function setDataTitles($title=array())
    {
        $this->dataTitles=$title;
    } // end setDataTitles

    function setWidth($width="100%")
    {
        $this->width=$width;
    } // end setWidth

    function setHeight($height="100%")
    {
        $this->height = $height;
    } // end setHeight

    function setId($id="genGraph")
    {
        $this->id = $id;
    }
    /*
     * set our data array
     */
    function setData($arr=array())
    {
        $this->data = $arr;
    } // end setData



    /*
     * return the database name to run the select against.
     */
    function getDbname()
    {
        return $this->dbname;
    } // end getDbname

    /*
     * return the select statement
     */
    function getSelect()
    {
        return $this->select;
    } // end getSelect

    /*
     * Get the type of graph we want.
     * @return string
     */
    function getType()
    {
        return $this->type;
    } // end getType

    /*
     * Get legend direction.
     * @return String
     */
    function getLegendDir()
    {
        return $this->legendDir;
    } // end getLegendDir

    /*
     * return the title
     */
    function getTitle()
    {
        return $this->title;
    }

    /*
     * return the width
     */
    function getWidth()
    {
        if ( !$this->width )
        {
            $this->setWidth("100%");
        }

        return $this->width;
    } // end getWidth

    /*
     * return the height
     */
    function getHeight()
    {
        if ( ! $this->height )
        {
            $this->setHeight("100%");
        }
         
        return $this->height;
    } // end getHeight

    function getId()
    {
        return $this->id;
    }
    /**
     * get the data titles
     */

    function getDataTitles()
    {
        $ret = "";

        if ( is_array($this->dataTitles) )
        {
            foreach ( $this->dataTitles as $k => $v)
            {
                $ret .= "title_{$k}={$v};";
            }
        }
        return $ret;
    }

    /*
     * get our data array
     */
    function getData()
    {
        $ret = "";

        foreach ( $this->data as $k => $v)
        {
            $ret .= "{$k}={$v};";
        }

        return $ret;
    } // end getData

    /*
     * return the SWF file needed to render the Chart.
     */
    function getSWF()
    {
        if ( $this->type == "PIE")
        {
            return "genPieGraph";
        }

        return "genGraph";
    } // end getSWF

    /*
     * get the FlashVars line. ie. the options we are passing to the SWF file.
     */
    function getFlashVars()
    {
        $flashvars = "select={$this->getSelect()}&dbname={$this->getDbname()}&graphTitle={$this->getTitle()}&legendDirection={$this->getLegendDir()}&dataTitles={$this->getDataTitles()}&graphData={$this->getData()}";
        $lang = $this->idsadmin->phpsession->get_lang();
        if ($lang != "en_US")
        {
             $flashvars .= "&localeChain={$lang},en_US";
             if ($this->type == "PIE")
             {
                 $flashvars .= "&resourceModuleURLs=swfs/Charts/genPieGraph_{$lang}.swf";
             } else {
                 $flashvars .= "&resourceModuleURLs=swfs/Charts/genGraph_{$lang}.swf";
             }
        }
        return $flashvars;
    } // end getFlashVars

    /*
     * return the Params. ie. the options we are passing to the SWF file for IE.
     */
    function getParams()
    {
        return <<< EOF
        <param name="select" value="{$this->getSelect()}"/>
        <param name="dbname" value="{$this->getDbname()}"/>
        <param name="graphTitle" value="{$this->getTitle()}"/>
        <param name="legendDirection" value="{$this->getLegendDir()}"/>
        <param name="graphData" value="{$this->getData()}"/>
EOF;
    } // end getParams

    function Render()
    {
        $this->idsadmin->load_template("template_Charts");
        $this->idsadmin->html->add_to_output($this->idsadmin->template['template_Charts']->renderChart($id));
    }

    function __destruct()
    {

    }
}
?>
