<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007, 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

class database extends PDO {
	 
	public $idsadmin = "";
	private $fetch_mode = PDO::FETCH_ASSOC;

	/******************************************
	 * Constructor:
	 *******************************************/
	function __construct(&$idsadmin,$dbname="sysmaster",$locale="",$username="",$password="")
	{
		$this->idsadmin=&$idsadmin;
	  $this->idsadmin->load_lang("database");
		$dsn = "";
		putenv("INFORMIXCONTIME={$this->idsadmin->get_config("INFORMIXCONTIME",20)}");
		putenv("INFORMIXCONRETRY={$this->idsadmin->get_config("INFORMIXCONRETRY",3)}");
		
		$dsn .= "informix:host={$this->idsadmin->phpsession->instance->get_host()}";
		$dsn .= ";service={$this->idsadmin->phpsession->instance->get_port()}";
		$dsn .= ";database={$dbname}";
		$dsn .= ";protocol={$this->idsadmin->phpsession->instance->get_idsprotocol()}";
		$dsn .= ";server={$this->idsadmin->phpsession->instance->get_servername()}";
		 
		$informixdir = $this->idsadmin->get_config("INFORMIXDIR");
		if ( substr(PHP_OS,0,3) != "WIN" )
		{
			$libsuffix = (strtoupper(substr(PHP_OS,0,3)) == "DAR")? "dylib":"so";
			$dsn .= ";TRANSLATIONDLL={$informixdir}/lib/esql/igo4a304.".$libsuffix;
			$dsn .= ";Driver={$informixdir}/lib/cli/libifdmr.".$libsuffix.";";
		}

		$envvars = $this->idsadmin->phpsession->instance->get_envvars();
		$conn_num = $this->idsadmin->phpsession->instance->get_conn_num();
		if (is_null($envvars) && $conn_num != "")
		{
			// If $envvars is null, this is the first connection after login,
			// so we need to get the env vars from the connections.db
			require_once ROOT_PATH."/lib/connections.php";
			$conndb = new connections($this->idsadmin);

			$sql  = "select envvar_name, envvar_value from conn_envvars where conn_num = $conn_num ";
			$stmt = $conndb->db->query($sql);

			$envvars = "";
			while ($row = $stmt->fetch())
			{
				$envvars .= "{$row['ENVVAR_NAME']}={$row['ENVVAR_VALUE']};";
				if (strcasecmp($row['ENVVAR_NAME'],"DELIMIDENT") == 0)
				{
					$this->idsadmin->phpsession->set_delimident($row['ENVVAR_VALUE']);
				}
			}

			// Store envvars in the session, so we don't have to requery each time
			$this->idsadmin->phpsession->set_envvars($envvars);
		}

		if (!is_null($envvars) && $envvars != "" )
		{
			// add envvars to connection string
			$dsn .= ";$envvars";
		}

		if ( $locale != "" )
		{
			$dsn .= ";CLIENT_LOCALE={$locale};DB_LOCALE={$locale};";
		}

		if ($username!=""&&$password!="")
		{
			$user	= $username;
			$passwd	= $password;
		}else
		{
			$user   = $this->idsadmin->phpsession->instance->get_username();
			$passwd = $this->idsadmin->phpsession->instance->get_passwd();	
		}

		try {
			parent::__construct($dsn,$user,utf8_decode($passwd) );
		} catch(PDOException $e) {
			$idsadmin->load_lang("database");
			if ( $idsadmin->render === false )
			{
        error_log("{$idsadmin->lang('DatabaseConnFailed')} {$idsadmin->lang('ConnFailed')}:<br/>DSN:{$dsn}<br/>{$e->getMessage()}");
        trigger_error("{$idsadmin->lang('DatabaseConnFailed')} - {$idsadmin->lang('QueryF')}:DSN:{$dsn}{$e->getMessage()}",E_USER_ERROR);
			}

			error_log("{$idsadmin->lang('ConnFailed')}:<br/>DSN:{$dsn}<br/>{$e->getMessage()}");
			$idsadmin->db_error("{$e->getMessage()}<br/><br/>DSN:{$dsn}", "{$idsadmin->lang('ConnFailed')}.");
			$idsadmin->html->render();
			die();
		}

		$this->idsadmin->phpsession->set_connected(true);

	}

	function query($sql="",$params=false,$throw_exceptions_only=false)
	{
		if ($sql=="")
		{
			if ( ! $this->idsadmin->render )
			{
                                error_log("Error: no sql to run. ");

                                trigger_error("Error: no sql to run. ");
			}
			$this->idsadmin->error("{$this->idsadmin->lang('NoSQLToExecute')}");
			return null;
		}

		if ( $params === true )
		{
			$this->setAttribute(PDO::ATTR_STATEMENT_CLASS,array("xmlOATStatement"));
		}
	    else
	    {
			$this->setAttribute(PDO::ATTR_STATEMENT_CLASS,array("PDOStatement"));
	    }
		$stmt = parent::query($sql);
		
		if($throw_exceptions_only && ($this->errorCode() != 0 || ($stmt==false) )){
			$errstr=$this->errorInfo();
			throw new PDOException( $errstr[2],$errstr[1]);
			die();
		}

		if ($this->errorCode() != 0 || ($stmt==false) )
		{
			$errstr=$this->errorInfo();
			error_log("{$this->idsadmin->lang('QueryF')}<br/> {$sql} <br/> {$this->idsadmin->lang('ErrorF')} {$errstr[1]} <br/> {$errstr[2]} <br/>");
			if ( ! $this->idsadmin->render )
			{
                                error_log("{$this->idsadmin->lang('DatabaseQueryFailed')} {$this->idsadmin->lang('QueryF')} {$sql} {$this->idsadmin->lang('ErrorF')} {$errstr[1]} {$errstr[2]} ");

                                trigger_error("{$this->idsadmin->lang('DatabaseQueryFailed')} - {$this->idsadmin->lang('QueryF')} {$sql} {$this->idsadmin->lang('ErrorF')} {$errstr[1]} {$errstr[2]} ",E_USER_ERROR);
 
			}
			$this->idsadmin->db_error("{$this->idsadmin->lang('ErrorF')} {$errstr[1]} <br/> {$errstr[2]} <br/><br/>{$this->idsadmin->lang('QueryF')}<br/> {$sql} ");
			$this->idsadmin->html->render();
			die();
		}

		if ($stmt)
		{
			$stmt->setFetchMode($this->fetch_mode);
		}

		return $stmt;
	} // end query

	/**********************************
	 * setFetchMode
	 * set the default setFetchMode
	 **********************************/
	function setFetchMode($fetchmode)
	{
		$this->fetch_mode = $fetchmode;
	} // end setFetchMode
	 
	/******************************************
	 * Destruct:
	 *******************************************/
	function __destruct()
	{
		 
	} //end __destruct
	 
} // end database

class xmlOATStatement extends PDOStatement
{
	function fetch( $fetch_style = PDO::FETCH_ASSOC,
	                $cursor_orientation = PDO::FETCH_ORI_NEXT,
	                $cursor_offset = 0 )
	{
		 
		$row = parent::fetch($fetch_style, $cursor_orientation, $cursor_offset);

		if (is_array($row))
		return $this->row_to_xml($row);
		 
		return "";
	}
	function row_to_xml($array, $level=1) {
		$xml = '<row>';
		$cnt=0;
		foreach ($array as $key=>$value) 
		{
			$colInfo = $this->getColumnMeta($cnt++);
			$xml .= "\t<{$key}";
			foreach ($colInfo as $t => $v)
			{
				if ( $t == "native_type")
				{
					if ( stripos($v,"CHAR") === false )
					{
						$value = $value;
					}
					else
					{
						$value = trim($value);
						$value = "<![CDATA[{$value}]]>";
					}
				}
			}
			$xml .= ">{$value}</{$key}>";
		}
		
		$xml .= "</row>\n";
		return $xml;
	}

} //end xmOATlStatement

?>
