<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007, 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

/*********************************************************************
 * OAT V2
 *
 * Main OAT class
 **********************************************************************/

class IDSAdmin {

	public   $html;                      /* the output class   */
	public   $template=array();          /* Array of templates */
	private  $version="2.27";     /* the version string */
	private  $buildtime="12/16/09 06:31:18"; /* the build time     */
	private  $config=array();            /* CONFIG info from conf/config.php */
	public   $phpsession;                /* the phpsession class */
	private  $language = array();        /* the language array */
	public   $in = array();              /* the values from post or get */
	private  $database = array();        /* array of database we have connections too */
	private  $currdb;
	private  $last_lang_file;            /* the last lang file that was loaded */
	private  $isreadonly=false;          /* is the group we are connected too readonly */
	public   $render = true;             /* whether to render our output or not */
	public   $serverInfo;                /* Information about the server we are connected too */
	public   $iserror;                   /* has a fatal error occurred */
	public   $crumb  = array();          /* bread crumb array */
	private  $redirect = array();        /* where to redirect too if the server is switched */
	private  $currMenuItem = "";         /* current menu item */
	private  $is_Plugin = 0;     /* Is the current URL a plugin */
	private  $url_fullpath = "";         /* Full path to module name */

	/******************************************
	 * Constructor:
	 *******************************************/
	function __construct($is_webservice=false)
	{

		/* load the 'global template' */
		$this->load_template("template_global");

		/* load the 'html' class */
		require_once(ROOT_PATH."lib/output.php");
		$this->html = new output();
		
		$this->html->idsadmin =& $this;

		/* load the configuration params */

		if ( sizeof($this->config) == 0 && file_exists(ROOT_PATH."/conf/config.php"))
		{
			$CONF=array();
			require(ROOT_PATH."/conf/config.php");
			$this->config = $CONF;
			unset($CONF);
		}

		if ( ! $is_webservice )
		{
			$this->render=true;
		}
		else
		{
			$this->render=false;
			// IN_ADMIN gets set in index.php, so if we are running as a webservice
			// that does not go through index.php, we need to set it here
			@define('IN_ADMIN',false); 
		}

		/* load the 'phpsession' class */
		require_once(ROOT_PATH."/lib/phpsession.php");
		$this->phpsession = new phpsession($this);

		/* If the language is set in the session, use that.
		 Otherwise, set the default lang */
		$lang = (isset($_SESSION['lang']))? $_SESSION['lang']:$this->get_config("LANG","en_US");
		$this->phpsession->set_lang($lang);

		/* Make sure config.php exists.  If not, the user needs to run the install */
		if ( ! file_exists(ROOT_PATH."/conf/config.php"))
		{
			$msg = "Cannot find config.php. Have you run the install ? <br/>" .
                   "<a href='" . ROOT_PATH . "install/index.php'>Start the install</a>";
			$this->fatal_error($msg,false);
			return;
		}

		/* check our version */
		if ( $this->checkForUpgrade() == 1)
		{
			$msg = "OAT has been upgraded. You need to re-run the install to upgrade.<br/>" .
                   "<a href='" . ROOT_PATH . "install/index.php'>Start the upgrade</a>";
			$this->fatal_error($msg,false);
			return;
		}

		/* Sanity check PHP modules */
		$this->check_modules();

		/* Setup the INFORMIXDIR as part of the config */
		$this->set_INFORMIXDIR();

		/* Set up the in array */
		$this->set_in();

		/* set readonly */
		$this->set_isreadonly( $this->isGroupReadOnly( $this->phpsession->group ) );

	} #end __construct

	function set_fullpath( $name )
	{
		$this->url_fullpath = $name;
			
		if ( strpos($actname,"/") === false)
		{
			// $this->set_isPlugin(0);
		}
		else
		{
			// $this->set_isPlugin(1);
		}
	}

	/* Get the full pathname to the module code */
	function get_fullpath()
	{
		return $this->url_fullpath;
	}

	/* Are we using an Plugin */
	function isPlugin()
	{
		//print("returning extension: {$this->is_Plugin}<br/>");
		return $this->is_Plugin;
	}

	function set_isPlugin($val=0)
	{
		$this->is_Plugin = $val;
	}

	/* Get the name of the class */
	function get_classname()
	{
		$cname = substr(strrchr($this->url_fullpath, "/"), 1);

		return substr($cname,0,-4);   // remove the .php extention
	}

	function get_classpath()
	{
		return substr($this->url_fullpath,0,strrpos($this->url_fullpath,"/"));
	}

	/**
	 * set our redirect array.
	 */
	function set_redirect($rdo,$ract="")
	{
		$this->redirect['rdo'] = $rdo;
		$this->redirect['ract'] = $ract;
	}

	function get_redirect()
	{
		return "ract={$this->redirect['ract']}&amp;rdo={$this->redirect['rdo']}";
	}

	/**
	 * Set the readonly flag.
	 *
	 * @param boolean $_isreadonly
	 */
	function set_isreadonly($_isreadonly=false)
	{
		$this->isreadonly = $_isreadonly;
	}
	/**
	 * return the value of isreadonly
	 */
	function isreadonly()
	{
		return $this->isreadonly;
	}

	/**
	 * check if OAT has been upgraded. ie. the version in connections.db doesnt
	 * match OAT version.
	 *
	 * returns true if upgrade is needed.
	 */
	function checkForUpgrade()
	{
		$version = $this->get_version();
		$version = preg_replace("/[^0-9B]/","",$version);
		$split_ver = explode("B",$version);
		$maj_version = $split_ver[0];
		//$beta_version = $split_ver[1];

		require_once(ROOT_PATH."lib/connections.php");
		$conndb = new connections($this);
			
		$qry = "SELECT * FROM VERSION";
		$stmt = $conndb->db->query($qry);
		$err = $conndb->db->errorInfo();
		if ( $conndb->db->errorCode() != "00000" )
		{
			return 1;
		}
		$row = $stmt->fetch();
		$dbVersion = $row['VERSION_NUM'];
		$stmt->closeCursor();

		if ( $dbVersion < $maj_version )
		{
			return 1;
		}

		return 0;
	}

	/**
	 * Get the readonly value from the groups
	 */
	function isGroupReadOnly($group)
	{
		if ( !isset($group) || $group == "")
		{
			return false;
		}

		require_once(ROOT_PATH."lib/connections.php");
		$conndb = new connections($this);

		$qry = "select * from groups where group_num = {$group} ";
		$stmt = $conndb->db->query($qry);
		$row = $stmt->fetch();
		return $row['READONLY'];
	}

	/*****************************************************************
	 * set_in:
	 * Setup the in array and clean the passed in params from GET/POST
	 ******************************************************************/
	function set_in()
	{
		$return = array();          /* temp variable used to hold the values from the GET and POST */

		/*
		 * Default the 'act' and 'do' params to ""
		 */
		$return['do'] = "";
		$return['act'] = "";

		if( !empty($_GET) )
		{
			foreach( $_GET as $i=>$v )
			{
				if( is_array($_GET[$i]) )
				{
					foreach( $_GET[$i] as $i2=>$v2 )
					{
						$return[$i][$i2] = $this->clean_data($v2);
					}
				}
				else
				{
					$return[$i] = $this->clean_data($v);
				}
			}
		}

		if( !empty($_POST) )
		{

			foreach( $_POST as $i=>$v )
			{
				if( is_array($_POST[$i]) )
				{
					foreach( $_POST[$i] as $i2=>$v2 )
					{
						$return[$i][$i2] = $this->clean_data($v2);
					}
				}
				else
				{
					$return[$i] = $this->clean_data($v);
				}
			}
		}

		$this->in=$return;
	}  //end set_in

	/*************************************************************************
	 * clean_data:
	 * trim and remove slashes from data. used when processing browser 'input'
	 **************************************************************************/
	function clean_data($data)
	{
		if ($data == "")
		{
			return "";
		}

		$data = trim($data);
		if ( get_magic_quotes_gpc() )
		{
			$data = stripslashes($data);
		}
		return $data;
	} // end clean_data

	/*************************************
	 * get_config:
	 * Get a config parameter
	 **************************************/
	function get_config($what,$default=NULL)
	{
		if ($what == "*")
		{
			return $this->config;
		}
			
		return isset($this->config[$what]) ? $this->config[$what] : $default;
	} // end get_config

	/**	
	 * switch_user:
	 * Switch to another user.
	 * Unset the database private variable.
	 * Changes username and password in this idsadmin's phpsession->instance.
	 */
	function switch_user($who,$pwd)
	{
		$_SESSION['SQLTOOLBOX_USERNAME'] = $who;
		$_SESSION['SQLTOOLBOX_PASSWORD'] = $pwd;
	}

	/**
	 * get a connection to a database within this server...
	 *
	 * @param String $dbname
	 * @return PDO database connection
	 */
	function get_database($dbname="sysmaster",$use_user=false)
	{
		if ( ( isset($this->in['act']) )
		   &&( $use_user === true )
		   &&( ($this->in['act'] == "sqlwin" && $this->in['do'] != "dbtab") || $this->in['act'] == "qbe" )
		   &&( $this->get_config('SECURESQL',"on") == "on" ) ) 
		 {
			$sql_dbname="SQLTOOLBOX_".$dbname;
			if (! isset( $this->database[$sql_dbname] ) )
			{
				$this->set_database($dbname , $use_user);
			}
			return $this->database[$sql_dbname];			
		 }else{
			if (! isset($this->database[$dbname]))
			$this->set_database($dbname,$use_user);
	
			return $this->database[$dbname];	
		}
	}

	/**
	 * unset database from our database array...
	 * used if we switch the context of the connection using a database statement
	 */
	function unset_database($dbname)
	{
		if ((isset($this->in['act']))&&($this->in['act'] == "sqlwin")&&($this->get_config('SECURESQL') == "on")){
			$sql_dbname="SQLTOOLBOX_".$dbname;
			if (isset($this->database[$sql_dbname]))
			{
				unset($this->database[$sql_dbname]);
			}
		}else{
			if (isset($this->database[$dbname]))
			{
				unset($this->database[$dbname]);
			}
		}
	}

	/**
	 * add a database connection to our database array ...
	 *
	 * @param String $dbname
	 */
	function set_database($dbname="sysmaster",$use_user=true)
	{
		if ((isset($this->in['act']))
		   &&( $use_user === true )
		   &&(($this->in['act'] == "sqlwin" && $this->in['do'] != "dbtab") || $this->in['act'] == "qbe" )
		   &&($this->get_config('SECURESQL','on') == "on") ) 
		   {
			$sql_dbname="SQLTOOLBOX_".$dbname;
			if (isset($this->database[$sql_dbname]))
			{
				return;
			}
			
			$locale = $this->get_locale($dbname);
			require_once(ROOT_PATH."/lib/database.php");
			$this->database[$sql_dbname] = new database($this,$dbname,$locale,$_SESSION['SQLTOOLBOX_USERNAME'],$_SESSION['SQLTOOLBOX_PASSWORD']);
		}else{
			if (isset($this->database[$dbname]))
			{
				return;
			}
	
			$locale = $this->get_locale($dbname);
	
			require_once(ROOT_PATH."/lib/database.php");
			$this->database[$dbname] = new database($this,$dbname,$locale,"","");
		}	
	}
	/**
	 * get the locale of a database.
	 *
	 * @param String $dbname
	 * @return String
	 */
	function get_locale($dbname)
	{

		if ( $dbname == "sysmaster"
		|| $dbname == "sysadmin"
		|| $dbname == "sysha"
		|| $dbname == "sysuser"
		|| $dbname == "sysutils" )
		{
			return "";
		}

		$smdb = $this->get_database("sysmaster");
		$stmt = $smdb->query("select trim(dbs_collate) as dbs_collate from sysdbslocale where dbs_dbsname='{$dbname}'");
		$row  = $stmt->fetch();

		if (is_null($row['DBS_COLLATE']) || $row['DBS_COLLATE'] == "")
		{
			return "";
		}

		return $row['DBS_COLLATE'];
	}

	/**
	 * set the current database.
	 *
	 * @param String $dbname
	 */
	function set_currdb($dbname="sysmaster")
	{
		if (isset($this->database[$dbname]))
		{
			$this->currdb = $this->database[$dbname];
			return;
		}
		$locale = $this->get_locale($dbname);
		$this->database[$dbname] =  new database($this,$dbname,$locale,"","");
	}

	/*************************************
	 * set_config:
	 * Set a config parameter
	 **************************************/
	function set_config($what,$value)
	{
		$this->config[$what] = $value;
	} #end set_config

	/*************************************
	 * check_modules:
	 * Sanity check php modules: PDO,
	 * 	 pdo_informix, pdo_sqlite, soap
	 **************************************/
	function check_modules()
	{
		$required = array("PDO" => "true",
        	"pdo_informix" => "true",
        	"pdo_sqlite" => "true",
        	"soap" => "true");
		$missing = false;

		$plugins = get_loaded_extensions();
		foreach ($required as $module => $value)
		{
			if (!in_array($module, $plugins ) )
			{
				$required[$module] = false;
				$missing = true;
			}
		}

		// If any of required modules are missing, print error and die
		if ($missing)
		{
			$errmsg = "Not all required PHP modules are loaded. <BR><BR>";
			$errmsg .= "Missing module(s) : <BR> ";
			foreach ($required as $module => $value)
			{
				if (!$value)
				{
					$errmsg .= " -  $module <BR>";
				}
			}

			$this->fatal_error($errmsg, false, E_USER_ERROR);
			die();
		}
	} #end check_modules

	/*************************************
	 * set_INFORMIXDIR:
	 * Set the INFORMIXDIR config parameter
	 **************************************/
	function set_INFORMIXDIR()
	{
		$INFORMIXDIR=getenv("INFORMIXDIR");
		if ( $INFORMIXDIR == "" || $INFORMIXDIR == NULL)
		{
			$this->fatal_error("INFORMIXDIR is not set.", false, E_USER_ERROR);
			die();
		}
		$this->set_config("INFORMIXDIR",$INFORMIXDIR);
	} #end set_INFORMIXDIR

	/******************************************
	 * load_template:
	 * Load a template into the idsadmin array.
	 *******************************************/
	function load_template($template_name)
	{
			
		if ( !isset($this->template[$template_name]) )
		{
			$template_fname = ROOT_PATH."templates/{$template_name}.php";

			// If current module is plugin, template could be located
			// in the default location or in the plugin's templates directory.
			if ($this->isPlugin() == 1 && !file_exists($template_fname))
			{
				$template_fname = $this->get_classpath() ."/templates/{$template_name}.php";
			}

			// Check if template file exists
			if (!file_exists($template_fname))
			{
				$this->fatal_error("Cannot find template $template_name");
			}

			require_once($template_fname);

			$this->template[$template_name] = new $template_name($this);
			$this->template[$template_name]->idsadmin = &$this;
		}
		return ;
	}# end load_template

	/************************************************************
	 * get_version:
	 * Returns the version of IDSAdmin.
	 *************************************************************/
	function get_version()
	{
		return $this->version;
	}

	/************************************************************
	 * get_buildtime:
	 * Returns the build time of IDSAdmin.
	 *************************************************************/
	function get_buildtime()
	{
		return $this->buildtime;
	}

	function get_lang_file( $file, $lang="en_US" )
	{
		if ( $file != "menu" )
		{
			$fname = $this->get_classpath();
			if ( $this->isPlugin() == 0 )
			{
				/* remove the module directory */
				$fname = substr($fname,0,-8);   // remove the '/modules'
			}
		}
		else
		{
			$fname = ".";
		}

		if ( IN_ADMIN == 1 )
		{
			$fname = "../";
		}

		/* If we couldn't find it still, then check if its a services PHP file
		 * and check path relative to that */     
		if (!file_exists($fname . "/lang/$lang/lang_$file.xml"))
		{
		    if (isset($this->render) && (!$this->render))
		    {
		        // It is a service
		        
                        // Now is it a plugin service trying to load a regular OAT lang file?
                        if ( file_exists("../../../../lang/$lang/lang_$file.xml"))
                        {
                            $fname = "../../../..";
                        } elseif ( file_exists("../lang/$lang/lang_$file.xml")) {
                                // Or is it a plugin trying to load its own lang file?
                                $fname = "..";
                        } else {
                                // Or is it a regular service?
                                $fname = "../..";
                        }

		    }
		}
		
		/* Last path to try if we haven't found the lang file yet*/
		if (!file_exists($fname . "/lang/$lang/lang_$file.xml"))
		{
		    $fname = ".";
        }
        
		$fname = $fname . "/lang/$lang/lang_$file.xml";

		return $fname;

	}

	/**
	 * Load the language file for the plugin installed in $dir
	 *
	 * @param string $dir
	 */
	function load_plugin_menu_lang($dir)
	{
		if ( $dir == "" )
		{
			return;
		}
		// get the current language.
		
		$l = $this->phpsession->get_lang();
		if ( $l == "" )
		{
			$l = "en_US";
		}
		
        $fname = "";
		if ( IN_ADMIN == true ) 
		{
			$fname = "../";
		}
		
		// As a fallback for incomplete translations, load English strings first.
		$fname = "{$fname}plugin/{$dir}/lang/en_US/lang_menu.xml";
		
		if ( file_exists($fname) )
		{
			$xml = simplexml_load_file( $fname );
			if (! is_null($xml))
			{
				foreach ( $xml as $k )
				{
					$name = (string)$k->getName();
					$this->language[$name]=(string)$xml->$name;
				}
			}
			unset($xml);
		}

		// Now we'll load the correct language file, which will overwrite all English strings
		// that translations exist for.
		if ($l != "en_US")
		{
    		$fname = str_replace("en_US",$l,$fname);
    		if ( file_exists($fname) )
    		{
    			$xml = simplexml_load_file( $fname );
    			if (! is_null($xml))
    			{
    				foreach ( $xml as $k )
    				{
    					$name = (string)$k->getName();
    					$this->language[$name]=(string)$xml->$name;
    				}
    			}
    			unset($xml);
    		}
		}
	} // end load_plugin_menu_lang
	/************************************************************
	 * load_lang:
	 * Load a modules language file into the idsadmin lang array.
	 *************************************************************/
	function load_lang($file)
	{
		$l = $this->phpsession->get_lang();
		if ( $l == "" )
		{
			$l="en_US";
		}

		$lang = array();
		$fname = $this->get_lang_file($file,$l);

		if ( file_exists($fname) )
		{
			$xml = simplexml_load_file( $fname );
		}
		else
		{
			// lets see if its in the plugins ..
			$fname =  ( $this->get_lang_file( $file ) ) ;
			if ( file_exists( $fname ) )
			{
				$xml = simplexml_load_file( $fname );
			}
		}

		if (! is_null($xml))
		{
			foreach ( $xml as $k )
			{
				$name = (string)$k->getName();
					
				$this->language[$name]=(string)$xml->$name;
			}
		}

		unset($xml);
		$this->set_last_lang_file($fname);

	}# end load_lang


	/*****************************************
	 *  remove a parameter from the url
	 *****************************************/
	function removefromurl($toremove="",$url="")
	{
		if ($url == "")
		$url = $_SERVER['REQUEST_URI'];
		return preg_replace('/&'.$toremove.'\=[^&]*/', '',$url);
	} // end removefromurl

	function set_last_lang_file($file)
	{
		if (isset($this->last_lang_file))
		{
			array_push($this->last_lang_file, $file);
		}
		else
		{
			$this->last_lang_file = Array($file);
		}

	} // end set_last_lang_file

	function get_last_lang_file()
	{
		return $this->last_lang_file;
	} // end get_last_lang_file

	/*********************************************************
	 *  function lang( $item, $parameters=array() )
	 *
	 *  Find a translation string and convert it to
	 *  the correct language.  If we are not able to
	 *  find what we want in a non-english language then
	 *  look it up in english.  If we still can not
	 *  the string then print out an error.
         *
         *  Optional parameters will get substituted for '{0}',
         *  '{1}', '{2}', etc.
	 **********************************************************/
	function lang($item, $parameters=array())
	{
			
		$item=trim($item);

		if ( $item == "" )
		{
			return ;
		}

		if( isset($this->language["{$item}"]) && !empty( $this->language["{$item}"] ) )
		{
			return $this->lang_substitute_parameters($this->language["{$item}"], $parameters);
		}

		/*
		 * If the translation files are correct we will never
		 *  get here, but if a foreign lang file is missing a
		 *  string then try and look it up in english file
		 */

		$lang_array = $this->get_last_lang_file();
		foreach ($lang_array as $langfile)
		{
			$langfile = str_replace($this->phpsession->get_lang(),"en_US",$langfile);
			$xml_lang = simplexml_load_file($langfile);
			if( isset( $xml_lang->{$item} ) && !empty(  $xml_lang->{$item}  ) )
			{
				return $this->lang_substitute_parameters((string)$xml_lang->{$item}, $parameters);
			}
		}
		/* If we can not find what we want print
		 * out what we are missinig
		 */

		return "MISSING LANG FILE ITEM $item";
	} // end lang

        /*********************************************************
         *  function lang_substitute_parameters( $string, $parameters=array() )
         *
         *  Substitute parameters into our localized string.
         *  Optional parameters will get substituted for '{0}',
         *  '{1}', '{2}', etc.
         **********************************************************/
        private function lang_substitute_parameters($string, $parameters=array())
        {
            for ($i = 0; $i < count($parameters); $i++)
            {
                $replace = "/\\{" . $i . "\\}/";
                $string = preg_replace($replace,$parameters[$i],$string);
            }
            return $string;

        } // end lang_substitute_parameters


	// display an error in a red box
	function error($msg="")
	{
		$html = $this->template["template_global"]->global_error($msg);
		$this->html->add_to_output($html);
	} //end error

	/**
	 * called if an error occurs connecting to the database.
	 *
	 * @param string $msg
	 */
	function db_error($msg="",$title="")
	{
		/*
		 * check if we are actually rendering output , and this not part of a webservice call ,
		 * if we arent rendering then use trigger_error and hopefully this will get caught by the service.
		 */
		if ( $this->render == false )
		{
			$msg .= " ROOT_PATH=".ROOT_PATH;
			trigger_error(htmlentities($msg),E_USER_ERROR);
		}

		$html = $this->template["template_global"]->global_error($msg, $title);
		$this->html->add_to_output($html);
		$this->html->render();
		die();
	} //end db_error
	
	
	/**
         * Display a fatal error in a red box
         * The $render_menu parameter indicates whether the menu should be rendered with the error
         **/
	function fatal_error($msg="",$render_menu=true)
	{
		/*
		 * check if we are actually rendering output , and this not part of a webservice call ,
		 * if we arent rendering then use trigger_error and hopefully this will get caught by the service.
		 */
		if ( $this->render == false )
		{
			$msg .= " ROOT_PATH=".ROOT_PATH;
			trigger_error(htmlentities($msg),E_USER_ERROR);
		}
		
		$html = $this->template["template_global"]->global_error($msg);
                if (! $render_menu) 
		{ 
			$this->iserror=true;
		}
		$this->html->add_to_output($html);
		$this->html->render();
		die();
	} //end fatal_error

	// format to human readable
	function format_units($value,$prec=2)
	{
		$suffix = array("B","KB","MB","GB","TB","PB","EB");
		//assume value is byte for now ..
		$cnt = 0;
		while ( $value >= 1024 )
		{
			$cnt++;
			$value /= 1024;
		}
			
		$value = round($value,$prec);
			
		return $value." ".$suffix[$cnt];
			
	} // end format_units

	// display a 'status' message
	function status($msg="")
	{
		$html = $this->template["template_global"]->global_status($msg);
		$this->html->add_to_output($html);
	} //end status

	// timedays - given the number of seconds return a days hours:mins:sec
	function timedays($sec)
	{
		$this->load_lang("updstats");
		$days = "";
		$hours = intval(intval($sec) / 3600);
		if ($hours > 24) {
			$days  = floor($hours / 24)." ".$this->lang("days");
			$hours = $hours % 24;
		}
		$hours = str_pad($hours, 2, "0", STR_PAD_LEFT).":";
		$minutes = intval(($sec / 60) % 60);
		$minutes = str_pad($minutes, 2, "0", STR_PAD_LEFT).":";
		$seconds = intval($sec % 60);
		$seconds = str_pad($seconds, 2, "0", STR_PAD_LEFT);
		return "{$days} {$hours}{$minutes}{$seconds}";
	} #end timedays

	/**
	 * Save the url params.
	 */
	function saveurl()
	{
		$this->phpsession->set_lasturl($_SERVER['REQUEST_URI']);
	}

	/**
	 * Returns an HTML Select box listing all database names
	 */
	function getDatabaseSelect()
	{
	    // if we are in the sql part of the toolbox we want to redirect 
	    // the user back to that part , so we add default to our url so that 
	    // we can check this later in sql_dbconnect .
		if ( $this->in['do'] == "sqltab" || $this->in['do'] == "sqlrestab" )
		{
			$default = "&amp;default=1";
		}
		$db = $this->get_database("sysmaster");
		$qry = "select name as dbname from sysmaster:sysdatabases";
		$stmt = $db->query($qry);
		$dbselect = $this->template["template_global"]->switchDatabaseJscript();
		$dbselect .= "<form name='dbswitch' method='post' action=\"index.php?act=sqlwin&amp;do=connect{$default}\">"
		."<select name=\"val\" onchange='switchDatabase(this)' class='switchSelect'>";
		while ($res = $stmt->fetch())
		{
			$dbname = trim($res['DBNAME']);
			if (strcmp($dbname, $this->phpsession->get_sqldbname()) == 0)
			{
				$dbselect .= "<option class=\"switchSelectOption\" value=\"{$dbname}\" selected=\"yes\">{$dbname}</option>";
			} else {
				$dbselect .= "<option class=\"switchSelectOption\" value=\"{$dbname}\">{$dbname}</option>";
			}
		}
		$dbselect .= "</select></form>";
		$stmt->closeCursor();
		return $dbselect;
	}

	/**
	 * return a list of servers in my current group.
	 */
	function getServers()
	{
	    if ( $this->iserror )
	    {
	    	return;
	    }
	    
		if ( $this->phpsession->get_group() == "" )
		{
			return  $this->phpsession->instance->get_servername();
		}

		require_once("lib/connections.php");
		$conndb = new connections($this);

		$sql = "select conn_num,server,host from connections where group_num='{$this->phpsession->get_group()}' order by server";
		$stmt = $conndb->db->query($sql);

		$html = "";
		$options = "";
		$found = false;
		while ( $row = $stmt->fetch() )
		{
			$selected = "";
			if ( $this->phpsession->instance->get_servername() == $row['SERVER'] &&
                             $this->phpsession->instance->get_conn_num() == $row['CONN_NUM'] )
			{
				$selected = "SELECTED";
				$found=true;
			}

			$options .= "<option class='switchSelectOption' value='{$row['CONN_NUM']}' {$selected}>{$row['SERVER']}@{$row['HOST']}</option> ";
		}
		$stmt->closeCursor();

		// if our current server wasnt found then add it to the list.
		if ( $found === false )
		{
			$options .= "<option class='switchSelectOption' value='' SELECTED>{$this->phpsession->instance->get_servername()}@{$this->phpsession->instance->get_host()}</option>";
		}
		$html  = $this->template["template_global"]->switchServerJscript();
		$html .= "<form method='post' action='index.php?act=login&amp;do=loginnopass&amp;{$this->get_redirect()}' name='serverswitch' >";
		$html .= " <select onChange='switchServers(this)' name='conn_num' class='switchSelect' style='vertical-align: middle'>{$options}</select>";
		$html .= "</form>";
		return $html;
	}

	/**
	 * experimental bread crumbs..
	 *
	 * @param string $title
	 * @param string $link
	 */
	function addCrumb($title,$link="")
	{
		if ( $link == "" )
		{
			$link = $this->phpsession->get_lasturl();
		}

		if ( $link == "NONE" )
		{
			$link = "";
		}

		$this->crumb[] = array( "title" => $title , "link" => $link );
	}

	/**
	 * returns the current menu item
	 * i.e. which menu item should be highlighted
	 */
	function getCurrMenuItem()
	{
		return $this->currMenuItem;
	}

	/**
	 * sets the current menu item
	 * i.e. which menu item should be highlighted
	 */
	function setCurrMenuItem($item="")
	{
		$this->currMenuItem = $item;
	}

	// check if there is a group
	function checkForFeature($feature , $desc="")
	{
	    
	    $db = $this->get_database("sysadmin");
	    
    	// Check if '{$feature}' exists in ph_group
    	$qry = "select COUNT(*) as GROUPEXISTS from ph_group where group_name = '{$feature}'";	    	
	    $stmt = $db->query($qry);
	    $res = $stmt->fetch();
	    $stmt->closeCursor(); 	    		    		    	    	
	    if (count($res) == 1)
	    {
	    	if ($res['GROUPEXISTS'] == 1)
	    	{	
	    		// '{$feature}' group already exists, so return
	    		return;    			
	    	}
	    }
	    
	    // If not, create the group
    	$qry = "insert into ph_group(group_name,group_description) "
    		 . "values ('{$feature}','{$desc}')";
    	$result = $db->query($qry);
		return $result;    			
    
	}
	 /**
     * Tests if the admin_async UDR exists on sysadmin.
     * If not, this function creates the admin_async procedure.
     **/    
    function testAndDeployAdminAsync($feature="",$desc="")
    {
    	// Check if the admin_async procedure already exists
    	$qry = "select COUNT(*) as UDREXISTS from sysprocedures where procname = 'admin_async'";
    	$db = $this->get_database("sysadmin");	
    	$stmt = $db->query($qry);
    	    	
	    $res = $stmt->fetch();
	    $stmt->closeCursor();
	   // error_log( var_export( $res,true )); 	    		    		    	    	
	    
	    if (count($res) == 1)
	    {
	    	if ($res['UDREXISTS'] == 1)
	    	{	
	    		// procedure already exists, so return
	    		
	    		// lets do check for feature in ph_group ..
	    		if ( $feature != "" )
	    		{ 
	    		    $this->checkForFeature($feature,$desc);
	    		}	    			
	    	}
	    }
	    	    
	    if ( $res['UDREXISTS'] != 1 )
	    {
	    // If not, create the procedure
    	$qry = <<<EOM
CREATE FUNCTION admin_async(cmd lvarchar(4096),
                          cur_group CHAR(129),
                          comments lvarchar(1024)
                               DEFAULT "Background admin API",
                          start_time DATETIME hour to second
                               DEFAULT CURRENT hour to second,
                          end_time   DATETIME hour to second
                               DEFAULT NULL,
                          frequency  INTERVAL day(2) to second
                               DEFAULT NULL,
                          monday    BOOLEAN DEFAULT 't',
                          tuesday   BOOLEAN DEFAULT 't',
                          wednesday BOOLEAN DEFAULT 't',
                          thursday  BOOLEAN DEFAULT 't',
                          friday    BOOLEAN DEFAULT 't',
                          saturday  BOOLEAN DEFAULT 't',
                          sunday    BOOLEAN DEFAULT 't'
                          )
   RETURNING INTEGER
   DEFINE ret_task_id  INTEGER;
   DEFINE del_time     INTERVAL DAY TO SECOND;
   DEFINE id           INTEGER;
   DEFINE task_id      INTEGER;
   DEFINE seq_id       INTEGER;
   DEFINE cmd_num      INTEGER;
   DEFINE boot_time    DATETIME YEAR TO SECOND;

--SET DEBUG FILE TO "/tmp/debug.out";
--TRACE ON;

   IF cur_group IS NULL THEN
       LET cur_group = "MISC";
   END IF

   SELECT FIRST 1 value::INTERVAL DAY TO SECOND INTO del_time FROM ph_threshold
      WHERE name = "BACKGROUND TASK HISTORY RETENTION";
   IF del_time IS NULL THEN
       LET del_time = 7 UNITS DAY;
   END IF

    BEGIN
        ON EXCEPTION IN ( -310, -316 )
        END EXCEPTION WITH RESUME

            CREATE TABLE job_status (
               js_id         SERIAL,
               js_task       INTEGER,
               js_seq        INTEGER,
               js_comment    LVARCHAR(512),
               js_command    LVARCHAR(4096),
               js_start      DATETIME year to second
                             DEFAULT CURRENT year to second,
               js_done       DATETIME year to second DEFAULT NULL,
               js_result     INTEGER
           );
            CREATE INDEX job_status_ix1 ON job_status(js_id);
            CREATE INDEX job_status_ix2 ON job_status(js_task);
            CREATE INDEX job_status_ix3 ON job_status(js_result);
     END

     BEGIN
        ON EXCEPTION IN ( -8301 )
        END EXCEPTION WITH RESUME

        CREATE SEQUENCE background_task START 1 NOMAXVALUE ;

     END

    INSERT INTO ph_task
        ( tk_name,
        tk_description,
        tk_type,
        tk_group,
        tk_execute,
        tk_start_time,
        tk_stop_time,
        tk_frequency,
        tk_Monday,
        tk_Tuesday,
        tk_Wednesday,
        tk_Thursday,
        tk_Friday,
        tk_Saturday,
        tk_Sunday,
        tk_attributes
        )
        VALUES
        (
        "Background Task ("||  background_task.NEXTVAL ||")",
        TRIM(comments),
        "TASK",
        cur_group,
        "insert into job_status (js_task, js_seq , js_comment,js_command) VALUES(\$DATA_TASK_ID,\$DATA_SEQ_ID, '"||TRIM(comments)||"','"||TRIM(REPLACE(REPLACE(cmd,"'"),"""") )||"' ); update job_status set (js_result)= ( admin("||TRIM(cmd)||") ) WHERE js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID ;update job_status set (js_done)  = ( CURRENT ) WHERE js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID ",
        start_time,
        end_time,
        frequency,
        monday,
        tuesday,
        wednesday,
        thursday,
        friday,
        saturday,
        sunday,
        8);

   LET ret_task_id = DBINFO("sqlca.sqlerrd1");

   /* Cleanup the job_status table */

   SELECT dbinfo('UTC_TO_DATETIME',sh_boottime)
          INTO boot_time
           FROM sysmaster:sysshmvals;

   FOREACH  SELECT js_id, js_task, js_seq, js_result
        INTO id, task_id,  seq_id, cmd_num
        FROM job_status J, OUTER ph_run, command_history
        WHERE  ( CURRENT - js_done > del_time OR
                (js_start < boot_time AND js_done IS NULL ) )
        AND    js_task = run_task_id
        AND    js_seq  = run_task_seq
        AND    js_result = ABS(cmd_number)

       DELETE FROM ph_run WHERE run_task_id = task_id
                          AND run_task_seq = seq_id;
       DELETE FROM command_history WHERE cmd_number = cmd_num;
       DELETE FROM job_status WHERE js_id = id;

       -- Cleanup the task table only if this is not a repeating task
       DELETE FROM ph_task WHERE tk_id = task_id AND tk_next_execution IS NULL;

   END FOREACH

   RETURN  ret_task_id;
END FUNCTION;
   
EOM
;    	
    	$stmt = $db->query($qry);
    	
    	// With the admin_async procedure, we also need to add into the ph_threshold
    	// table the 'BACKGROUND TASK HISTORY RETENTION' threshold.
    	$qry = <<<EOM
INSERT INTO ph_threshold
(name,task_name,value,value_type,description)
VALUES
("BACKGROUND TASK HISTORY RETENTION", "Cleanup background tasks","7 0:00:00","NUMERIC",
"Remove all dormant background tasks that are older than then the threshold.")
EOM
; 	
		$result = $db->query($qry);
	    }
		// now lets create the job_status table so that we can potentially use it before 
		// running the procedure . 
		
	    $qry = "select count(*) as cnt from systables where tabname = 'job_status'";
	    $stmt = $db->query($qry);
	    $row = $stmt->fetch();
	    $stmt->closeCursor();
	   
	    if ( $row['CNT'] == 0 )
	    {
	    
		$qry = <<<EOM
		            CREATE TABLE job_status (
               js_id         SERIAL,
               js_task       INTEGER,
               js_seq        INTEGER,
               js_comment    LVARCHAR(512),
               js_command    LVARCHAR(4096),
               js_start      DATETIME year to second
                             DEFAULT CURRENT year to second,
               js_done       DATETIME year to second DEFAULT NULL,
               js_result     INTEGER
           );
EOM;
		

        $db->query($qry);
        // now the indexes.
        $db->query(" CREATE INDEX job_status_ix1 ON job_status(js_id); ");
        $db->query(" CREATE INDEX job_status_ix2 ON job_status(js_task) ");
        $db->query(" CREATE INDEX job_status_ix3 ON job_status(js_result) ");
	    }
	    
        $this->checkForFeature($feature,$desc);
        
    	return;    			
    } 
	
	// TODO: Add functionality to cleanuptmp
	function cleanuptmp()
	{
			
	}
	/******************************************
	 * Destruct:
	 *******************************************/
	function __destruct()
	{
			
	} #end __destruct

} #end IDSAdmin class
?>
