<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

/**
 * The compression page for OpenAdmin
 *
 */
class compression {

	public $idsadmin;

	# the 'constructor' function
	# called when the class "new'd"

	function __construct(&$idsadmin)
	{
		$this->idsadmin = &$idsadmin;
		$this->idsadmin->load_template("template_compression");
		$this->idsadmin->load_lang("compression");
		$this->idsadmin->html->set_pagetitle($this->idsadmin->lang('compressionPageTitle'));
	}


	/*
	 * the run function
	 * this is what index.php will call
	 * the decision of what to actually do is based on
	 * the value of 'act' which is either posted or getted
	 */

	function run()
	{
		// check server version is UC4 .
		/* Is the version of the server compatiable with OAT */
		$feature = Feature::CHEETAH2_UC4;


		if ( ! Feature::isAvailable ( $feature, $this->idsadmin->phpsession->serverInfo->getVersion()  )  )
		{
			$this->idsadmin->fatal_error( $this->idsadmin->lang('CompressOnlyAvail') . " " . Feature::getVersion ( $feature ) );
			$this->idsadmin->html->render();
			die();
		}


		switch($this->idsadmin->in['do'])
		{
			default:
				$this->idsadmin->setCurrMenuItem("compression");
				$this->def();
				break;
		}
	} # end function run


	function def()
	{

		$this->idsadmin->testAndDeployAdminAsync("COMPRESSION" , "Compression jobs");
		$this->createDBCronTaskEstimateCompression();

		$db = $this->idsadmin->get_database("sysmaster");
		$sel = "SELECT decode(bitand(feature_flags, '0x2000'),0,0,1) as value from sysmaster:syslicenseinfo ";
		$stmt = $db->query($sel);
		$row  = $stmt->fetch();
		$stmt->closeCursor();

		$compress_enabled = 0;

		if ( isset($row['VALUE']) && $row['VALUE'] > 0 ) // 16
		{
			$compress_enabled = 1;
		}

		if ( $compress_enabled == 0 )
		{
			$sel = " SELECT count(*) AS value FROM ( SELECT FIRST 1 * FROM sysmaster:syscompdicts_full)";

			$stmt = $db->query($sel);
			$row  = $stmt->fetch();
			$stmt->closeCursor();
			if ( isset($row['VALUE']) && $row['VALUE'] > 0 ) // 16
			{
				$compress_enabled = 1;
			}
		}


		$this->idsadmin->html->add_to_output($this->idsadmin->template["template_compression"]->render($compress_enabled,$this->idsadmin->phpsession->get_lang()));
	} // end def

	function createDBCronTaskEstimateCompression()
	{

		// first let check if the mon_compression_estimates table exists ,
		// this is normally created by the SENSOR , but if that has yet to run
		// then errors will ensue about the table not existing ..

		$sel = "SELECT count(*) as CNT FROM systables WHERE tabname = 'mon_compression_estimates'";
		$db  = $this->idsadmin->get_database("sysadmin");
		$stmt = $db->query($sel);
		$row  = $stmt->fetch();
		$stmt->closeCursor();
		if ( isset($row['CNT']) && $row['CNT'] == 0 )
		{
			$sel = "create table mon_compression_estimates(id  integer, est_partnum integer,est_dbname varchar(128),est_owner  varchar(32),est_tabname varchar(128),est_estimate lvarchar(32000),est_date datetime year to second default CURRENT year to second);";
			$db->exec($sel);
			$sel = "create index mon_estimate_compression_idx1 on mon_compression_estimates ( est_partnum );";
			$db->exec($sel);
		}

		$name = "mon_compression_estimates";
		$sel = "select count(*) as cnt from ph_task WHERE tk_name = '{$name}' ";
		$db = $this->idsadmin->get_database("sysadmin");
		$stmt = $db->query($sel);
		$row = $stmt->fetch();
		$stmt->closeCursor();

		/* we found it so just return */
		if ( isset($row['CNT']) && $row['CNT'] == 0 )
		{

			/* lets create it */

			$sel = <<< EOF
         CREATE FUNCTION mon_estimate_compression(task_id integer, id integer)
         RETURNING integer;

DEFINE p INTEGER;
DEFINE d VARCHAR(128);
DEFINE o VARCHAR(32);
DEFINE t VARCHAR(128);
DEFINE e VARCHAR(255);

FOREACH
SELECT tab.partnum , TRIM(dbsname) AS dbsname , TRIM(owner) AS owner, TRIM(tabname) AS tabname
,( SUBSTR( sysadmin:task('fragment estimate_compression',tab.partnum) , 102 ) )

INTO p , d , o , t , e
FROM sysmaster:systabnames tab , sysmaster:sysptnhdr hdr
WHERE dbsname NOT IN ( 'sysmaster','sysutils','syscdr','sysuser','system'
, 'syscdcv1','syscdcv2','syscdcv3','syscdcv4','syscdcv5','syscdcv6','syscdcv7'
, 'syscdcv8','syscdcv9')
AND tabname != dbsname
AND nrows >= 2000
--(nvl((SELECT env_value FROM sysenv WHERE env_name = 'IFX_COMPRESSION_MIN_ROWS'),2000))
AND tab.partnum = hdr.partnum

AND tabname NOT IN ( SELECT tabname FROM systables WHERE tabid < 100 )
AND tabname != 'TBLSpace'
AND npdata > 0
AND bitand(flags,'0x0004') != 4



INSERT INTO  mon_compression_estimates
(id, est_partnum , est_dbname , est_owner , est_tabname , est_estimate )
VALUES
(id, p,d,o,t,e);

END FOREACH

RETURN 0;
END FUNCTION

EOF;

			$db->query($sel);

			$sel = <<<EOF
        INSERT INTO ph_task
(
tk_name,
tk_type,
tk_group,
tk_description,
tk_result_table,
tk_create,
tk_execute,
tk_stop_time,
tk_start_time,
tk_frequency,
tk_delete
)
VALUES
(
'{$name}',
'SENSOR',
'COMPRESSION',
'Get compression estimates',
'mon_compression_estimates',
'create table mon_compression_estimates(id  integer, est_partnum integer,est_dbname varchar(128),est_owner  varchar(32),est_tabname varchar(128),est_estimate lvarchar(32000),est_date datetime year to second default CURRENT year to second); create index mon_estimate_compression_idx1 on mon_compression_estimates ( est_partnum );',
'mon_estimate_compression',
NULL,
DATETIME(02:30:00) HOUR TO SECOND,
INTERVAL ( 7 ) DAY TO DAY,
INTERVAL ( 30 ) DAY TO DAY
);

EOF;
			$db->query($sel);

		}

		$name = "get_compression_estimate";
		$sel = "select count(*) as cnt from sysadmin:sysprocedures WHERE procname = '{$name}' ";
		$db = $this->idsadmin->get_database("sysadmin");
		$stmt = $db->query($sel);
		$row = $stmt->fetch();
		$stmt->closeCursor();

		/* we found it so just return */
		if ( isset($row['CNT']) && $row['CNT'] == 0 )
		{

			$sel = <<< EOF
        CREATE FUNCTION get_compression_estimate( pnum varchar(128) , comptype varchar(12)  , dbname varchar(128) , owner varchar(32) )
returning int;

define e LVARCHAR;
define cmd integer;

--SET DEBUG FILE TO "/tmp/debug2.out";
--TRACE ON;

IF comptype == 'fragment' THEN
select admin(comptype||' estimate_compression',pnum) INTO cmd from sysmaster:sysdual ;
ELSE
select admin(comptype||' estimate_compression',pnum , dbname , owner ) INTO cmd from sysmaster:sysdual ;
END IF

return cmd;
end function;

EOF;
			$db->query($sel);
		}

		$name = "admin_async_estimates";
		$sel = "select count(*) as cnt from sysadmin:sysprocedures WHERE procname = '{$name}' ";
		$db = $this->idsadmin->get_database("sysadmin");
		$stmt = $db->query($sel);
		$row = $stmt->fetch();
		$stmt->closeCursor();

		/* we found it so just return */
		if ( isset($row['CNT']) && $row['CNT'] == 0 )
		{

			$sel = <<< EOF
CREATE FUNCTION admin_async_estimates(cmd varchar(128),
                          comptype varchar(12),
                          dbname varchar(128),
                          owner  varchar(32),
                          cur_group CHAR(129),
                          comments lvarchar(1024)
                               DEFAULT "Background admin API",
                          start_time DATETIME hour to second
                               DEFAULT CURRENT hour to second,
                          end_time   DATETIME hour to second
                               DEFAULT NULL,
                          frequency  INTERVAL day(2) to second
                               DEFAULT NULL,
                          monday    BOOLEAN DEFAULT 't',
                          tuesday   BOOLEAN DEFAULT 't',
                          wednesday BOOLEAN DEFAULT 't',
                          thursday  BOOLEAN DEFAULT 't',
                          friday    BOOLEAN DEFAULT 't',
                          saturday  BOOLEAN DEFAULT 't',
                          sunday    BOOLEAN DEFAULT 't'
                          )
   RETURNING INTEGER
   DEFINE ret_task_id  INTEGER;
   DEFINE del_time     INTERVAL DAY TO SECOND;
   DEFINE id           INTEGER;
   DEFINE task_id      INTEGER;
   DEFINE seq_id       INTEGER;
   DEFINE cmd_num      INTEGER;
   DEFINE boot_time    DATETIME YEAR TO SECOND;

   IF cur_group IS NULL THEN
       LET cur_group = 'MISC';
   END IF

   SELECT FIRST 1 value::INTERVAL DAY TO SECOND INTO del_time FROM ph_threshold
      WHERE name = 'BACKGROUND TASK HISTORY RETENTION';
   IF del_time IS NULL THEN
       LET del_time = 7 UNITS DAY;
   END IF

    BEGIN
        ON EXCEPTION IN ( -310, -316 )
        END EXCEPTION WITH RESUME

            CREATE TABLE job_status (
               js_id         SERIAL,
               js_task       INTEGER,
               js_seq        INTEGER,
               js_comment    LVARCHAR(512),
               js_command    LVARCHAR(4096),
               js_start      DATETIME year to second
                             DEFAULT CURRENT year to second,
               js_done       DATETIME year to second DEFAULT NULL,
               js_result     INTEGER
           );
            CREATE INDEX job_status_ix1 ON job_status(js_id);
            CREATE INDEX job_status_ix2 ON job_status(js_task);
            CREATE INDEX job_status_ix3 ON job_status(js_result);
     END

     BEGIN
        ON EXCEPTION IN ( -8301 )
        END EXCEPTION WITH RESUME

        CREATE SEQUENCE background_task START 1 NOMAXVALUE ;

     END
   IF comptype == 'fragment' THEN

    INSERT INTO ph_task
        ( tk_name,
        tk_description,
        tk_type,
        tk_group,
        tk_execute,
        tk_start_time,
        tk_stop_time,
        tk_frequency,
        tk_Monday,
        tk_Tuesday,
        tk_Wednesday,
        tk_Thursday,
        tk_Friday,
        tk_Saturday,
        tk_Sunday,
        tk_attributes
        )
        VALUES
        (
        'Background Task ('||  background_task.NEXTVAL ||')',
        TRIM(comments),
        'TASK',
        cur_group,
        "insert into job_status (js_task, js_seq , js_comment,js_command) VALUES(\$DATA_TASK_ID,\$DATA_SEQ_ID, '"||TRIM(comments)||"','"||"Estimate for "||comptype||" "||TRIM(REPLACE(REPLACE(cmd,"'"),"""") )||"' ); update job_status set js_result=get_compression_estimate("""||TRIM(cmd)||""" ,"""||comptype||""" , """" , """" )  WHERE js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID; insert into mon_compression_estimates SELECT 0,tab.partnum , TRIM(dbsname) , TRIM(owner) , TRIM(tabname) , substr(trim(cmd_ret_msg),102) , current FROM sysmaster:systabnames tab , sysmaster:sysptnhdr hdr , command_history where hdr.partnum = tab.partnum and tab.partnum ="""||cmd||""" and cmd_number = ( select case when ( js_result < 0 ) then js_result *-1 else js_result end from job_status where  js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID ) ;update job_status set (js_done)  = ( CURRENT ) WHERE js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID ;" ,
                start_time,
        end_time,
        frequency,
        monday,
        tuesday,
        wednesday,
        thursday,
        friday,
        saturday,
        sunday,
        8);
        ELSE

    INSERT INTO ph_task
        ( tk_name,
        tk_description,
        tk_type,
        tk_group,
        tk_execute,
        tk_start_time,
        tk_stop_time,
        tk_frequency,
        tk_Monday,
        tk_Tuesday,
        tk_Wednesday,
        tk_Thursday,
        tk_Friday,
        tk_Saturday,
        tk_Sunday,
        tk_attributes
        )
        VALUES
        (
        'Background Task ('||  background_task.NEXTVAL ||')',
        TRIM(comments),
        'TASK',
        cur_group,

        "insert into job_status (js_task, js_seq , js_comment,js_command) VALUES(\$DATA_TASK_ID,\$DATA_SEQ_ID, '"||TRIM(comments)||"','"||"Estimate for "||comptype||" "||TRIM(REPLACE(REPLACE(cmd,"'"),"""") )||"' ); update job_status set js_result=get_compression_estimate("""||TRIM(cmd)||""" ,"""||comptype||""", """||dbname||""" , """||owner||""")  WHERE js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID; insert into mon_compression_estimates SELECT 0,tab.partnum , TRIM(dbsname) , TRIM(owner) , TRIM(tabname) , substr(trim(cmd_ret_msg),102) , current FROM sysmaster:systabnames tab , sysmaster:sysptnhdr hdr , command_history where hdr.partnum = tab.partnum and tab.tabname ="""||cmd||"""  and dbsname = """||dbname||""" and owner = """||owner||""" and cmd_number = ( select case when ( js_result < 0 ) then js_result *-1 else js_result end from job_status where  js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID ) ;update job_status set (js_done)  = ( CURRENT ) WHERE js_task = \$DATA_TASK_ID AND js_seq = \$DATA_SEQ_ID ;" ,

        start_time,
        end_time,
        frequency,
        monday,
        tuesday,
        wednesday,
        thursday,
        friday,
        saturday,
                sunday,
        8);


END IF

   LET ret_task_id = DBINFO('sqlca.sqlerrd1');

   /* Cleanup the job_status table */

   SELECT dbinfo('UTC_TO_DATETIME',sh_boottime)
          INTO boot_time
           FROM sysmaster:sysshmvals;

   FOREACH  SELECT js_id, js_task, js_seq, js_result
        INTO id, task_id,  seq_id, cmd_num
        FROM job_status J, OUTER ph_run, command_history
        WHERE  ( CURRENT - js_done > del_time OR
                (js_start < boot_time AND js_done IS NULL ) )
        AND    js_task = run_task_id
        AND    js_seq  = run_task_seq
        AND    js_result = ABS(cmd_number)

       DELETE FROM ph_run WHERE run_task_id = task_id
                          AND run_task_seq = seq_id;
       DELETE FROM command_history WHERE cmd_number = cmd_num;
       DELETE FROM job_status WHERE js_id = id;

       -- Cleanup the task table only if this is not a repeating task
       DELETE FROM ph_task WHERE tk_id = task_id AND tk_next_execution IS NULL;

   END FOREACH

   RETURN  ret_task_id;
END FUNCTION;

EOF;
			$db->query($sel);
		}

	}
} // end class compression
?>
