<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007, 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

/**
 * This class is used to track each onstat option
 *
 */
class onstatoption {


    /**
     *  The HTML tag name.
     * Always the same as cmd, as long as
     * cmd is unique accross all classes.  If
     * cmd is not unqiue then you must make this
     * different.
     *
     * @var string
     */
    public $option;
    /**
     * The class name that contains this reports function.
     *
     * @var string
     */
    public $cname;
    /**
     * The option used to access this report inside the
     * run() function. (i.e. the value used by
     * $this->idsadmin->in['do'] )
     *

     *
     * @var string
     */
    public $cmd;
    /**
     * The group of functionality to associate with
     * this onstat option.
     *
     * @var string
     */
    public $group;

    /** 
     * NOTE: So that report title and descriptions are localized,
     * the TITLE of the onstatoption should be retrieved using $this->idsadmin->lang($option)
     * and the DESCRIPTION of the onstatoption should be retrieved using $this->idsadmin->lang($option . "_tooltip") 
     **/
    
    /**
     * Used to create different onstat options that will
     * be displayed in the reprot builder.
     *
     * @param string $option
     * @param string $cmd
     * @param string $cname
     * @param string $group
     * @return onstatoption
     */
    function __construct($option, $cmd, $cname="onstat",$group="NONE")
    {
        $this->option  = $option;
        $this->cmd     = $cmd;
        $this->cname   = $cname;
        $this->group   = $group;
    }

    /**
     * Get the Option
     *
     * @return string
     */
    function getOption() {
        return $this->option;
    }
    /**
     * Get the command name
     *
     * @return string
     */
    function getCmd() {
        return $this->cmd;
    }
    /**
     * Return the class name
     *
     * @return string
     */
    function getCname() {
        return $this->cname;
    }

    /**
     * Return the group
     *
     * @return string
     */
    function getGroup() {
        return $this->group;
    }
    /**
     * This function will return if the currently set
     * global onstat option "onstatgoups::cur_group"
     * is set to this option's group.  If so return
     * the string which is passed in else return
     * nothing.
     *
     * @param string $ret
     * @return string
     */
    function getGrpStatus($ret="CHECKED ") {
        if (strcasecmp( onstatgroups::$cur_group, $this->group) == 0 ){
            return  $ret;
        } else if (strcasecmp( onstatgroups::$cur_group, "ALL") == 0 ){
            return  $ret;
        }
        return  "";
    }

}

/**
 *  This class is used to group a group
 */
/**
 * This is a list of groups that collects
 * similar onstat function together
 *
 */
class onstatgroups {


    /**
     * The name of the group
     *
     * @var string
     */
    public $gname;
    /**
     * The long description for the group
     *
     * @var string
     */
    public $desc;

    /**
     * The name of the current group, default
     * if the performance group
     *
     * @var string
     */
    public static $cur_group = "perf";

    /**
     * Used to create a new group
     *
     * @param string $gname
     * @param string $desc
     * @return onstatgroups
     */
    function onstatgroups($gname, $desc)
    {
        $this->gname   = $gname;
        if (empty($desc))
        $this->desc    = $this->gname;
        else
        $this->desc    = $desc;
    }

    /**
     * Get the group name
     *
     * @return string
     */
    function getGname() {
        return $this->gname;
    }
    /**
     * Get the long description
     *
     * @return string
     */
    function getDesc() {
        return $this->desc;
    }
    /**
     * Use to see if the current group matches
     * the global group.  If so return the
     * string passed in as "ret" else return
     * nothing.
     *
     * @param string $ret
     * @return string
     */
    function getGrpStatus($ret="CHECKED ") {
        if (strcasecmp( self::$cur_group, $this->getGname()) == 0 ) {
            return  $ret;
        } else if (strcasecmp( self::$cur_group, "ALL") == 0 ){
            return  $ret;
        }
        return  "";
    }
    /**
     * Use to see if the $name matches
     * the global group or the "ALL".  If so return the
     * string passed in as "ret" else return
     * nothing.
     *
     * @param string $name
     * @param string $ret
     * @return string
     */
    static function getMatchGroup($name, $ret="CHECKED ") {
        if (strcasecmp( self::$cur_group, $name) == 0 ) {
            return  $ret;
        } else if (strcasecmp( self::$cur_group, "ALL") == 0 ) {
            return  $ret;
        }

        return  "";
    }
    /**
     * Use to see if the group $name matches
     * the global group.  If so return the
     * string passed in as "ret" else return
     * nothing.
     *
     * @param string $name
     * @param string $ret
     * @return string
     */
    static function equalGroup($name, $ret="CHECKED ") {
        if (strcasecmp( self::$cur_group, $name) == 0 )
        {
            return  $ret;
        }

        return  "";
    }

}

/**
 * System specific report list
 */
global $olist;

$olist = array();
$olist[] = new onstatoption("diskspace","show","chunk","Disk");
$olist[] = new onstatoption("diskio","chunkio","chunk","Disk");
$olist[] = new onstatoption("showOnlineLogTail","showOnlineLogTail", "show","Server");
$olist[] = new onstatoption("showCommands","showCommands", "show","Disk");
$olist[] = new onstatoption("showllogs","showllogs", "rlogs","Disk");
$olist[] = new onstatoption("showplogs","showplogs", "rlogs","Disk");
$olist[] = new onstatoption("checkpoints","checkpoints", "rlogs","Disk");
$olist[] = new onstatoption("backup","backup", "onstat","Disk");
$olist[] = new onstatoption("mem","mem", "onstat","Mem");
$olist[] = new onstatoption("seg","seg", "onstat","Mem");
$olist[] = new onstatoption("net","net", "onstat","Net");
$olist[] = new onstatoption("ppf","ppf", "onstat","Tables");
$olist[] = new onstatoption("config","config","onstat","Perf");
$olist[] = new onstatoption("ses","ses","onstat","User");
$olist[] = new onstatoption("sqlcache","sqlcache", "onstat","Perf");
$olist[] = new onstatoption("showByType","showByType", "sqltraceforreports","Sql");
$olist[] = new onstatoption("showslowSQL","showslowSQL","sqltraceforreports","SQL");
$olist[] = new onstatoption("showmostIO","showmostIO","sqltraceforreports","SQL");
$olist[] = new onstatoption("showmostBuff","showmostBuff","sqltraceforreports","SQL");
$olist[] = new onstatoption("glo","glo", "onstat","Perf");
$olist[] = new onstatoption("showdbtab","showdbtab", "sqlwin","SQL");
$olist[] = new onstatoption("showComputerInfo","compinfo","onstat","Server");
$olist[] = new onstatoption("showComputerResource","rusage","onstat","Server");
$olist[] = new onstatoption("showWaitingSessions","waitses","onstat","User");
/**
 * User specific report list
 */
/*global $oulist;
$oulist = array( );
$oulist[] = new onstatoption("ses","ses","onstat","Basic Session Information",
"List detials of user session.",
"Basic");
$oulist[] = new onstatoption("showSumByUser","showSumByUser","sqltraceforreports",
"SQL Summary",
"Summary of recent SQL run by user.",
"SQL");
$oulist[] = new onstatoption("showslowSQL","showslowSQL","sqltraceforreports",
"Slowest SQL Statements",
"Show the slowest SQL statements by this User.",
"SQL");
$oulist[] = new onstatoption("showmostIO","showmostIO","sqltraceforreports",
"SQL with the most IO time",
"Show the SQL statements consuming the most IO time.",
"SQL");
$oulist[] = new onstatoption("showmostBuff","showmostBuff","sqltraceforreports",
"SQL with the most Buffer Activity",
"Show the SQL statements with the most bufer pool activity.",
"SQL");
*/
class onstat {

    public $idsadmin;

    /**
     * the 'constructor' function
     * called when the class "new'd"
     */
    function __construct(&$idsadmin)
    {
        $this->idsadmin = &$idsadmin;
        $this->idsadmin->load_lang("onstat");
        $this->idsadmin->load_lang("onconfig");
    }


    /**
     * the run function
     * this is what index.php will call
     * the decision of what to actually do is based on
     * the value of 'do' which is either posted or getted
     */
    function run()
    {
        $this->idsadmin->html->set_pagetitle($this->idsadmin->lang('mainTitle'));

        switch($this->idsadmin->in['do'])
        {
            case 'glo':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("VPReport"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showVPList( );
                break;
            case 'net':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("NetworkUsage"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showNetUser( );
                break;
            case 'config':
                $this->idsadmin->setCurrMenuItem("onconfigparam");
                $this->showOnconfig( );
                break;
            case 'config_details':
                $this->idsadmin->set_redirect("config");
                $this->idsadmin->setCurrMenuItem("onconfigparam");
                $this->onconfigDetails( $this->idsadmin->in['param_id'] );
                break;
            case 'save_config':
                $this->idsadmin->set_redirect("config");
                $this->idsadmin->setCurrMenuItem("onconfigparam");
                if (isset($this->idsadmin->in['save']))
                {
                    $this->saveOnconfigParam();
                }
                else
                {  // for cancel
                    $this->showOnconfig();
                }
                break;
            case 'mempool':
            case 'mem':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("MemoryPools"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showpoollst();
                break;
            case 'seg':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("MemoryUsage"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showseglist();
                break;
            case 'backup':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("SysBackups"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showbackups();
                break;
            case 'ses':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("Sessions"));
                $this->idsadmin->setCurrMenuItem("Reports");
                if (empty($this->idsadmin->in['sid']) )
                {
                    $this->showSessions();
                }
                else
                {
                    $this->showUserSessions($this->idsadmin->in['sid']);
                }
                break;
            case 'sqlcache':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("SQLCache"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showsqlcache();
                break;
            case 'ppf':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("TableProf"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showTableProfile();
                break;
            case 'reports':
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("SystemReports"));
                $this->ReportList();
                break;
            case 'runreports':
                $this->ReportRun();
                break;
            /*case 'userreports':
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->ReportListUser();
                break;
            case 'runuserreports':
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->ReportRunUser();
                break;*/
            case 'compinfo':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("CompInfo"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showComputerInfo();
                break;
            case 'rusage':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("ResourceUsage"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showResourceHistory();
                break;
            case 'waitses':
                $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("WaitSesRpt"));
                $this->idsadmin->setCurrMenuItem("Reports");
                $this->showWaitingSessions();
                break;
            default:
                $this->idsadmin->error($this->idsadmin->lang('sorry'));
                break;
        }
    }

    /**
     * Display all the system reports and let the user build their
     * own report.
     * 1. Setup a set of report groups
     * 2. Display a list of system reports from the
     *    array $olistgroup
     * To add another report just add the report to the olistgroup
     * and everything else is done for you.
     *
     */
    function ReportList()
    {
        global $olist;
        if ( isset($this->idsadmin->in['reporttype']) )
        onstatgroups::$cur_group = $this->idsadmin->in['reporttype'];
        else
        onstatgroups::$cur_group = "Clear All";


        $olistgroup = array();
        $olistgroup[] = new onstatgroups("Clear All","{$this->idsadmin->lang('ClearAll')}");
        $olistgroup[] = new onstatgroups("All","{$this->idsadmin->lang('AllRept')}");
        $olistgroup[] = new onstatgroups("Disk","{$this->idsadmin->lang('Disk')}");
        $olistgroup[] = new onstatgroups("Perf","{$this->idsadmin->lang('Performance')}");
        $olistgroup[] = new onstatgroups("Net","{$this->idsadmin->lang('Network')}");
        $olistgroup[] = new onstatgroups("SQL","{$this->idsadmin->lang('SQL')}");
        $olistgroup[] = new onstatgroups("Tables","{$this->idsadmin->lang('Tables')}");
        $olistgroup[] = new onstatgroups("User","{$this->idsadmin->lang('Users')}");
        $olistgroup[] = new onstatgroups("Server","{$this->idsadmin->lang('Server')}");

        $html=<<<END
<TABLE class="gentab_nolines">
<TR>
<td class="tblheader" colspan="4" align="center">{$this->idsadmin->lang('SystemReports')}</td>
</TR>
<TR>
  <TD align="center" colspan=4>
  <FORM METHOD="get" action="index.php" NAME="groupname">
  {$this->idsadmin->lang('SelectReportType')} 
  <INPUT type=hidden name="act" value="onstat" />
  <INPUT type=hidden name="do"  value="reports" />
  <select name="reporttype" onchange="groupname.submit()">
END;
        foreach( $olistgroup as $val )
        {
            $html.=<<<END
            <option value="{$val->getGname()}" {$val->equalGroup($val->getGname(),"SELECTED")} >{$val->getDesc()}</option>

END;
        }
        $html.=<<<END
  </select>
  </FORM>
 </TD>
</TR>
</TABLE>
END;

        $html.='<FORM METHOD="post" action="index.php" name="reporter"><TABLE class="gentab_nolines">';

        $start=true;
        foreach( $olist as $val )
        {
            if ($start)
            $html.="<TR>";

            $html.=<<<END

            <TD align="right" >
            <INPUT TYPE="checkbox" NAME="{$val->getOption()}" {$val->getGrpStatus()} />
            </TD>
            <TD align="left">
            <A HREF="index.php?act={$val->getCname()}&amp;do={$val->getCmd()}" title="{$this->idsadmin->lang($val->getOption() . "_tooltip")}">{$this->idsadmin->lang($val->getOption())}</A>
    </TD>
END;
            if ($start)
            $start=false;
            else
            {
                $start=true;
                $html.="</TR>";
            }
        }
        if ($start)
        $html.="</TR>";

        $html.=<<<END


        <TR>
        <TD align="center" colspan="4">
        <INPUT type="hidden" name="act" value="onstat"/>
        <INPUT type="hidden" name="do"  value="runreports"/>
        <INPUT type="hidden" name="fullrpt"  value="true"/>
        <INPUT type="submit" class="button" name="runReports" value="{$this->idsadmin->lang('CreateReport')}"/>
		</TD>
		</TR>
</TABLE>
</FORM>
END;
        $this->idsadmin->html->add_to_output( $html );

    } # end function

    /**
     * Run all the system reports that we requested
     * The list of reports are stored under app->in
     *
     */
    function ReportRun()
    {
        global $olist;
        $db = $this->idsadmin->get_database("sysmaster");
        $html="";
        /*
         * Do the list of reports
         */
        $stmt = $db->query(
        " select " .
        " dbinfo('UTC_TO_DATETIME', sh_boottime) as boottime, " .
        " dbinfo('UTC_TO_DATETIME', sh_curtime) as curtime, " .
        " (sh_curtime - sh_boottime) as uptime " .
        " from sysshmvals"
        );
        $res = $stmt->fetch();
        $res['UPTIME']=$this->idsadmin->timedays($res['UPTIME']);

        $html.=<<<END
        <html>
        <head></head>
        <body style="height:100% ; width:100%;" >
        <TABLE WIDTH="100%" border="0">
        <TR>
        <TD>
        <TABLE border="0">
        <TR>
        <TH> <A NAME='ReportTop'>{$this->idsadmin->lang('ReportRun')}</A> </TH>
        <TD> {$res['CURTIME']} </TD>
        </TR>
        </TABLE>
        <TD align="right">
        <TABLE border="0">
        <TR>
        <TH>{$this->idsadmin->lang('ServerUptime')}</TH>
        <TD> {$res['UPTIME']} </TD>
           </TR>
          </TABLE>
        </TD>
      </TR>
      </TABLE>

END;
        $html.="<OL>";
        foreach( $olist as $val )
        {
            if ( isset( $this->idsadmin->in[ $val->getOption() ] ) )
            {
                $tname = $this->idsadmin->lang($val->getOption());
                $html.="<LI><A HREF=\"#{$tname}\">{$tname}</A></LI>";
            }
        }
        $html.="</OL>";
        $this->idsadmin->html->add_to_output( $html );

        /*
         * Run each report requested
         */
        foreach( $olist as $val )
        {
            if ( isset( $this->idsadmin->in[ $val->getOption() ] ) )
            {
                $cname = $val->getCname();
                if ( strcasecmp( $cname, "onstat") == 0 )
                $obj=$this;
                else
                {
                    require_once ROOT_PATH."modules/". $val->getCname().".php";
                    $this->idsadmin->html->debug($cname);
                    $obj = new $cname(&$this->idsadmin);
                }
                $html="<H2><A NAME='{$this->idsadmin->lang($val->getOption())}'>{$this->idsadmin->lang($val->getOption())}</A></H2><BR>";
                $this->idsadmin->html->add_to_output( $html );
                $this->idsadmin->in['do'] = $val->getCmd();
                $obj->run();
                $html="<h3 align='right'><A HREF=\"#ReportTop\">{$this->idsadmin->lang('Top')}</A></h3>";
                $this->idsadmin->html->add_to_output( $html );
            }
        }
        $this->idsadmin->html->add_to_output("</body></html>");
        return;
    }

    /**
     * Display a list of users and a list of user reports
     * Let the users build their own reports
     *
     * To add a user defined report just add the report
     * to the $oulist array.  The user's sid will be passed
     * in $this->idsadmin->in['sid'].  reports must validate that the user
     * has not logged off since being asked to run the
     * report
     *
     */
    function ReportListUser()
    {
        global $oulist;
        $db = $this->idsadmin->get_database("sysmaster");
        /* get the current SID */
        if ( isset($this->idsadmin->in['sid']) )
        {
            $sid = $this->idsadmin->in['sid'];
        }
        else
        {
            $stmt = $db->query(
            "select first 1 sid as sid, " .
            " trim(username)||'('||sid||')' as name " .
            " from sysscblst " .
            " where sid != dbinfo('sessionid') and connected > 0 order by 2, 1 DESC" );
            $res = $stmt->fetch();
            $sid = $res['SID'];
        }

        if ( isset($this->idsadmin->in['reporttype']) )
        onstatgroups::$cur_group = $this->idsadmin->in['reporttype'];
        else
        onstatgroups::$cur_group = "Clear All";

        $olistgroup = array();
        $olistgroup[] = new onstatgroups("Clear All","{$this->idsadmin->lang('ClearAll')}");
        $olistgroup[] = new onstatgroups("All"," {$this->idsadmin->lang('AllRept')}");
        $olistgroup[] = new onstatgroups("Basic"," {$this->idsadmin->lang('Basic')}");
        $olistgroup[] = new onstatgroups("SQL"," {$this->idsadmin->lang('SQL')}");


        $html=<<<END
<TABLE class="gentab_nolines">
<TR>
<td class="tblheader" colspan="4" align="center"> User Reports </td>
</TR>
<TR>
  <TD align="center" colspan=2>
  <FORM METHOD="get" action="index.php" NAME="uinfo">
   Select Report Type
  <INPUT type=hidden name="act" value="onstat"/>
  <INPUT type=hidden name="do"  value="userreports"/>
  <select name="reporttype" onchange="uinfo.submit()">
END;


        foreach( $olistgroup as $index => $val )
        {
            $html.=<<<END
            <option value="{$val->getGname()}"
END;
            $html.=onstatgroups::equalGroup($val->getGname(),"SELECTED");
            $html.=<<<END
            >{$val->getDesc()}</option>

END;
        }
        $html.=<<<END
  </select>
  </TD>
  <TD colspan=2 align="center">
  User
END;


        $html.=$this->idsadmin->html->autoSelectList(
        "uinfo",
        "sid",
        "select sid as sid, trim(username)||'('||sid||')' as name " .
        "from sysscblst where sid != dbinfo('sessionid') and connected > 0 order by 2, 1 DESC",
        "sysmaster",
        $sid
        );
        $html.=<<<END
</FORM>
 </TD>
</TR>
END;

        $html.='<FORM METHOD="post" action="index.php">';
        $start=true;
        foreach( $oulist as $val )
        {
            if ($start)
            $html.="<TR>";
            $html.=<<<END

            <TD align="right" >
            <INPUT TYPE="checkbox" NAME="{$val->getOption()}"
END;
            $html.=onstatgroups::getMatchGroup($val->getGroup());
            $html.=<<<END
            />

            </TD>
            <TD align="Left">
            <A HREF="index.php?act={$val->getCname()}&amp;do={$val->getCmd()}&amp;sid={$sid}"  title="{$val->getDesc()}">{$val->getTitle()}</A>
    </TD>
END;
            if ($start)
            $start=false;
            else
            {
                $start=true;
                $html.="</TR>";
            }
        }
        if ($start)
        $html.="</TR>";
        $html.=<<<END


        <TR>
        <TD align="center" colspan=4>
        <INPUT type=hidden name="act" value="onstat"/>
        <INPUT type=hidden name="do"  value="runuserreports"/>
        <INPUT type=hidden name="sid"  value="{$sid}"/>
        <INPUT type=hidden name="fullrpt"  value="true"/>
        <INPUT type=submit class=button name=runReports value="{$this->idsadmin->lang('CreateReport')}"/>
   </FORM>
</TD>
</TR>
</TABLE>
END;
        /***
        <INPUT type=submit class=button name=runReports value="Save"/>
        ***/
        $this->idsadmin->html->add_to_output( $html );

    } # end function

    /**
     * Run the list of user defined reports
     *
     */
    function ReportRunUser()
    {
        global $oulist;
        $db = $this->idsadmin->get_database("sysmaster");
        $html="";
        /*
         * Do the list of reports
         */
        $stmt = $db->query(
        " select " .
        " dbinfo('UTC_TO_DATETIME', sh_boottime) as boottime, " .
        " dbinfo('UTC_TO_DATETIME', sh_curtime) as curtime, " .
        " (sh_curtime - sh_boottime) as uptime " .
        " from sysshmvals"
        );
        $res = $stmt->fetch();
        $res['UPTIME']=$this->idsadmin->timedays($res['UPTIME']);

        $html.=<<<END
        <TABLE WIDTH="100%" border=0>
        <TR>
        <TD>
        <TABLE border=0>
        <TH> <A NAME='ReportTop'>{$this->idsadmin->lang('ReportRun')}</A> </TH>
        <TD> {$res['CURTIME']} </TD>
        </TABLE>
        <TD align="right">
        <TABLE border=0>
        <TH>{$this->idsadmin->lang('ServerUptime')}</TH>
        <TD>{$res['UPTIME']}</TD>
          </TABLE>
        </TD>
      </TR>
      </TABLE>

END;
        $html.="<OL>";


        /*
         * Run each report requested
         */
        foreach( $oulist as $index => $val )
        {
            if ( isset( $this->idsadmin->in[ $val->getOption() ] ) )
            {
                $tname = $val->getTitle();
                $html.="<LI><A HREF=\"#{$tname}\">{$tname}</A></LI>";
            }
        }
        $html.="</OL>";
        $this->idsadmin->html->add_to_output( $html );


        foreach( $oulist as $val )
        {
            if ( isset( $this->idsadmin->in[ $val->getOption() ] ) )
            {
                $cname = $val->getCname();
                if ( strcasecmp( $cname, "onstat") == 0 )
                $obj=$this;
                else
                {
                    require_once ROOT_PATH."modules/". $val->getCname().".php";
                    $obj = new $cname(&$this->idsadmin);
                }
                $html="<H2 align='left'>
                <A NAME='{$val->getTitle()}'>{$val->getTitle()}</A></H2><BR>";
                $this->idsadmin->html->add_to_output( $html );
                $this->idsadmin->in['do'] = $val->getCmd();
                $obj->run();
                $html="<h3 align='right'><A HREF=\"#ReportTop\">{$this->idsadmin->lang('Top')}</A></h3>";
                $this->idsadmin->html->add_to_output( $html );
            }
        }
        return;
    }

    /**
     * Main report for the Virtual Processor report.
     * 1.  Displays a graph of the Virtual Processors
     * 2.  Display a table list of the VP information
     *
     */
    function showVPList( )
    {
        $this->idsadmin->html->add_to_output( "<TABLE  style='width:100%;height:100%'><TR   style='width:100%;height:50%'><TD ALIGN=center>" );
        $this->showVPGraph();
        $this->idsadmin->html->add_to_output( "</TD></TR><TR><TD>" );
        $this->showVPTable();
        $this->idsadmin->html->add_to_output( "</TD></TABLE>" );

    } #end default

    /**
     * Show a tabular format of the Virtual Processor Information
     *
     */
    function showVPTable( )
    {
        require_once ROOT_PATH."lib/gentab.php";

        $tab = new gentab(&$this->idsadmin);

        /* Individual user information for the table */
        $qry = "SELECT ".
        "A.vpid, " .
        "A.pid, " .
        "B.txt, " .
        "A.usecs_user, ".
        "A.usecs_sys, ".
        "A.num_ready ".
        "from sysvplst A, flags_text B   " .
        "WHERE  B.tabname = 'sysvplst' " .
        " AND  a.class = B.flags " .
        " order by vpid" ;

        $qrycnt = "SELECT count(*) from sysvplst";

        $tab->display_tab_by_page($this->idsadmin->lang('NetworkUsage'),
        array(
        "1" => $this->idsadmin->lang('VPID'),
        "2" => $this->idsadmin->lang('PPID'),
        "3" => $this->idsadmin->lang('Class'),
        "4" => $this->idsadmin->lang('UserTime'),
        "5" => $this->idsadmin->lang('SystemTime'),
        "6" => $this->idsadmin->lang('ReadyQ'),
        ),
        $qry, $qrycnt, 10, "template_gentab_order.php");

    } #end

    /**
     * Show a Graph of virtual process by class
     *
     */
    function showVPGraph( )
    {

        $db = $this->idsadmin->get_database("sysmaster");

        require_once ROOT_PATH."lib/Charts.php";
        // $mygraph = new idsgraphs(&$this->idsadmin);

        $sql ="SELECT sum(usecs_sys + usecs_user) as time , " .
        " B.txt  " .
        " FROM sysvplst A, flags_text B   " .
        " WHERE  B.tabname = 'sysvplst' " .
        " AND  a.class = B.flags " .
        " GROUP by B.txt " ;

        $stmt = $db->query($sql);
        $udata = array( );
        while ($res = $stmt->fetch())
        {
            $udata[  $res['TXT']  ] = $res[ 'TIME' ];
        }

        $this->idsadmin->Charts = new Charts($this->idsadmin);
        $this->idsadmin->Charts->setType("PIE");
        $this->idsadmin->Charts->setData($udata);
        $this->idsadmin->Charts->setTitle($this->idsadmin->lang('VP'));
        $this->idsadmin->Charts->setLegendDir("vertical");
        $this->idsadmin->Charts->setWidth("100%");
        $this->idsadmin->Charts->setHeight("250");
        $this->idsadmin->Charts->Render();

        //$this->idsadmin->html->add_to_output( $mygraph->pieGraph( $udata,
        //    $this->idsadmin->lang('VP'),300,300,true) );
    }

    /**
     * Display the network user list in both a Graphical
     * view and a tabular view.
     *
     */
    function showNetUser( )
    {
        $this->idsadmin->html->add_to_output( "<TABLE style='width:100%;height:100%'><TR   style='width:100%;height:50%'><TD ALIGN=center valign='top'>" );
        $this->showNetUserGraph( );
        $this->idsadmin->html->add_to_output( "</TD></TR><TR><TD>" );
        $this->showNetUserTable( );
        $this->idsadmin->html->add_to_output( "</TD></TABLE>" );
    } #end default

    /**
     * Display the network user list in a tabular view.
     *
     */
    function showNetUserTable( )
    {

        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        /* Individual user information for the table */
        $qry = "SELECT ".
        "net_id, " .
        "sid, " .
        "net_client_name, ".
        "net_read_cnt as reads, ".
        "net_write_cnt as writes, ".
        "format_units(net_read_bytes,'b') as readbytes, ".
        "format_units(net_write_bytes,'b') as writebytes, ".
        "dbinfo('UTC_TO_DATETIME', net_open_time) as connect, " .
        "net_protocol, ".
        "net_write_cnt + net_read_cnt as netio ".
        "from sysnetworkio  " .
        " order by netio DESC" ;

        $qrycnt = "SELECT count(*) from sysnetworkio";

        $tab->display_tab_by_page($this->idsadmin->lang('NetUserRep'),
        array(
        "1" => $this->idsadmin->lang('NetID'),
        "2" => $this->idsadmin->lang('SID'),
        "3" => $this->idsadmin->lang('ClientName'),
        "4" => $this->idsadmin->lang('ReadCount'),
        "5" => $this->idsadmin->lang('WriteCount'),
        "6" => $this->idsadmin->lang('ReadBytes'),
        "7" => $this->idsadmin->lang('WriteBytes'),
        "8" => $this->idsadmin->lang('ConTime'),
        "9" => $this->idsadmin->lang('Protocol'),
        ),
        $qry, $qrycnt , 10, "template_gentab_order.php");

    } #end


    /**
     * Display a network system graph
     *
     */
    function showNetUserGraph( )
    {
        $db = $this->idsadmin->get_database("sysmaster");

        require_once ROOT_PATH."lib/Charts.php";

        //$mygraph = new idsgraphs(&$this->idsadmin);

        $sql ="SELECT ng_writes as writes, ng_reads as reads, " .
        "ng_his_writes_bytes as write_data, " .
        "ng_his_reads_bytes as read_data, " .
        " ng_connects AS CONNECT " .
        " FROM sysnetglobal" ;

        $stmt = $db->query($sql);
        $res = $stmt->fetch();
        $stmt->closeCursor();

        $udata = array(
        "Sends" => (int)$res['WRITES'],
        "Receives" => (int)$res['READS'],
        "Connects" => (int)$res['CONNECT'],
        );
        $udata1 = array(
        "Sends" => (int)$res['WRITE_DATA'],
        "Receives"  => (int)$res['READ_DATA'],
        );

        $sql ="SELECT "  .
        " sum(net_write_bytes) as writes, " .
        " sum(net_read_bytes) as reads " .
        " FROM sysnetworkio" ;

        $stmt = $db->query($sql);
        $res = $stmt->fetch();
        $stmt->closeCursor();

        $udata1["Sends"] += (int)$res['WRITES'];
        $udata1["Receives"] += (int)$res['READS'];

        $this->idsadmin->Charts = new Charts($this->idsadmin);
        $this->idsadmin->Charts->setType("PIE");
        $this->idsadmin->Charts->setData($udata);
        $this->idsadmin->Charts->setTitle($this->idsadmin->lang('SysNetIO'));
        $this->idsadmin->Charts->setLegendDir("vertical");
        $this->idsadmin->Charts->setWidth("100%");
        $this->idsadmin->Charts->setHeight("150");
        $this->idsadmin->Charts->setId("networkIOGraph");

        $this->idsadmin->Charts->Render();


        //$this->idsadmin->html->add_to_output("</br>HERE</br>");
        //$this->idsadmin->html->add_to_output( $mygraph->pieGraph( $udata,
        //$this->idsadmin->lang('SysNetIO'),290,220,true) );

        /*
         *
         */
        $this->idsadmin->Charts = new Charts($this->idsadmin);
        $this->idsadmin->Charts->setType("PIE");
        $this->idsadmin->Charts->setData($udata1);
        $this->idsadmin->Charts->setTitle($this->idsadmin->lang('SysNetTXF'));
        $this->idsadmin->Charts->setLegendDir("vertical");
        $this->idsadmin->Charts->setWidth("100%");
        $this->idsadmin->Charts->setHeight("150");
        $this->idsadmin->Charts->Render();


        //$this->idsadmin->html->add_to_output( $mygraph->pieGraph( $udata1,
        // $this->idsadmin->lang('SysNetTXF'),290,220,true) );

    } #end

    /**
     * This function display the oncofig file (i.e. centeral
     * configuration file.
     *
     */
    function showOnconfig()
    {
        require_once ROOT_PATH."lib/onconfig.php";

        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $this->idsadmin->html->set_pagetitle($this->idsadmin->lang('ServerConfig'));

        $dynamic_list = "'" . implode("','", onconfig::get_dynamic_onconfig_params($this->idsadmin)) . "'";

        $allOption = "SELECTED";
        $dynamicOption = "";
        $recommendOption = "";
        if (isset($this->idsadmin->in['show']) && strcmp($this->idsadmin->in['show'],'ShowDynamic')==0)
        {
            // "ShowDynamic"
            $allOption = "";
            $dynamicOption = "SELECTED";
            $where_clause = " where cf_name in ( $dynamic_list )";
        }
        elseif (isset($this->idsadmin->in['show']) && strcmp($this->idsadmin->in['show'],'ShowRecommend')==0)
        {
            // "ShowRecommend"
            $allOption = "";
            $recommendOption = "SELECTED";
            // If querying for only onconfig parameters recommendations,
            // which are not being followed, append that onto the query's where clause.
            $recommend_list = $this->getOnconfigRecommendNonComplianceList();
            if (count($recommend_list) == 0) {
                // no non-compliant recommendations, so make query return no rows
                $recommend_list = null;
                $where_clause = " where cf_id in ( -1 )";
            } else {
                $recommend_list = implode(",",$recommend_list);
                $where_clause = " where cf_id in ( $recommend_list )";
            }
        } else {
            // "ShowAll"
            // We will display all onconfig parameters where cf_flags=0.
            // For the onconfig parameters where cf_flags=1 (unsupported), we will display
            // them only if they do not follow OAT recommendation.  So will need to add
            // to the where clause the cf_flags=1 parameters that we want to view.
            $where_clause = " where cf_flags=0";
            $recommend_list = $this->getOnconfigUnsupportedList();
            if (count($recommend_list) != 0) {
                $recommend_list = implode(",",$recommend_list);
                $where_clause .= " OR cf_id in ( $recommend_list )";
            }
        }

        // If not in report mode, give user a drop-down box with
        // options to view all onconfig parameters or only dynamic ones
        if (!isset($this->idsadmin->in["runReports"]))
        {
            $HTML=<<<END
            <TABLE WIDTH=100% BORDER=0>
            <TR>
            <TD align="center" colspan=4>
            <FORM METHOD="get" action="index.php" NAME="onconfigoption">
            <B>{$this->idsadmin->lang("OnconfigOption")}</B>
            <INPUT type=hidden name="act" value="onstat" />
            <INPUT type=hidden name="do"  value="config" />
            <select name="show" onchange="onconfigoption.submit()">
            <option value="ShowAll" $allOption>{$this->idsadmin->lang("ShowAll")}</option>
            <option value="ShowDynamic" $dynamicOption>{$this->idsadmin->lang("ShowDynamic")}</option>
            <option value="ShowRecommend" $recommendOption>{$this->idsadmin->lang("ShowRecommend")}</option>
            </select>
            </FORM>
            {$this->idsadmin->lang("RecommendationMessage")}
</TR>
END;
            $this->idsadmin->html->add_to_output($HTML);
        }

        $qry =  "select " .
        	" trim(cf_name) as name, " .
        	" trim(cf_effective) as effective, " .
        	" case " .
        	"   when (cf_name in ($dynamic_list))" .
        	"     then 'Dynamic' ".
        	"   else " .
        	"     'NotDynamic' " .
        	" end as configurable, " .
        	" cf_id as id " .
        	" from syscfgtab $where_clause" .
        	" order by name";

        $qrycnt = "SELECT count(*) from syscfgtab $where_clause";

        // If in report mode, use generic template.
        // Otherwise use template_gentab_onconfig to create dynamic
        // links for each onconfig parameter name.
        if (isset($this->idsadmin->in["runReports"]))
        {
            $template = "template_gentab_order.php";
        } else {
            $template ="template_gentab_onconfig.php";
        }

        $tab->display_tab_by_page($this->idsadmin->lang("OnconfigFile"),
        array(
        "name" => $this->idsadmin->lang("ParamName"),
        "effective" => $this->idsadmin->lang("Value"),
        "configurable" => $this->idsadmin->lang("Configurable")
        ),
        $qry,
        $qrycnt,
        20,
        $template);

    } #end

    /**
     * This function returns a list of parameter ids for those
     * onconfig parameters that do not comply with OAT's recommendations.
     */
    function getOnconfigRecommendNonComplianceList()
    {
        require_once ROOT_PATH."lib/onconfig_param.php";
        $recommend_list = Array();
        $dbsysmaster= $this->idsadmin->get_database("sysmaster");
        $qry = "SELECT cf_id as id, "
        ." cf_name as name, "
        ." cf_effective as value"
        ." FROM syscfgtab";
        $stmt = $dbsysmaster->query( $qry );
        while ($res = $stmt->fetch(PDO::FETCH_ASSOC) )
        {
            $onconfig_param = new onconfig_param($res['ID'],$res['NAME'],$res['VALUE'],$this->idsadmin);
            $compliance = $onconfig_param->checkRecommendation();
            if (!is_null($compliance) && !$compliance) {
                $recommend_list[] = $res['ID'];
            }
        }
        return $recommend_list;
    }

    /**
     * For unsupported onconfig parameters (cf_flags=1), we will only display
     * them if they do not comply with OAT's recommendations.
     *
     * This function returns a list of parameter ids for those onconfig
     * parameters with cf_flags=1 AND that do not comply with OAT's recommendations.
     */
    function getOnconfigUnsupportedList()
    {
        require_once ROOT_PATH."lib/onconfig_param.php";
        $recommend_list = Array();
        $dbsysmaster= $this->idsadmin->get_database("sysmaster");
        $qry = "SELECT cf_id as id, "
        ." cf_name as name, "
        ." cf_effective as value"
        ." FROM syscfgtab where cf_flags=1";
        $stmt = $dbsysmaster->query( $qry );
        while ($res = $stmt->fetch(PDO::FETCH_ASSOC) )
        {
            $onconfig_param = new onconfig_param($res['ID'],$res['NAME'],$res['VALUE'],$this->idsadmin);
            $compliance = $onconfig_param->checkRecommendation();
            if (!is_null($compliance) && !$compliance) {
                $recommend_list[] = $res['ID'];
            }
        }
        return $recommend_list;
    }


    /**
     * This function displays the details of an onconfig parameter.
     * If it is a dyanamic onconfig parameter, this page will also
     * allow the user to edit the value.
     */
    function onconfigDetails($param_id)
    {
        require_once ROOT_PATH."lib/onconfig_param.php";

        $this->idsadmin->html->set_pagetitle($this->idsadmin->lang('OnconfigParamDetails'));

        // if $param_id is not set, redirect to show onconfig
        // NOTE: this will not occur if user manually types "act=onstat&do=config_details"
        // in URL without the param_id=$param_id
        if (! isset($param_id))
        $this->idsadmin->html->add_to_output($this->idsadmin->template['template_global']->
        global_redirect("Onconfig","index.php?act=onstat&do=config"));

        $readonly = "";
        $disabled = "";
        $dynamic_buttons = "<INPUT type=submit class=button name=save value='{$this->idsadmin->lang("Save")}'/>" .
           "<INPUT type=button class=button name=cancel value='{$this->idsadmin->lang("Cancel")}' onClick='history.back()'/>";

        if ( $this->idsadmin->isreadonly() )
        {
            $readonly="READONLY";
            $disabled="DISABLED";
            $dynamic_buttons="<INPUT type=button class=button name=cancel value='{$this->idsadmin->lang("Back")}' onClick='history.back()'/>";
        }

        $dbsysmaster= $this->idsadmin->get_database("sysmaster");

        $qry = "SELECT cf_id as id, "
        ." cf_name as name, "
        ." cf_effective as effective"
        ." FROM syscfgtab"
        ." WHERE cf_id='$param_id'";

        $stmt = $dbsysmaster->query( $qry );
        if (($res = $stmt->fetch(PDO::FETCH_ASSOC))==false)
        {
            $this->idsadmin->error("{$this->idsadmin->lang('OnconfigParamError')} {$param_id}");
            return;
        }

        // create config_param object to represent this parameter
        $param_name = trim($res['NAME']);
        $onconfigParam = new onconfig_param(
        $param_id,
        $param_name,
        $res['EFFECTIVE'],
        $this->idsadmin);

        // If save of onconfig parameter value has failed because
        // the user entered an invalid value, redisplay the value
        // entered by user, instead of the one in the database
        if (isset($this->idsadmin->in['param_value']))
        {
            $onconfigParam->setValue($this->idsadmin->in['param_value']);
            if (isset($this->idsadmin->in['save_option']) &&
            strcmp($this->idsadmin->in['save_option'], 'wf') == 0)
            {
                $wf="SELECTED";
            }
        }

        // Retreive and pass along any URL options that indicate the table position in the onconfig
        // table before drilling down, so that we can return the user to the same position.
        $url_options = "";
        if(isset($this->idsadmin->in['show']))
        {
            $url_options .= "&amp;show={$this->idsadmin->in['show']}";
        }
        if(isset($this->idsadmin->in['pos']))
        {
            $url_options .= "&amp;pos={$this->idsadmin->in['pos']}";
        }
        if(isset($this->idsadmin->in['orderby']))
        {
            $url_options .= "&amp;orderby={$this->idsadmin->in['orderby']}";
        }
        if(isset($this->idsadmin->in['orderway']))
        {
            $url_options .= "&amp;orderway={$this->idsadmin->in['orderway']}";
        }
        if(isset($this->idsadmin->in['perpage']))
        {
            $url_options .= "&amp;perpage={$this->idsadmin->in['perpage']}";
        }


        if ($onconfigParam->isDynamic() && !$this->idsadmin->isreadonly())
        {
            $HTML="<FORM METHOD=\"post\" action=\"index.php?act=onstat&do=save_config{$url_options}\">"
            . "<input type=hidden  name=\"param_id\" value=\"$param_id\"/>"
            . "<input type=hidden  name=\"param_name\" value=\"$param_name\"/>";
        } else {
            $HTML="<FORM METHOD=\"post\" action=\"index.php?act=onstat&do=config{$url_options}\">";
        }
        $HTML.=<<<END

        <TABLE class="onconfig" align=center cellpadding=2 cellspacing=5>
        <tr>
        <td class="tblheader" colspan="2" align="center">{$this->idsadmin->lang("OnconfigParamDetails")}</td>
        </tr>
        <TR>
        <TD><B>{$this->idsadmin->lang("ParamName")}</B></TD>
        <TD>$param_name</TD>
        </TR>
        <TR>
        <TD><B>{$this->idsadmin->lang("Description")}</B></TD>
        <TD>{$onconfigParam->getDescription()}</TD>
</TR>
END;
        if (!is_null($onconfigParam->getType()))
        {
            $HTML.="<TR><TD><B>{$this->idsadmin->lang("ParamType")}</B></TD>".
    	    "<TD>{$onconfigParam->getType()}</TD></TR>";
        }
        if (!is_null($onconfigParam->getMin()))
        {
            $HTML.="<TR><TD><B>{$this->idsadmin->lang("MinValue")}</B></TD>".
    	    "<TD>{$onconfigParam->getMin()}</TD></TR>";
        }
        if (!is_null($onconfigParam->getMax()))
        {
            $HTML.="<TR><TD><B>{$this->idsadmin->lang("MaxValue")}</B></TD>".
    	    "<TD>{$onconfigParam->getMax()}</TD></TR>";
        }
        if (!is_null($onconfigParam->getValueSet()))
        {
            $val_list = implode(', ', $onconfigParam->getValueSet());
            $HTML.="<TR><TD><B>{$this->idsadmin->lang("PossibleValues")}</B></TD>".
    	    "<TD>$val_list</TD></TR>";
        }
        $recommend_check = $onconfigParam->checkRecommendation();
        if (!is_null($recommend_check))
        {
            $HTML .= "<TR><TD><B>{$this->idsadmin->lang("Recommendation")}</B></TD>";
            $HTML .= "<TD>";
            $HTML .= "{$onconfigParam->getRecommendation()}<BR>";
            $HTML.="</TD></TR>";
        }

        if ($onconfigParam->isDynamic())
        {
            // For Dynamic Onconfig Params

            $HTML .= <<<END
            <TR>
            <TD><B>{$this->idsadmin->lang("Value")}</B></TD>
<TD>
END;
            if ($onconfigParam->getType() == onconfig::BOOLEAN)
            {
                $off = ($onconfigParam->getValue()==0)? "SELECTED":"";
                $on = ($onconfigParam->getValue()==1)? "SELECTED":"";

                $HTML.=<<<END
                <select $disabled name="param_value">
                <option value="0" $off>0 (OFF)</option>
                <option value="1" $on>1 (ON)</option>
</select>
END;
            }
            else
            {
                $HTML.="<INPUT $readonly type=text name=\"param_value\" value=\"{$onconfigParam->getValue()}\">";
            }
            if (!is_null($recommend_check) && !$recommend_check)
            {
                $HTML .= "<BR><B><I><FONT color = red>" . $this->idsadmin->lang("Warning") . ":</FONT> " .
                $this->idsadmin->lang("Recommendation_NO") . "</I></B>";
            }
            $HTML.="</TD></TR>";

            if (!$this->idsadmin->isreadonly() &&
            !onconfig::is_ER_config_parameter($onconfigParam->getName()))
            {
                $HTML.=<<<END

                <TR>
                <TD><B>{$this->idsadmin->lang("SaveOption")}</B></TD>
                <TD>
                <select name="save_option">
                <option value="wm">{$this->idsadmin->lang("SaveMem")}</option>
                <option value="wf" $wf>{$this->idsadmin->lang("SaveMemFile")}</option>
</select>
</TD>
</TR>
END;
            }
            $HTML.=<<<END
            <TR></TR>
            <TR align=center>
            <TD colspan=2>
            $dynamic_buttons
</TD>
</TR>
END;
        }
        else
        {
            // For non-dynamic onconfig params
            $HTML.=<<<END
            <TR>
            <TD><B>{$this->idsadmin->lang("Value")}</B>
            </TD>
            <TD>{$onconfigParam->getValue()}
END;
            if (!is_null($recommend_check) && !$recommend_check)
            {
                $HTML .= "<BR><B><I><FONT color = red>{$this->idsadmin->lang("Warning")}:</FONT>" .
                          "{$this->idsadmin->lang("Recommendation_NO")}</I></B>";
            }
            $HTML.=<<<END
            </TD></TR>
            <TR>
            <TD colspan=2><B>{$this->idsadmin->lang("ParamNotDynamic")}</B></TD>
            </TR>
            <TR align=center>
            <TD colspan=2>
            <INPUT type=button class=button name=back value='{$this->idsadmin->lang("Back")}' onClick='history.back()'/></TD>
</TR>
END;
        }
        $HTML .= <<<END
</TABLE>
</FORM>

END;
        $this->idsadmin->html->add_to_output($HTML);

    } #end onconfigDetails

    /*
     * This functions saves user modifications of dynamic
     * onconfig parameters
     */
    function saveOnconfigParam()
    {
        if ($this->idsadmin->isreadonly())
        {
            $this->idsadmin->fatal_error("<center>You do not have permission to do this.</center>");
        }

        require_once ROOT_PATH."lib/onconfig_param.php";

        // create onconfig_parameter object with the new value
        $onconfigParam = new onconfig_param(
        $this->idsadmin->in['param_id'],
        $this->idsadmin->in['param_name'],
        $this->idsadmin->in['param_value'],
        $this->idsadmin);

        // Check constraints to make sure the value entered by the user
        // if valid for that onconfig param
        $check = $onconfigParam->checkValue();
        if ($check != "VALID")
        {
            // if check failed, print error and return user to onconfigDetails page
            $this->idsadmin->error($check);
            $this->onconfigDetails($onconfigParam->getId());
            return;
        }

        // Update the value on the server
        $status_msg = "";
        $sql = "";
        $dbadmin= $this->idsadmin->get_database("sysadmin");
        if (onconfig::is_ER_config_parameter($onconfigParam->getName()))
        {
            // For ER onconfig parameters, use the 'cdr change onconfig' command
            $sql = "execute function task('cdr change onconfig', "
            . "'\"{$onconfigParam->getName()} {$onconfigParam->getValue()}\"')";
            $status_msg = "Update {$onconfigParam->getName()}... ";
        } else {
            // For all others, use the 'onmode -wm/wf' command
            $sql = "execute function task('onmode', '{$this->idsadmin->in['save_option']}', " .
    	    	"'{$onconfigParam->getName()}={$onconfigParam->getValue()}')";
        }

        /**
         * Special case: DS_NONPDQ_QUERY_MEM
         *
         * It is a server requirement that DS_TOTAL_MEMORY must be greater than or equal to
         * 4 times the size of DS_NONPDQ_QUERY_MEM.  Therefore, if a user is trying to
         * increase DS_NONPDQ_QUERY_MEM to a value that is more that 1/4 the size of
         * DS_TOTAL_MEMORY, then automatically also increase DS_TOTAL_MEMORY to the
         * necessary value.  The goal is to keep it simple for the user.
         */
        if (strcasecmp($onconfigParam->getName(),"DS_NONPDQ_QUERY_MEM") == 0)
        {
            // Query sysmaster to find the current value of DS_TOTAL_MEMORY
            $dbsysmaster = $this->idsadmin->get_database("sysmaster");
            $qry = "SELECT cf_effective FROM syscfgtab" .
                " WHERE cf_name like 'DS_TOTAL_MEMORY'";
            $stmt = $dbsysmaster->query( $qry );
            while ($res = $stmt->fetch(PDO::FETCH_ASSOC))
            {
                $ds_total_memory = $res['CF_EFFECTIVE'];
            }

            // If DS_TOTAL_MEMORY is less than 4 times the new value for DS_NONPDQ_QUERY_MEM,
            // increase the value of DS_TOTAL_MEMORY first.
            if ($ds_total_memory < (4 * $onconfigParam->getValue()) )
            {
                $ds_total_memory_sql = "execute function task('onmode', " .
                    "'{$this->idsadmin->in['save_option']}', " .
                    "'DS_TOTAL_MEMORY=" . (4 * $onconfigParam->getValue()) . "')";
                $stmt = $dbadmin->query($ds_total_memory_sql);
                $res = $stmt->fetch(PDO::FETCH_ASSOC);
                $status_msg .= $res[''] . "  ";
            }
        }

        $stmt = $dbadmin->query($sql);
        $res = $stmt->fetch(PDO::FETCH_ASSOC);
        $status_msg .= $res[''];
        $this->idsadmin->status($status_msg);
        $stmt->closeCursor();

        // if save successul, return to onconfig list page
        $this->showOnconfig();
    }

    /**
     * Show the memory pools
     *
     */
    function showpoollst()
    {
        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $qrycnt = "SELECT count(*) from syspoollst where po_id>0";
        $tab->display_tab_by_page("{$this->idsadmin->lang('MemPoolList')}",
        array(
        "1" => "{$this->idsadmin->lang('ID')}",
        "2" => "{$this->idsadmin->lang('Name')}",
        "3" => "{$this->idsadmin->lang('Class')}",
        "8" => "{$this->idsadmin->lang('Total')}",
        "9" => "{$this->idsadmin->lang('Free')}",
        "10" => "{$this->idsadmin->lang('Used')}",
        "7" => "{$this->idsadmin->lang('FLAGS')}",
        ),
        "select po_id, " .
        " po_name, ".
        " decode(po_class,1, 'Resident', "  .
        "2, 'Virtual' , "  .
        "3, 'Message' , "  .
        "   'Other' ) as class, ".
        " format_units(po_usedamt+po_freeamt, 'b') as total, ".
        " format_units(po_freeamt, 'b') as free, ".
        " format_units(po_usedamt,'b') as used, ".
        " po_flags, ".
        " po_usedamt + po_freeamt, ".
        " po_freeamt, ".
        " po_usedamt ".
        " from syspoollst " .
        " order by po_name",
        $qrycnt,
        20);

    } #end

    /**
     * Show the memory segment overview in both a graphical
     * view and a tabular view.
     *
     */
    function showseglist( )
    {
        $this->idsadmin->html->add_to_output( "<TABLE style='width:100%;height:100%'><TR  style='width:50%;height:50%'>" );
        $this->idsadmin->html->add_to_output( "<TD ALING=center>" );
        $this->showSegListGraph();

        if ( Feature::isAvailable ( Feature::CHEETAH2, $this->idsadmin )  )
        {
            $this->idsadmin->html->add_to_output( "</TD><TD style='width:50%; height:100%'>" );
            $this->showOSMemGraph();
            $this->idsadmin->html->add_to_output( "</TD></TR><TR><TD COLSPAN=2>" );
        } else {
            $this->idsadmin->html->add_to_output( "</TD></TR><TR><TD>" );
        }
        $this->showSegTable();
        $this->idsadmin->html->add_to_output( "</TD></TABLE>" );

    } #end default

    /**
     * Show a graphical view of the memory segments
     *
     */
    function showSegListGraph( )
    {
        $db = $this->idsadmin->get_database("sysmaster");
        require_once ROOT_PATH."lib/Charts.php";

        //$mygraph = new idsgraphs(&$this->idsadmin);

        $sql ="SELECT sum(seg_blkused) as mUsed , " .
        " sum(seg_blkfree) as mFree, " .
        " decode(seg_class,1, 'Resident', "  .
        "2, 'Virtual' , "  .
        "3, 'Message' , "  .
        "   'Other' ) as class ".
        " FROM sysseglst   " .
        " GROUP by 3" ;

        $stmt = $db->query($sql);
        $udata = array( );
        while ($res = $stmt->fetch())
        {
            $udata[ $res['CLASS'] . " " . $this->idsadmin->lang('Allocated') ] = $res['MUSED'];
            /*
             * Only consider the free portion if we have more than two blocks free
             *   some of the segments like the resident will always have 1 or 2 blocks
             *   free, but we do not need to waste space showing these in the graph.
             */
            if ( $res['MFREE'] > 2 )
            $udata[ $res['CLASS'] . " " . $this->idsadmin->lang('Free') ] = $res['MFREE'];
        }

        $this->idsadmin->Charts = new Charts($this->idsadmin);
        $this->idsadmin->Charts->setType("PIE");
        $this->idsadmin->Charts->setData($udata);
        $this->idsadmin->Charts->setTitle($this->idsadmin->lang('MemSegUsage'));
        $this->idsadmin->Charts->setLegendDir("vertical");
        $this->idsadmin->Charts->setWidth("100%");
        $this->idsadmin->Charts->setHeight("200");
        $this->idsadmin->Charts->Render();


        //$this->idsadmin->html->add_to_output( $mygraph->pieGraph( $udata,
        // $this->idsadmin->lang('MemSegUsage'),600,300,true) );
    }

    function showOSMemGraph()
    {

        $db = $this->idsadmin->get_database("sysmaster");
        require_once ROOT_PATH."lib/Charts.php";

        $sql = "SELECT os_mem_total - os_mem_free as mUsed , "
        . " os_mem_free as mFree "
        . " FROM sysmachineinfo   " ;

        $stmt = $db->query($sql);
        $udata = array( );
        while ($res = $stmt->fetch())
        {
            $udata[ $res['CLASS'] . " " . $this->idsadmin->lang('Allocated') ] = $res['MUSED'];
            $udata[ $res['CLASS'] . " " . $this->idsadmin->lang('Free') ] = $res['MFREE'];
        }

        $this->idsadmin->Charts = new Charts($this->idsadmin);
        $this->idsadmin->Charts->setType("PIE");
        $this->idsadmin->Charts->setData($udata);
        $this->idsadmin->Charts->setTitle($this->idsadmin->lang('MemOSUsage'));
        $this->idsadmin->Charts->setLegendDir("vertical");
        $this->idsadmin->Charts->setWidth("100%");
        $this->idsadmin->Charts->setHeight("200");
        $this->idsadmin->Charts->Render();
    }

    /**
     * Show a tabular view of the shared memory segmetns
     *
     */
    function showSegTable()
    {
        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $qrycnt = "SELECT count(*) from sysseglst";
        $tab->display_tab_by_page("{$this->idsadmin->lang('SharedMemorySegList')}",
        array(
        "1" => "{$this->idsadmin->lang('SegmentType')}",
        "7" => "{$this->idsadmin->lang('Size')}",
        "8" => "{$this->idsadmin->lang('Used')}",
        "9" => "{$this->idsadmin->lang('Free')}",
        "5" => "{$this->idsadmin->lang('Address')}",
        "6" => "{$this->idsadmin->lang('Key')}",
        ),
        "select " .
        " decode(seg_class,1, 'Resident', "  .
        "2, 'Virtual' , "  .
        "3, 'Message' , "  .
        "   'Other' ) as class, ".
        " format_units(seg_size, 'b') as size, ".
        " format_units(seg_blkused,'4') as used, ".
        " format_units(seg_blkfree,'4') as free, ".
        " seg_shmaddr, ".
        " seg_osshmkey, ".
        " seg_size, ".
        " seg_blkused, ".
        " seg_blkfree ".
        " from sysseglst " .
        " order by seg_class",
        $qrycnt,
        20);
    } #end


    /**
     * Show the usage of of the various sql Caches
     *
     */
    function showsqlcache()
    {
        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $qrycnt = "SELECT count(*) from syssqlcacheprof";
        $tab->display_tab_by_page("{$this->idsadmin->lang('SQLCachesList')}",
        array(
        "1" => "{$this->idsadmin->lang('CacheName')}",
        "2" => "{$this->idsadmin->lang('Ratio')}",
        "3" => "{$this->idsadmin->lang('Hits')}",
        "4" => "{$this->idsadmin->lang('Misses')}",
        "5" => "{$this->idsadmin->lang('Removed')}",
        "10" => "{$this->idsadmin->lang('UsedMemory')}",
        "11" => "{$this->idsadmin->lang('FreeMemory')}",
        "8" => "{$this->idsadmin->lang('CacheEntries')}",
        "9" => "{$this->idsadmin->lang('InUse')}",
        ),
        "select " .
        " name, " .
        " TRUNC(decode(hits,-1,0,0,0,100*hits/(hits+misses)),2) " .
        "       ||'%' as rat, " .
        " decode(hits,-1,'0',hits::char(20)) as hits, " .
        " decode(misses,-1,'0',misses::char(20)) as misses, " .
        " decode(removed,-1,'0',removed::char(20)) as removed, " .
        " format_units(mem_used,'b') as used, " .
        " format_units(mem_free,'b') as free, " .
        " total_entries, " .
        " inuse_entries, " .
        " mem_used, " .
        " mem_free " .
        " from syssqlcacheprof " .
        " order by 1",
        $qrycnt,
        20);

    } #end

    /**
     * Show all database user session and basic user inforamtion
     *
     */
    function showSessions(  )
    {
        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $qrycnt = "SELECT count(*) from sysscblst ";

        $qry = "SELECT " .
        "sid, " .
        "trim(username)||'@'||decode(length(hostname),0,'localhost',hostname)::lvarchar as usr, " .
        "pid, " .
        "dbinfo('UTC_TO_DATETIME',connected)::DATETIME MONTH TO SECOND as con, " .
        "format_units(memtotal,'b') as mtotal, " .
        "format_units(memused,'b') as mused, " .
        "memtotal, " .
        "memused " .
        "FROM sysscblst " .
        "ORDER BY 2,4" ;

        $tab->display_tab_by_page("{$this->idsadmin->lang('SessionList')}",
        array(
        "1" => "{$this->idsadmin->lang('SIDShort')}",
        "2" => "{$this->idsadmin->lang('User')}",
        "3" => "{$this->idsadmin->lang('PID')}",
        "4" => "{$this->idsadmin->lang('ConnectTime')}",
        "7" => "{$this->idsadmin->lang('TotalMemory')}",
        "8" => "{$this->idsadmin->lang('UsedMemory')}",
        ),
        $qry,
        $qrycnt,
        15);

    } #end


    /**
     * Show detial user information when give a database seission id
     *
     * @param integer $sid
     */
    function showUserSessions( $sid )
    {
        $db = $this->idsadmin->get_database("sysmaster");
        $qry = "SELECT " .
        "sysscblst.sid, " .
        "trim(sysscblst.username) as user, " .
        "decode(length(hostname),0,'localhost',hostname) as hostname, " .
        "sysscblst.uid, " .
        "sysscblst.gid, " .
        "sysscblst.pid, " .
        "dbinfo('UTC_TO_DATETIME',connected)::DATETIME MONTH TO SECOND as con, " .
        "format_units(memtotal,'b') as mtotal, " .
        "format_units(memused,'b') as mused, " .
        "nreads, " .
        "nwrites, " .
        "nfiles, " .
        "upf_rqlock, " .
        "upf_wtlock, " .
        "upf_deadlk, " .
        "upf_lktouts, " .
        "upf_lgrecs, " .
        "upf_isread, " .
        "upf_iswrite, " .
        "upf_isrwrite, " .
        "upf_isdelete, " .
        "upf_iscommit, " .
        "upf_isrollback, " .
        "upf_longtxs, " .
        "upf_bufreads, " .
        "upf_bufwrites, " .
        "format_units(upf_logspuse,'b') as logspuse, " .
        "format_units(upf_logspmax,'b') as logspmax, " .
        "upf_seqscans, " .
        "upf_totsorts, " .
        "upf_dsksorts, " .
        "upf_totsorts - upf_dsksorts as memsorts, " .
        "format_units(upf_srtspmax,'b') as srtspmax, " .
        "nlocks " .
        "FROM sysscblst , sysrstcb " .
        "WHERE sysscblst.address = sysrstcb.scb " .
        " AND bitval(sysrstcb.flags,'0x80000')>0 " .
        " AND sysscblst.sid = {$sid} ";

        $stmt = $db->query( $qry );
        $res = $stmt->fetch();
        if ( count( $res ) == 0 )
        {
            $this->idsadmin->html->add_to_output( "<H2> No Data </H2>" );
            return;
        }


        $res['HOSTNAME']= substr( $res['HOSTNAME'],0,strrpos($res['HOSTNAME'],'.') );

        $html=<<<END
        <TABLE class="gentab">
        <tr>
        <td class="tblheader" align="center" colspan=8>{$res['USER']}@{$res['HOSTNAME']}</td>
        </tr>

        <TR>
        <TH>{$this->idsadmin->lang('SessionID')}</TH>
        <TH>{$this->idsadmin->lang('UserID')}</TH>
        <TH>{$this->idsadmin->lang('GroupID')}</TH>
        <TH>{$this->idsadmin->lang('PID')}</TH>
        <TH>{$this->idsadmin->lang('Connected')}</TH>
        <TH>{$this->idsadmin->lang('OpenTables')}</TH>
        <TH>{$this->idsadmin->lang('TotalMemory')}</TH>
        <TH>{$this->idsadmin->lang('UsedMemory')}</TH>
        </TR>
        <TR>
        <TD>{$res['SID']}</TD>
        <TD>{$res['UID']}</TD>
        <TD>{$res['GID']}</TD>
        <TD>{$res['PID']}</TD>
        <TD>{$res['CON']}</TD>
        <TD>{$res['NFILES']}</TD>
        <TD>{$res['MTOTAL']}</TD>
        <TD>{$res['MUSED']}</TD>
        </TR>

        <TR>
        <TH>{$this->idsadmin->lang('Locks')}</TH>
        <TH>{$this->idsadmin->lang('LockRequests')}</TH>
        <TH>{$this->idsadmin->lang('LockWaits')}</TH>
        <TH>{$this->idsadmin->lang('DeadLocks')}</TH>
        <TH>{$this->idsadmin->lang('LockTimeouts')}</TH>
        <TH>{$this->idsadmin->lang('LogRecords')}</TH>
        <TH>{$this->idsadmin->lang('LogSpace')}</TH>
        <TH>{$this->idsadmin->lang('MaxLogSpace')}</TH>
        </TR>
        <TR>
        <TD>{$res['NLOCKS']}</TD>
        <TD>{$res['UPF_RQLOCK']}</TD>
        <TD>{$res['UPF_WTLOCK']}</TD>
        <TD>{$res['UPF_DEADLK']}</TD>
        <TD>{$res['UPF_LKTOUTS']}</TD>
        <TD>{$res['UPF_LGRECS']}</TD>
        <TD>{$res['LOGSPUSE']}</TD>
        <TD>{$res['LOGSPMAX']}</TD>
        </TR>

        <TR>
        <TH>{$this->idsadmin->lang('RowsProcessed')}</TH>
        <TH>{$this->idsadmin->lang('RowsInserted')}</TH>
        <TH>{$this->idsadmin->lang('RowsUpdated')}</TH>
        <TH>{$this->idsadmin->lang('RowsDeleted')}</TH>
        <TH>{$this->idsadmin->lang('Commits')}</TH>
        <TH>{$this->idsadmin->lang('Rollbacks')}</TH>
        <TH>{$this->idsadmin->lang('LongTXs')}</TH>
        <TH>{$this->idsadmin->lang('SequentialScans')}</TH>
        </TR>
        <TR>
        <TD>{$res['NREADS']}</TD>
        <TD>{$res['NWRITES']}</TD>
        <TD>{$res['UPF_RQLOCK']}</TD>
        <TD>{$res['UPF_WTLOCK']}</TD>
        <TD>{$res['UPF_DEADLK']}</TD>
        <TD>{$res['UPF_LKTOUTS']}</TD>
        <TD>{$res['UPF_LONGTXS']}</TD>
        <TD>{$res['UPF_SEQSCANS']}</TD>
        </TR>

        <TR>
        <TH>{$this->idsadmin->lang('FGReads')}</TH>
        <TH>{$this->idsadmin->lang('FGWrites')}</TH>
        <TH>{$this->idsadmin->lang('BufferReads')}</TH>
        <TH>{$this->idsadmin->lang('BufferWrites')}</TH>
        <TH>{$this->idsadmin->lang('Sorts')}</TH>
        <TH>{$this->idsadmin->lang('MemorySorts')}</TH>
        <TH>{$this->idsadmin->lang('DiskSorts')}</TH>
        <TH>{$this->idsadmin->lang('LargestSorts')}</TH>
        </TR>
        <TR>
        <TD>{$res['NREADS']}</TD>
        <TD>{$res['NWRITES']}</TD>
        <TD>{$res['UPF_BUFREADS']}</TD>
        <TD>{$res['UPF_BUFWRITES']}</TD>
        <TD>{$res['UPF_TOTSORTS']}</TD>
        <TD>{$res['MEMSORTS']}</TD>
        <TD>{$res['UPF_DSKSORTS']}</TD>
        <TD>{$res['SRTSPMAX']}</TD>
</TR>

</TABLE>

END;
        $this->idsadmin->html->add_to_output( $html );

    } #end


    /**
     * Show the table profile information.
     *
     */
    function showTableProfile(  )
    {
        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $qrycnt = "SELECT count(unique tablock) as cnt " .
        " from sysptntab " .
        " WHERE partnum > 0" ;

        $qry = "SELECT " .
        "trim(dbsname)||'.'||trim(tabname) as name, " .
        "sum(pf_rqlock) as lockreq, " .
        "sum(pf_wtlock + pf_deadlk+pf_lktouts) as lockwaits, " .
        "sum(pf_bfcread) as rdcache, " .
        "sum(pf_bfcwrite) as wrcache, " .
        "sum(pf_dskreads) as dskreads, " .
        "sum(pf_dskwrites) as dskwrites, " .
        "sum(pf_isread)  as isread, " .
        "sum(pf_iswrite) as iswrite, " .
        "sum(pf_isrwrite) as isrwrite, " .
        "sum(pf_isdelete) as isdelete, " .
        "sum(pf_seqscans) as seqscans " .
        "FROM sysptntab, systabnames  " .
        "WHERE sysptntab.tablock = systabnames.partnum " .
        "GROUP BY 1 " .
        "ORDER BY 1" ;

        $tab->display_tab_by_page("{$this->idsadmin->lang('TableProfileInfo')}",
        array(
        "1" => "{$this->idsadmin->lang('TableName')}",
        "2" => "{$this->idsadmin->lang('Locks')}",
        "3" => "{$this->idsadmin->lang('LockWaits')}",
        "4" => "{$this->idsadmin->lang('ReadCache')}",
        "5" => "{$this->idsadmin->lang('WriteCache')}",
        "6" => "{$this->idsadmin->lang('DiskReads')}",
        "7" => "{$this->idsadmin->lang('DiskWrites')}",
        "8" => "{$this->idsadmin->lang('RowsProcessed')}",
        "9" => "{$this->idsadmin->lang('RowsInserted')}",
        "10" => "{$this->idsadmin->lang('RowsUpdated')}",
        "11" => "{$this->idsadmin->lang('RowsDeleted')}",
        "12" => "{$this->idsadmin->lang('SequentialScans')}",
        ),
        $qry,
        $qrycnt,
        15);

    } #end

    /**
     * Show when the last backups have occurred.
     *
     */
    function showbackups()
    {

        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $qrycnt = "SELECT count(*) from sysdbstab " .
        "WHERE bitval(flags,'0x2000')==0 " ;

        $sql = "SELECT " .
        "dbsnum," .
        "name," .
        "decode(level0,0,'NEVER',dbinfo('UTC_TO_DATETIME',level0)::char(40)) as level0,".
        "decode(logid0,0,'NEVER',logid0||':'||(logpos0)) as logpos0, " .
        "decode(level1,0,'NEVER',dbinfo('UTC_TO_DATETIME',level1)::char(40)) as level1,".
        "decode(logid1,0,'NEVER', logid1||':'||(logpos1)) as logpos1, " .
        "decode(level2,0,'NEVER',dbinfo('UTC_TO_DATETIME',level2)::char(40)) as level2,".
        "decode(logid2,0,'NEVER', logid2||':'||(logpos2)) as logpos2 " .
        "FROM sysdbstab " .
        "WHERE bitval(flags,'0x2000')==0 " .
        "ORDER BY 1";

        $tab->display_tab_by_page("{$this->idsadmin->lang('BackupReport')}",
        array(
        "1" => "{$this->idsadmin->lang('Number')}",
        "2" => "{$this->idsadmin->lang('Name')}",
        "3" => "{$this->idsadmin->lang('Level0')}",
        "4" => "{$this->idsadmin->lang('Position')}",
        "5" => "{$this->idsadmin->lang('Level1')}",
        "6" => "{$this->idsadmin->lang('Position')}",
        "7" => "{$this->idsadmin->lang('Level2')}",
        "8" => "{$this->idsadmin->lang('Position')}",
        ),
        $sql,
        $qrycnt,
        20);

    } #end

    function showComputerInfo()
    {

        if ( !Feature::isAvailable ( Feature::CHEETAH2, $this->idsadmin )  )
        {
            $this->idsadmin->html->add_to_output(
		 	"This feature is not available for your current server version." );
            return;
        }


        $db = $this->idsadmin->get_database("sysmaster");

        $sql = "SELECT  "
        . " format_units(os_mem_total/1024,'k') as mem_total,  "
        . " format_units(os_mem_free/1024,'k') as mem_free,  "
        . " *  "
        . " FROM sysmaster:sysmachineinfo   "
        ;
        $HTML="<TABLE></TR>";
        $stmt = $db->query($sql);
        if ($res = $stmt->fetch())
        {
            $HTML.="<TR><TH COLSPAN=2 ALIGN='center'>{$this->idsadmin->lang('ComputerInfo')}</TH></TR>"	;

            if ( isset($res['OS_NODENAME']) )
            $HTML.="<TR><TD>{$this->idsadmin->lang('HostName')} </TD><TD>{$res['OS_NODENAME']} </TD></TR> ";
            if ( isset($res['OS_NAME'])  )
            $HTML.="<TR><TD>{$this->idsadmin->lang('OSName')} </TD><TD>{$res['OS_NAME']} </TD></TR> ";
            if ( isset($res['OS_RELEASE'])  )
            $HTML.="<TR><TD>{$this->idsadmin->lang('OSRel')} </TD><TD>{$res['OS_RELEASE']}  </TD></TR> ";
            if (  isset($res['OS_VERSION']) )
            $HTML.="<TR><TD>{$this->idsadmin->lang('OSVer')} </TD><TD> {$res['OS_VERSION']} </TD></TR> ";
            if ( isset($res['OS_MACHINE']) )
            $HTML.="<TR><TD>{$this->idsadmin->lang('CPUType')}</TD><TD> {$res['OS_MACHINE']}  </TD></TR> ";
            if ( isset($res['MEM_TOTAL']) )
            $HTML.="<TR><TD>{$this->idsadmin->lang('TotalMem')}</TD><TD> {$res['MEM_TOTAL']}  </TD></TR> ";
            if (  isset($res['MEM_FREE']) )
            $HTML.="<TR><TD>{$this->idsadmin->lang('TotalFreeMem')}</TD><TD>{$res['MEM_FREE']} </TD></TR> ";
            if (  isset($res['OS_NUM_OLPROCS']) && isset($res['OS_NUM_OLPROCS']) )
            if ( $res['OS_NUM_OLPROCS'] == $res['OS_NUM_OLPROCS'] )
            $HTML.="<TR><TD>{$this->idsadmin->lang('CPUs')}</TD><TD>{$res['OS_NUM_PROCS']} </TD></TR> ";
            else
            {
                $HTML.="<TR><TD>{$this->idsadmin->lang('OnlineCPUs')}</TD><TD>{$res['OS_NUM_OLPROCS']} </TD></TR> ";
                $HTML.="<TR><TD>{$this->idsadmin->lang('TotalCPUs')}</TD><TD>{$res['OS_NUM_PROCS']} </TD></TR> ";
            }
            if ( isset($res['OS_PAGESIZE']) )
            $HTML.="<TR><TD>{$this->idsadmin->lang('OSPageSize')}</TD><TD> {$res['OS_PAGESIZE']}  </TD></TR> ";
            if ( isset($res['OS_OPEN_FILE_LIM']) )
            $HTML.="<TR><TD>{$this->idsadmin->lang('MaxFilesPerProc')}</TD><TD> {$res['OS_OPEN_FILE_LIM']}  </TD></TR> ";

            $HTML.="<TR><TH COLSPAN=2 ALIGN='center'>{$this->idsadmin->lang('SharedMemInfo')}</TH></TR>"	;
            foreach( $res as $index => $val )
            {
                if ( strncmp( $index, "OS_SHM",6)!=0 )
                continue;
                $HTML.="<TR><TD> " . ucfirst(strtolower(substr($index,3))) ."</TD><TD>{$val}</TD></TR> ";
            }

            $HTML.="<TR><TH COLSPAN=2 ALIGN='center'>{$this->idsadmin->lang('SemaphoreInfo')}</TH></TR>"	;
            foreach( $res as $index => $val )
            {
                if ( strncmp( $index, "OS_SEM",6)!=0 )
                continue;
                $HTML.="<TR><TD>" . ucfirst(strtolower(substr($index,3))) ."</TD><TD>{$val}</TD></TR> ";
            }
            /*
             foreach( $res as $index => $val )
             {
             $HTML.="<TR><TD> {$index}</TD><TD>{$val}</TD></TR> ";
             }
             */
        }
        $HTML.="</TABLE>";

        $this->idsadmin->html->add_to_output( $HTML );



    }

    function showResourceHistory()
    {

        if ( !Feature::isAvailable ( Feature::CHEETAH2, $this->idsadmin )  )
        {
            $this->idsadmin->html->add_to_output(
		 	"This feature is not available for your current server version." );
            return;
        }


        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);



        $sql = "SELECT 'Maximum', " .
        "MAX(max_conns) as max_conns,".
        "MAX(max_cpu_vps) as max_cpu_vps,".
        "MAX(max_vps)as max_vps, " .
        "format_units(MAX(total_size),'M') as t_size, " .
        "format_units(MAX(total_size_used),'M') as tu_size,".
        "format_units(MAX(max_memory),'M') as m_memory, " .
        "format_units(MAX(max_memory_used),'M') as m_memory_used, " .
        "MAX(total_size ) as total_size_used, " .
        "MAX(total_size_used) as total_size_used, " .
        "MAX(max_memory) as max_memory, " .
        "MAX(max_memory_used) max_memory_used " .
        "FROM sysmaster:syslicenseinfo " .
        " UNION " .
        "SELECT 'Minimum', " .
        "MIN(max_conns) as max_conns,".
        "MIN(max_cpu_vps) as max_cpu_vps,".
        "MIN(max_vps)as max_vps, " .
        "format_units(MIN(total_size),'M') as t_size, " .
        "format_units(MIN(total_size_used),'M') as tu_size,".
        "format_units(MIN(max_memory),'M') as m_memory, " .
        "format_units(MIN(max_memory_used),'M') as m_memory_used, " .
        "MIN(total_size ) as total_size_used, " .
        "MIN(total_size_used) as total_size_used, " .
        "MIN(max_memory) as max_memory, " .
        "MIN(max_memory_used) max_memory_used " .
        "FROM sysmaster:syslicenseinfo " .
        " UNION " .
        "SELECT 'Average', " .
        "ROUND(AVG(max_conns),0) as max_conns,".
        "ROUND(AVG(max_cpu_vps),0) as max_cpu_vps,".
        "ROUND(AVG(max_vps),0)as max_vps, " .
        "format_units(TRUNC( AVG(total_size),4),'M') as t_size, " .
        "format_units(TRUNC (AVG(total_size_used),4),'M') as tu_size,".
        "format_units(TRUNC (AVG(max_memory),4),'M') as m_memory, " .
        "format_units(TRUNC (AVG(max_memory_used),4),'M') as m_memory_used, " .
        "TRUNC(AVG(total_size ),4) as total_size_used, " .
        "TRUNC(AVG(total_size_used),4) as total_size_used, " .
        "TRUNC(AVG(max_memory),4) as max_memory, " .
        "TRUNC(AVG(max_memory_used),4) max_memory_used " .
        "FROM sysmaster:syslicenseinfo " .
        "ORDER BY 1 "

        ;

        $this->idsadmin->in['fullrpt']=on;
        $tab->display_tab_by_page("{$this->idsadmin->lang('ResourceSummaryReport')}",
        array(
        "1" => "",
        "2" => "{$this->idsadmin->lang('MaxConns')}",
        "3" => "{$this->idsadmin->lang('MaxCPUVPS')}",
        "4" => "{$this->idsadmin->lang('MAXVPS')}",
        "9" => "{$this->idsadmin->lang('DiskSize')}",
        "10" => "{$this->idsadmin->lang('DiskUsed')}",
        "11" => "{$this->idsadmin->lang('Memory')}",
        "12" => "{$this->idsadmin->lang('MemoryUsed')}
        "
        ),
        $sql,
        3,
        3);

        $this->idsadmin->html->add_to_output( "<BR>" );

        $qrycnt = "SELECT count(*) from sysmaster:syslicenseinfo " .
        			"WHERE 1=1" ;

        $sql = "SELECT " .
        "(MDY(1,1,year) + (week*7) UNITS DAY)::DATE as edate," .
        "max_conns,".
        "max_cpu_vps,".
        "max_vps, " .
        "format_units(total_size,'M') as t_size, " .
        "format_units(total_size_used,'M') as tu_size,".
        "format_units(max_memory,'M') as m_memory, " .
        "format_units(max_memory_used,'M') as m_memory_used, " .
        "total_size, " .
        "total_size_used, " .
        "max_memory, " .
        "max_memory_used " .
        "FROM sysmaster:syslicenseinfo " .
        "ORDER BY 1 DESC";

        if (!isset($this->idsadmin->in["runReports"])) {
            unset($this->idsadmin->in['fullrpt']);
        }

        $tab->display_tab_by_page("{$this->idsadmin->lang('ResourceUsageReport')}",
        array(
        "1" => "{$this->idsadmin->lang('ForWeekEnding')}",
        "2" => "{$this->idsadmin->lang('MaxConns')}",
        "3" => "{$this->idsadmin->lang('MaxCPUVPS')}",
        "4" => "{$this->idsadmin->lang('MAXVPS')}",
        "9" => "{$this->idsadmin->lang('DiskSize')}",
        "10" => "{$this->idsadmin->lang('DiskUsed')}",
        "11" => "{$this->idsadmin->lang('Memory')}",
        "12" => "{$this->idsadmin->lang('MemoryUsed')}
        "
        ),
        $sql,
        $qrycnt,
        20);

        $this->idsadmin->html->add_to_output( $HTML );



    }

    function showWaitingSessions()
    {

        if ( !Feature::isAvailable ( Feature::CHEETAH, $this->idsadmin )  )
        {
            $this->idsadmin->html->add_to_output(
		 	"This feature is not available for your current server version." );
            return;
        }

        require_once ROOT_PATH."lib/gentab.php";
        $tab = new gentab(&$this->idsadmin);

        $this->idsadmin->html->add_to_output( "<BR>" );

        $qrycnt = "SELECT count(*) from sysmaster:sysrstcb R "
        . "WHERE r.lkwait <> 0 "
        . "OR r.bfwait <> 0 "
        . "OR bitand(R.flags, '0x5010')>0"
        ;

        $sql =     "SELECT "
        . "R.username wait_user,R.sid  wait_sid,  "
        . "O.username owner_user,O.sid owner_sid,  "
        . " 'LOCK-'||tx.txt OWNER_LOCK_TYPE, "
        . "TRIM(T.dbsname)||'.'||TRIM(T.owner)||'.'||TRIM(T.tabname) wtable, "
        . "'ROWID '||decode(L.rowidr,0,L.rowidn,L.rowidr)  wrowid, "
        . "(CURRENT - dbinfo('UTC_TO_DATETIME', grtime))::char(40) as LOCK_WAIT_TIME "
        . "FROM sysmaster:sysrstcb R, sysmaster:syslcktab L, sysmaster:systabnames T, "
        . "sysmaster:sysrstcb O, sysmaster:flags_text TX "
        . "WHERE R.lkwait  <> 0 "
        . "AND   R.lkwait  =  L.address "
        . "AND   T.partnum =  L.partnum "
        . "AND   O.txp     =  L.owner "
        . "and   TX.tabname = 'syslcktab' "
        . "AND   TX.flags   = R.lkwttype "
        /*	. "UNION  "
         . "SELECT "
         . "R.username wait_user,R.sid  wait_sid,  "
         . "O.username owner_user,O.sid owner_sid,  "
         . "'BUFFER-'||decode( R.bfwtflag, 16, 'SHARE', 'EXCLUSIVE' ), "
         . "TRIM(T.dbsname)||'.'||TRIM(T.owner)||'.'||TRIM(T.tabname) Table, "
         . "'PAGE '||PG.pg_pagenum, "
         . "'Unknown' "
         . "FROM sysmaster:sysrstcb R, sysmaster:sysbufhdr B, "
         . "sysmaster:syspaghdr PG, sysmaster:systabnames T, outer sysmaster:sysrstcb O "
         . "WHERE R.bfwait  <> 0 "
         . "AND R.bfwait      = B.address "
         . "AND PG.pg_chunk   = B.chunk "
         . "AND PG.pg_offset  = B.offset "
         . "AND PG.pg_partnum = T.partnum "
         . "AND B.owner       = O.address "*/
        . "UNION "
        . "SELECT "
        . "R.username wait_user,R.sid  wait_sid,  "
        . " ' ' , 0,  "
        . "decode(  bitand(R.flags, '0x5010'), '0x10','Checkpoint', '0x1000', 'Log Buffer','0x4000','Transaction','Unknown' ), "
        . "'None', "
        . "'None', "
        . "'Unknown' "
        . "FROM sysmaster:sysrstcb R "
        . "where bitand(R.flags, '0x5010')>0 "
        . "UNION "
        . "SELECT  NVL(R.username,'thread('||trim(T.name)||')') wait_user, "
        . " NVL(R.sid, T.tid), "
        . " ' ', "
        . " 0, "
        ." wait_reason, "
        ." '',"
        ." 'none',";
        // statedetail column only available in 11.50 or higher
        $sql .= ( Feature::isAvailable ( Feature::CHEETAH2, $this->idsadmin ) )? " statedetail" : " ''";
        $sql .= " from systcblst T, outer sysrstcb R"
        ." where wreason not in ( 21, 28 )"
        ." and R.mttcb = T.address "
        ." and wreason > 0 ";

        if (isset($this->idsadmin->in["fullrpt"]))
        {
            $flag=$this->idsadmin->in["fullrpt"];
            unset($this->idsadmin->in['fullrpt']);
        }

        $tab->display_tab_by_page($this->idsadmin->lang('WaitSesRpt'),
        array(
        "1" => $this->idsadmin->lang('WaitSes'),
        "2" => $this->idsadmin->lang('ResourceOwner'),
        "3" => $this->idsadmin->lang('ResourceType'),
        "4" => $this->idsadmin->lang('WaitObject'),
        "5" => $this->idsadmin->lang('WaitItem'),
        "6" => $this->idsadmin->lang('WaitInfo')
        ),
        $sql,
        $qrycnt,
        20,
        "template_gentab_session_waiters.php");

        $this->idsadmin->html->add_to_output( $HTML );


        if (isset($flag)) {
            $this->idsadmin->in["fullrpt"]=$flag;
        }


    }



    /**
     * Future Developement
     *
     * Fina a column which has a unique or primmary constraint or index
     * on it so we can use it to update by
     *
     * @param string $tabname
     * @param string $dbsname
     * @return array
     */
    function findUniqueColumn( $tabname, $dbsname="sysmaster" )
    {

        $db = $this->idsadmin->get_database($dbsname);

        ################################
        # See if rowid will work
        ################################
        $sql ="SELECT tabid, partnum " .
        " FROM systables " .
        " WHERE tabname ='" . $tabname . "'";


        $stmt = $db->query($sql);
        if ($res = $stmt->fetch() )
        {
            $tabid = $res['TABID'];
            if ( $res['PARTNUM'] != 0 )
            {
                return array ( '1' => "ROWID") ;
            }
        }
        else
        return false;

        ################################
        # See if we have any unqiue indexes
        #  i.e. primary key, unique constraints
        #       or unique indexes
        ################################

        # Find an index with the fewest key parts
        # search up to 16 indexes key parts
        for( $i=2 ; $i < 17 ; $i++ )
        {
            $sql = "SELECT * " .
            "FROM sysindexes " .
            "WHERE " .
            " (  ( idxname in  ( " .
            "        select idxname from sysconstraints " .
            "        where constrtype in ('U','P') " .
            "    ))  " .
            "  OR  " .
            "    (   idxtype='U'  ) " .
            " ) " .
            " AND " .
            "    part".$i. "=0 " .
            " AND " .
            "   tabid = " . $tabid  ;

            $stmt = $db->query($sql);
            if ($res = $stmt->fetch() )
            {
                $col_where="";
                foreach( $res as $index => $val )
                if ( (strncmp("PART",$index,4) == 0 ) && $val>0 )
                {
                    if ( $col_where == "" )
                    $col_where = " colno IN ( " . abs($val) ;
                    else
                    $col_where .= " , " . abs($val) ;
                }
                $col_where .= " ) ";
                $sql = "SELECT colname " .
                " FROM syscolumns " .
                " WHERE tabid = " . $tabid .
                " AND " . $col_where;
                $results = array();
                $stmt = $db->query($sql);
                while ($res = $stmt->fetch())
                {
                    $results[] = $res['COLNAME'];
                }
                return $results;
            }

        }
        return false;
    }


    /**
     * Get meta data for a specific table
     *
     * @param string $tabname
     * @param string $dbsname
     * @return array
     */
    function showTableColumns( $tabname , $dbsname="sysmaster" )
    {
        $db = $this->idsadmin->get_database($dbsname);

        ################################
        # See if rowid will work
        ################################
        $sql ="SELECT tabid " .
        " FROM systables " .
        " WHERE tabname ='" . $tabname . "'";


        $stmt = $db->query($sql);
        if ($res = $stmt->fetch() )
        {
            $tabid = $res['TABID'];
        }
        else
        return false;


        $sql = "SELECT " .
        "colname, " .
        "colno, " .
        "coltype, " .
        "collength, " .
        "colmin, " .
        "colmax, " .
        "extended_id " .
        " FROM syscolumns " .
        " WHERE tabid = " . $tabid .
        " ORDER BY 2" ;

        $stmt = $db->query($sql);
        if ($res = $stmt->fetchALL() )
        return $res;
        return false;
    }

}
?>
