<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2008.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions andab
 *  limitations under the License.
 **************************************************************************
 */

/**
 * Permissions Manager
 */
class privileges {

    public $idsadmin;
    
    private $dbname; 
    private $dbcount;  // number of non-system databases on the server
    
    // Array of available table-level privileges
    private $tabprivileges = Array("select", "update", "insert", "delete", 
    		"index", "alter", "references", "under");
    

    # the 'constructor' function

    function __construct(&$idsadmin)
    {
        $this->idsadmin = &$idsadmin;
        $this->idsadmin->load_lang("privileges");
        $this->idsadmin->html->set_pagetitle($this->idsadmin->lang("UserPrivileges"));
    }

    /*
     * the run function
     * this is what index.php will call
     * the decision of what to actually do is based on
     * the value of 'act' which is either posted or getted
     */

    function run()
    {
    	$this->idsadmin->setCurrMenuItem("privileges");
    	
    	// If no dbname, default to the one set in the SQL Toolbox (if any)
   		if (isset($this->idsadmin->in['dbname']))
		{
		    $this->dbname = $this->idsadmin->in['dbname'];
		} else {
			$this->dbname = $this->idsadmin->phpsession->get_sqldbname();
		}
		$dbselect = $this->createDatabaseSelect();
        
    	switch($this->idsadmin->in['do'])
        {
        	case 'database';
        		if (isset($this->idsadmin->in['grantuser']))
        		{
        			$this->execGrantUserPrivilege();
        		}
        		if (isset($this->idsadmin->in['save']))
        		{
        			$this->execModifyUserPrivilege();
        		}
     	        $this->privilegesSelector("database", $dbselect);
     	        $this->databasePrivileges();
                break;
                 
            case 'table';
            	if (isset($this->idsadmin->in['granttable']))
        		{
        			$this->execGrantTablePrivilege();
        		}
                if (isset($this->idsadmin->in['save']))
        		{
        			$this->execModifyTablePrivilege();
        		}
				$this->privilegesSelector("table", $dbselect);
				$this->tablePrivileges();
        	    break;
        	   
        	case 'roles';
           		if (isset($this->idsadmin->in['createrole']))
        		{
        			$this->execCreateNewRole();
        		}
        		$this->privilegesSelector("roles", $dbselect);
				$this->showRoles();
				break;
        	
           default:
                $this->idsadmin->error("{$this->idsadmin->lang('ErrorMsgSorry')}");
          		break; 
        }
        
    } // end function run

    /**
     * Displays the header of the privileges manager pages.  This allows the user 
     * to select a database and select whether they want to look at database or 
     * table level privileges
     */
    function privilegesSelector($privileges_page, $dbselect)
    {
		
        switch ($privileges_page)
    	{
    		case "table";
    		    $tabcheck = "CHECKED";
    		    $dbcheck = $rolescheck = "";
    		    break;
       		case "roles";
    		    $rolescheck = "CHECKED";
    		    $dbcheck = $tabcheck = "";
    		    break;    		    
    		case "database";
    		default;
    		    $dbcheck = "CHECKED";
    		    $tabcheck = $rolescheck = "";
    		    break;
    	}
		
    
		$html.=<<<END
<script>
function privilegesSwitch(page)
{
window.location= "{$this->idsadmin->get_config('BASEURL')}/index.php?act=privileges&do=" + page + "&dbname={$this->dbname}";
return;
}
</script>

<table class='gentab_nolines'>
  <tr>
    <td class='tblheader' style='text-align:center'>{$this->idsadmin->lang("ManagePrivileges")}</td>
  </tr>
  <tr style='text-align:center'>
    <td><b>{$this->idsadmin->lang("Database")}:</b> $dbselect</td> 
  </tr>
  <tr style='text-align:center'>
	<td><input type=radio class='radiobutton' name='dbradio' value='database' $dbcheck onclick='privilegesSwitch("database");' >
		<b>{$this->idsadmin->lang("DBPrivileges")}</b></input>
		<input type=radio class='radiobutton' name='tableradio' value='table' $tabcheck onclick='privilegesSwitch("table");'>
		<b>{$this->idsadmin->lang("TablePrivileges")}</b></input>
		<input type=radio class='radiobutton' name='rolesradio' value='roles' $rolescheck onclick='privilegesSwitch("roles");'>
		<b>{$this->idsadmin->lang("Roles")}</b></input>
	</td>
  </tr>
  </table>
END;

    	$this->idsadmin->html->add_to_output($html);
    	
    	// if no non-system databases, cannot display anything else in Privileges Manager
    	if ($this->dbcount == 0) 
    	{
    		$this->idsadmin->fatal_error($this->idsadmin->lang("NoDatabasesError"));
    	}
		
    }
    
    /**
     * Database-Level Privileges view
     */    
    function databasePrivileges () 
    {
	    require_once ROOT_PATH."lib/gentab.php";	
		$tab = new gentab(&$this->idsadmin);
		
		// Display table of user privileges for dbname
       	$db = $this->idsadmin->get_database($this->dbname);
  		$qry = "select username as name, " .
            "CASE " .
            " WHEN (usertype='C')" .
            "   THEN 'CONNECT' " .
            " WHEN (usertype='D')" .
            "   THEN 'DBA' " .
            " WHEN (usertype='R')" .
            "   THEN 'RESOURCE' " .
            "END  as usertype, " .
            "defrole, " .
            "'{$this->dbname}' as dbname " .
  		    "from {$this->dbname}:sysusers " .
  		    "where usertype<>'G'";

        $qrycnt = "select count(*) from {$this->dbname}:sysusers where usertype<>'G'";

        $tab->display_tab_by_page($this->idsadmin->lang("DBPrivileges"),
            array("1" => $this->idsadmin->lang("User"), 
			      "2" => $this->idsadmin->lang("Privilege"),
			      "3" => $this->idsadmin->lang("DefRole"),),
			$qry, $qrycnt, 10, "gentab_db_privileges.php",$db);
			
		if (!$this->idsadmin->isreadonly()) {
			$this->showGrantUserPrivilege();
		}
    }
    	
    /**
     * Table-Level Privileges view
     */
    function tablePrivileges () 
    {
    	require_once ROOT_PATH."lib/gentab.php";	
		$tab = new gentab(&$this->idsadmin);
		
	// Display table-level privileges for dbname
       	$db = $this->idsadmin->get_database($this->dbname);
  	$qry = "select trim(b.owner)||'.'||trim(b.tabname) AS tabname, " . 
		"a.grantee, a.grantor, a.tabauth, a.tabid, " .
		"'{$this->dbname}' as dbname " .
  		"from {$this->dbname}:systabauth as a, ". 
        	"{$this->dbname}:systables as b " .
        	"where a.tabid = b.tabid and b.tabname not like 'sys%' " .
        	"order by tabname, a.grantee, a.grantor";

        $qrycnt = "select count(*) from {$this->dbname}:systabauth as a, " .
		"{$this->dbname}:systables as b " .
		"where a.tabid = b.tabid and b.tabname not like 'sys%' ";

        $tab->display_tab_by_page($this->idsadmin->lang("TablePrivileges"),
            array("1" => $this->idsadmin->lang("Tabname"), 
			      "2" => $this->idsadmin->lang("User"),
			      "3" => $this->idsadmin->lang("Grantor"),
			      "4" => $this->idsadmin->lang("Privileges"),),
			$qry, $qrycnt, 20, "gentab_tab_privileges.php",$db);

		if (!$this->idsadmin->isreadonly()) {
 			$this->showGrantTablePrivilege();
		}
    }

    function showRoles()
    {
		require_once ROOT_PATH."lib/gentab.php";	
		$tab = new gentab(&$this->idsadmin);
    	
    	// Display table of all roles
		$db = $this->idsadmin->get_database($this->dbname);
  		$qry = "select username as name " .
  		    "from {$this->dbname}:sysusers " .
  		    "where usertype='G'";

        $qrycnt = "select count(*) from {$this->dbname}:sysusers where usertype='G'";

        $tab->display_tab_by_page($this->idsadmin->lang("Roles"),
            array("1" => $this->idsadmin->lang("Name")), 
			$qry, $qrycnt, 20, "gentab_role_privileges.php",$db);
		
		if (!$this->idsadmin->isreadonly()) {			
    		$this->showCreateNewRole();
		}
    }
    
    /**
     * Create a select drop-down of all non system databases
     */
    function createDatabaseSelect ()
    {
    	$do = (isset($this->idsadmin->in['do']))? $this->idsadmin->in['do']:"database";
    	$select =<<<EOF
<script type="text/javascript">
function privilegesChangeDB(select) {
var dbname = select.options[select.selectedIndex].value;
window.location= "{$this->idsadmin->get_config('BASEURL')}/index.php?act=privileges&do={$do}&dbname=" + dbname;
return;
}
</script>
<select onChange='privilegesChangeDB(this)' name='dbname' style='vertical-align: middle'>"
EOF;
        $db = $this->idsadmin->get_database("sysmaster");
        $stmt = $db->query("select name from sysdatabases where name not like 'sys%'");
        
        $count = 0;
        while ($res = $stmt->fetch() )
        {
        	$count++;
        	$name = trim($res['NAME']);
        	if ($this->dbname == "") 
        	{
        		// if no current dbname, use the first returned by the query as the default
				$this->dbname = $name;
        	}
        	$selected = (strcasecmp($this->dbname,$name)==0)? "SELECTED":"";
        	$select .= "<option $selected value='{$name}'>{$name}</option>";
        }
    	
        $select .= "</select>";
        
        $this->dbcount = $count;
        return $select;
    }
    
    /**
     * Create a select drop-down of all non system tables in the selected database
     */
    function createTableSelect ()
    {
    	$tabname = (isset($this->idsadmin->in['tabname']))? $this->idsadmin->in['tabname']:"";
    	$db = $this->idsadmin->get_database($this->dbname);
        $stmt = $db->query("select trim(owner)||'.'||trim(tabname) as tabname from sysmaster:systabnames ".
            "where tabname not like 'sys%' and tabname[1,1] != ' ' " .
            "and dbsname = '{$this->dbname}' order by tabname;");
    	$select ="<select name='tabname' style='vertical-align: middle'>";       
        	
        while ($res = $stmt->fetch() )
        {
        	$name = trim($res['TABNAME']);
            if ($this->idsadmin->phpsession->instance->get_delimident() == "Y")
            {
                // need to enclose table name in quotes if DELIMIDENT=Y
                $name_r = explode(".",$name);
                $name_r[1] = "\"{$name_r[1]}";
                $name_r[count($name_r) -1 ] = "{$name_r[count($name_r) -1]}\"";
                $name = implode(".",$name_r);
            }
        	$selected = (strcasecmp($name,$tabname)==0)? "SELECTED":"";	
        	$select .= "<option $selected value='{$name}'>{$name}</option>";
        }
    	
        $select .= "</select>";
        return $select;
    }
    
    
    /**
     * Show Grant User Privilege input form
     */
    function showGrantUserPrivilege () 
    {
    	$HTML = <<<EOF
<br />
<form method="post" name="grantUserPriv" action="index.php?act=privileges&amp;do=database&amp;dbname={$this->dbname}">
<table>
<tr>
   <th colspan=5>{$this->idsadmin->lang("GrantNewPriv")}:</th>
</tr>
<tr>
   <td><b>{$this->idsadmin->lang("Username")}</b></td>
   <td><input type=text name="username" value="" /></td>
   <td><b>{$this->idsadmin->lang("Privilege")}</b></td>
   <td><select name='newpriv'>
       <option value="Connect">{$this->idsadmin->lang('Connect')}</option>
       <option value="Resource">{$this->idsadmin->lang('Resource')}</option>
       <option value="DBA">{$this->idsadmin->lang('DBA')}</option>
      </select></td>
   <td><input type='submit' class='button' name='grantuser' value='{$this->idsadmin->lang("Grant")}' 
        onclick='grantUserPriv.submit()' /></td>
</tr>
</table>
</form>
EOF;
        $this->idsadmin->html->add_to_output($HTML);
    }
    
    /**
     * Show Grant Table Privilege input form
     */
    function showGrantTablePrivilege () 
    {
        if ($this->idsadmin->isreadonly())
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPermission"));
    	}
    	
    	$table_select = $this->createTableSelect();
    	$username = (isset($this->idsadmin->in['username']))? $this->idsadmin->in['username']:"";
		
    	$checked = Array();
    	foreach ($this->tabprivileges as $level)
		{
			$checked["$level"] = (isset($this->idsadmin->in["{$level}_box"]))? "CHECKED":"";	
		}
    	
    	$HTML = <<<EOF
<br />
<div class='borderwrapwhite'>
<form method="post" name="grantTablePriv" action="index.php?act=privileges&amp;do=table&amp;dbname={$this->dbname}">
<table>
<tr>
   <th colspan=5>{$this->idsadmin->lang("GrantNewPriv")}:</th>
</tr>
<tr>
   <td><b>{$this->idsadmin->lang("Username")}</b></td>
   <td><input type=text name="username" value="$username" /></td>
   <td><b>{$this->idsadmin->lang("Tabname")}</b></td>
   <td>$table_select</td>
<tr>
</tr>
    <td colspan=4>
<table class='gentab' >
<tr>
<td class='color_select'>S</td><td><input type=checkbox  name='select_box' {$checked['select']}/>{$this->idsadmin->lang('Select')}</td>
<td class='color_update'>U</td><td><input type=checkbox  name='update_box' {$checked['update']}/>{$this->idsadmin->lang('Update')}</td>
<td class='color_insert'>I</td><td><input type=checkbox  name='insert_box' {$checked['insert']}/>{$this->idsadmin->lang('Insert')}</td>
<td class='color_delete'>D</td><td><input type=checkbox  name='delete_box' {$checked['delete']}/>{$this->idsadmin->lang('Delete')}</td>
</tr>
<tr>
<td class='color_index'>X</td><td><input type=checkbox  name='index_box' {$checked['index']}/>{$this->idsadmin->lang('Index')}</td>
<td class='color_alter'>A</td><td><input type=checkbox  name='alter_box' {$checked['alter']}/>{$this->idsadmin->lang('Alter')}</td>
<td class='color_ref'>R</td><td><input type=checkbox  name='references_box' {$checked['references']}/>{$this->idsadmin->lang('References')}</td>
<td class='color_under'>N</td><td><input type=checkbox  name='under_box' {$checked['under']}/>{$this->idsadmin->lang('Under')}</td>
</tr>
</td></tr>
</table>
   </td>
   <td><input type='submit' class='button' name='granttable' value='{$this->idsadmin->lang("Grant")}' 
        onclick='grantTablePriv.submit()' /></td>
</tr>
</table>
</form>
</div>
EOF;
        $this->idsadmin->html->add_to_output($HTML);
    }   
    
    /**
     * Show create new role form
     */
    function showCreateNewRole () 
    {
    	$HTML = <<<EOF
<br />
<form method="post" name="createNewRole" action="index.php?act=privileges&amp;do=roles&amp;dbname={$this->dbname}">
<table>
<tr>
   <th colspan=5>{$this->idsadmin->lang("CreateNewRole")}:</th>
</tr>
<tr>
   <td><b>{$this->idsadmin->lang("RoleName")}</b></td>
   <td><input type=text name="role" value="" /></td>
   <td><input type='submit' class='button' name='createrole' value='{$this->idsadmin->lang("Create")}' 
        onclick='createNewRole.submit()' /></td>
</tr>
</table>
</form>
EOF;
        $this->idsadmin->html->add_to_output($HTML);
    }

        
    /**
     * Grant User Privilege
     */
    function execGrantUserPrivilege ()
    {
    	if ($this->idsadmin->isreadonly())
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPermission"));
    		return;
    	}
    	
    	$db = $this->idsadmin->get_database($this->dbname);
    	
    	if (!isset($this->idsadmin->in['username']) || $this->idsadmin->in['username'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoUserError"));
    		return;
    	}
    	if (!isset($this->idsadmin->in['newpriv']) || $this->idsadmin->in['newpriv'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPrivError"));
    		return;
    	}
    	
    	$username = $this->idsadmin->in['username'];
    	$newpriv = $this->idsadmin->in['newpriv'];
    	$grant = "GRANT $newpriv to '$username'";
    	
	    // If the user already has privileges on this database,
		// we need to revoke first, then grant;
		$sql = "select username, " .
		    " CASE WHEN (usertype='C') THEN 'Connect' " .
            " WHEN (usertype='D') THEN 'DBA' " .
            " WHEN (usertype='R') THEN 'Resource' " .
            "END  as usertype from sysusers where username='{$username}'";
		$stmt = $db->query($sql);
		$res = $stmt->fetch();
		if ($res) {
			$grant = "revoke {$res['USERTYPE']} from '$username' ; " . $grant;
		}
    	
    	$db->exec($grant);
        $err_r = $db->errorInfo();
        if ($err_r[1] != 0) {
            $this->idsadmin->error("{$this->idsadmin->lang('GrantFailed')}. <br> {$this->idsadmin->lang('ErrorF')} {$err_r[1]} {$err_r[2]} ");
        } else {
        	$this->idsadmin->status($this->idsadmin->lang('GrantForUserSuccess') . " {$username}.");
        }    	
    }
    
    
    /**
     * Modify User Privilege
     */
    function execModifyUserPrivilege ()
    {
        if ($this->idsadmin->isreadonly())
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPermission"));
    		return;
    	}
    	
    	$db = $this->idsadmin->get_database($this->dbname);
    	
    	if (!isset($this->idsadmin->in['username']) || $this->idsadmin->in['username'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoUserError"));
    		return;
    	}
    	if (!isset($this->idsadmin->in['newpriv']) || $this->idsadmin->in['newpriv'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPrivError"));
    		return;
    	}
    		
    	$username = $this->idsadmin->in['username'];
    	$newpriv = $this->idsadmin->in['newpriv'];
    	$prevpriv = (isset($this->idsadmin->in['prevpriv']))? $this->idsadmin->in['prevpriv']:"";
    	if (strcasecmp($newpriv,$prevpriv)==0) {
    		// if new privilege is same as previous privilege, then there is nothing to do
			return;
    	}

    	// Run revoke of current privilege, then run grant on the new one
    	$grant_stmt = "REVOKE $prevpriv from '$username'; "; 
    	if (strcasecmp($newpriv,"Revoke") !=0) {
  			$grant_stmt .= "GRANT $newpriv to '$username'";
    	}
    	$db->exec($grant_stmt);
        $err_r = $db->errorInfo();
        if ($err_r[1] != 0) {
            $this->idsadmin->error("{$this->idsadmin->lang('GrantFailed')}. <br> {$this->idsadmin->lang('ErrorF')} {$err_r[1]} {$err_r[2]} ");
        } else {
        	if (strcasecmp($newpriv,"Revoke") ==0) {
        		$this->idsadmin->status($this->idsadmin->lang('PrivilegeRevokedForUser') . " {$username}.");
        	} else {
        		$this->idsadmin->status($this->idsadmin->lang('PrivilegeSavedForUser') . " {$username}.");
        	}
        }    	
    }
    
        
    /**
     * Grant Table Privilege
     */
    function execGrantTablePrivilege ()
    {
        if ($this->idsadmin->isreadonly())
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPermission"));
    		return;
    	}
    	
    	$db = $this->idsadmin->get_database($this->dbname);
    	
    	if (!isset($this->idsadmin->in['username']) || $this->idsadmin->in['username'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoUserError"));
    		return;
    	}
    	if (!isset($this->idsadmin->in['tabname']) || $this->idsadmin->in['tabname'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoTabError"));
    		return;
    	}
    	
    	$username = $this->idsadmin->in['username'];
    	$tabname = $this->idsadmin->in['tabname'];
    	
    	// enclose owner in single quotes
        $tabname_r = explode(".",$tabname);
        $tabname_r[0] = "'{$tabname_r[0]}'";
        $tabname = implode(".",$tabname_r);
    	
    	$newprivlist = "";
		foreach ($this->tabprivileges as $level)
		{
			if (isset($this->idsadmin->in["{$level}_box"]))
			{
				$newprivlist .= "{$level},";
			}
		}
		
		if ($newprivlist == "")
		{
    		$this->idsadmin->error($this->idsadmin->lang("NoPrivError"));
    		return;
    	}
    		
		// strip off last comma in $newprivlist
    	$newprivlist = substr($newprivlist,0,strlen($newprivlist)-1);

		$grant = "GRANT $newprivlist on $tabname to '$username'";
 		$db->exec($grant);
        $err_r = $db->errorInfo();
        if ($err_r[1] != 0) {
            $this->idsadmin->error("{$this->idsadmin->lang('GrantFailed')}. <br> {$this->idsadmin->lang('ErrorF')} {$err_r[1]} {$err_r[2]} ");
        } else {
        	$this->idsadmin->status($this->idsadmin->lang('GrantSuccess1') . " {$tablename} " . $this->idsadmin->lang('GrantSuccess2') . " {$username}.");
        }    	
    }
    
    /**
     * Modify Table Privilege
     */
    function execModifyTablePrivilege ()
    {
        if ($this->idsadmin->isreadonly())
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPermission"));
    		return;
    	}
    	
    	$db = $this->idsadmin->get_database($this->dbname);
    	
    	if (!isset($this->idsadmin->in['grantee']) || $this->idsadmin->in['grantee'] == "" ||
    	    !isset($this->idsadmin->in['grantor']) || $this->idsadmin->in['grantor'] == "")
    	{
    		// can only happen if user manually types save into the url
    		$this->idsadmin->error($this->idsadmin->lang("NoUserError"));
    		return;
    	}
    	if (!isset($this->idsadmin->in['tabname']) || $this->idsadmin->in['tabname'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoTabError"));
    		return;
    	}

    	$username = $this->idsadmin->in['grantee'];
    	$grantor = $this->idsadmin->in['grantor'];
    	$tabname = $this->idsadmin->in['tabname'];
    	       
    	// enclose owner in single quotes
        $tabname_r = explode(".",$tabname);
        $tabname_r[0] = "'{$tabname_r[0]}'";
        $tabname = implode(".",$tabname_r);
    	
		foreach ($this->tabprivileges as $level)
		{
			if (isset($this->idsadmin->in["{$level}box"]))
			{
				$newprivlist .= "{$level},";
			}
		}
		// strip off last comma in $newprivlist
    	$newprivlist = substr($newprivlist,0,strlen($newprivlist)-1);
		
		// Revoke all for the user on the table first,
		// then grant the new privileges
		$grant_stmt = "REVOKE ALL on $tabname from '$username' as '$grantor'; ";
		if ($newprivlist != "") 
		{
			$grant_stmt .=  "GRANT $newprivlist on $tabname to '$username'  as '$grantor'";
		}
		
    	$db->exec($grant_stmt);
        $err_r = $db->errorInfo();
        if ($err_r[1] != 0) {
            $this->idsadmin->error("{$this->idsadmin->lang('GrantFailed')}. <br> $grant_stmt <br> {$this->idsadmin->lang('ErrorF')} {$err_r[1]} {$err_r[2]} ");
        } else {
        	if ($newprivlist == "") {
        		$this->idsadmin->status($this->idsadmin->lang('PrivilegeRevokedForUser') . " {$username} " . $this->idsadmin->lang('OnTable') . " {$tabname}.");
        	} else {
        		$this->idsadmin->status($this->idsadmin->lang('PrivilegeSavedForUser') . " {$username} " . $this->idsadmin->lang('OnTable') . " {$tabname}.");
        	}
        }    	
    }

    /**
     * Create New Role
     */
    function execCreateNewRole ()
    {
        if ($this->idsadmin->isreadonly())
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoPermission"));
    		return;
    	}
    	
    	$db = $this->idsadmin->get_database($this->dbname);
    	
    	if (!isset($this->idsadmin->in['role']) || $this->idsadmin->in['role'] == "")
    	{
    		$this->idsadmin->error($this->idsadmin->lang("NoRoleError"));
    		return;
    	}
    	
    	$role = $this->idsadmin->in['role'];
    	$create = "CREATE ROLE $role";
    	$db->exec($create);
        $err_r = $db->errorInfo();
        if ($err_r[1] != 0) {
            $this->idsadmin->error("{$this->idsadmin->lang('CreateFailed')}. <br> {$this->idsadmin->lang('ErrorF')} {$err_r[1]} {$err_r[2]} ");
        } else {
        	$this->idsadmin->status($this->idsadmin->lang('CreateSuccessForRole') . " {$role}.");
        }    	
    }
    
} // end class privileges
?>
