<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2008.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

/**
 * This is a simple example 'module' which generates a table showing usernames and the number of
 * sequential scans by that user.
 */

class example {

    /**
     * Each module should have an 'idsadmin' member , this gives access to the OAT API.
     */
    var $idsadmin;

    /**
     * Every class needs to have a constructor method that takes &$idsadmin as its argument
     * We are also going to load our 'language' file too.
     * @param Class $idsadmin
     */
    function __construct(&$idsadmin)
    {
        $this->idsadmin = $idsadmin;
        /**
         * load our language file.
         */
        $this->idsadmin->load_lang("example");
    } // end of function __construct

    /**
     * Every class needs a 'run' method , this is the 'entry' point of your module.
     *
     */
    function run()
    {
	    /**
	     * find out what the user wanted todo ..
	     */
	    $do = $this->idsadmin->in['do'];
	    
	    /**
	     * map our action to a function.
	     */
	    switch ($do)
	    {
	    	case "demo":
	    	default:
	    		$this->demo();
	    }
		
    } // end of function run

    /**
	 * our main demo function
	 */
    function demo()
    {
   		 /**
         * Set the Page Title - this is the title that is shown in the browser.
         */
        $this->idsadmin->html->set_pagetitle("Demo");

        /**
         * Lets select some information from the sysmaster:sysuserthreads table.
         * How about we display a list of threads and the number of seqscans that thread has done.
         */

        /**
         * we first need a 'connection' to the database.
         */
        $db = $this->idsadmin->get_database("sysmaster");

        /**
         * now we write our query
         */
        $qry = " SELECT us_name AS user_name , us_seqscans AS sequential_scans from sysuserthreads ";

        /**
         *  we need another query which would be the 'count' of the # of rows returned from the previous
         *  query.
         */
        $qrycnt = " SELECT count(*) as count FROM sysuserthreads ";


        /**
         * We can use the 'gentab' api in OAT to create the output for us.
         *
         * 1. first we load the gentab class.
         */

        require_once("lib/gentab.php");


        /**
         * create a new instance of the gentab class
         */
        $tab = new gentab($this->idsadmin);

        /**
         * call the display_tab_by_page function of the gentab class and pass the required arguments.
         *   arg1:  Title .
         *   arg2:  Array of 'column' headings.
         *          We use the idsadmin lang function to get our string to use as a heading.
         *   arg3:  the query.
         *   arg4:  the count query.
         *   arg5:  how many to display per page
         */
        $tab->display_tab_by_page("Users And Sequential Scans",
        array(
                  "1" => $this->idsadmin->lang("username"),
                  "2" => $this->idsadmin->lang("numseqscans"),
        ),
        $qry,$qrycnt,5);

        /**
         * we are done ..
         * test this in your browser using the url ..
         *      http://HOSTNAME/OATINSTALL/index.php?act=/ibm/example/example
         */
    
    } // end function demo
    
} // end of class example
?>