<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

class compressionServer {

    var $idsadmin;
    var $minRows; /* the minimum # of rows to allow a compress table */
    function __construct()
    {
        define( 'ROOT_PATH'    ,"../../");
        define( 'IDSADMIN'     ,  "1" );
        define( 'DEBUG'        , false);
        define( 'SQLMAXFETNUM' , 100 );

        include_once("../serviceErrorHandler.php");
        set_error_handler("serviceErrorHandler");

        require_once(ROOT_PATH."lib/idsadmin.php");
        $this->idsadmin = new idsadmin(true);
        $this->idsadmin->load_lang("compression");

        require_once("lib/TableService.php");
        require_once("lib/compressionOptionsService.php");
        $this->minRows = $this->getMinRows();
        set_time_limit(0);
    }

    function parse_estimate($r , $tab)
    {
        if ( ! isset ( $r[0] ) )
        {
            $ret = array();
            $ret[$tab] = array (
                    "est"    => "-99"
                    ,"curr"   => "0"
                    ,"change" => "0"
                    ,"table"  => ""
                    ,"est_date" => ""
                    );

                    return $ret;
        }

        //error_log( var_export ( $r , true ) );
        $ret = array();
        foreach ( $r as $key => $value)
        {
            $lines = $value['EST_ESTIMATE'];
            $xx = preg_split("/\n/",$lines);

            foreach ( $xx as $k => $v )
            {

                if ( $v == "")
                {
                    continue;
                }
                if ( substr($v,0,6) == "FAILED")
                {
                    $ret = array();
                    $ret[$tab] = array (
                    "est"    => "-99"
                    ,"curr"   => "0"
                    ,"change" => "0"
                    ,"table"  => ""
                    ,"est_date" => ""
                    );
                    continue;
                }
                if (substr($v,0,2) == "--")
                {
                    $v = substr($v,65);
                }


                if ( substr($v,0,8) == "Partiton"  )
                {
                    $v = substr($v,0,8);
                }

                if ( substr($v,0,2) > 0 === true)
                {

                    //error_log( var_export ( $v , true ) );

                    $v = str_replace("  "," ",$v);
                    $v = str_replace("  "," ",$v);
                    $v = str_replace("%","",$v);

                    if ( substr ( $v , 0 , 1) == " " )
                    {
                        $v = substr($v,1);
                    }

                    //error_log( var_export ( $v , true ) );


                    $val = preg_split("/ /",$v);
                    if ( count( $val ) < 5 )
                    {
                        error_log("short");
                    }
                    else
                    {
                    //error_log( var_export ( $val , true ) );
                    //est   curr  change partnum    table
                    $ret[$val[3]] = array (
                    "est"    => $val[0]
                    ,"curr"   => $val[1]
                    ,"change" => $val[2]
                    ,"table"  => $val[4]
                    ,"est_date" => $value['EST_DATE']
                    );
                    }
                }

            }
        }
        //error_log ( var_export ( $ret , true ));
        return $ret;

    }

    /**
     * get a list of pending jobs for compression
     */
    function getJobList()
    {
        $res = array();
        /* get the running jobs */

        $sel  = " SELECT smgr_opdesc as js_command ";
        $sel .= ",'R'||smgr_id as js_task";
        $sel .= ",trim(smgr_dbname)||':'||trim(smgr_owner)||'.'||trim(smgr_tabname) as js_comment ";
        $sel .= ",DBINFO('utc_to_datetime',smgr_starttime)::DATETIME year TO second as js_start";
        $sel .= ",(smgr_rowsprocessed/smgr_numrows)*100 as msg ";
        $sel .= ",DBINFO('utc_to_datetime',smgr_starttime+smgr_remainingtime)::DATETIME year TO second||'(est)' as js_done";
        $sel .= ",smgr_rowsprocessed as processed";
        $sel .= ",smgr_elapsedtime as elapsed";
        $sel .= ",smgr_numrows as numrows";
        $sel .= " FROM sysmaster:sysstoragemgr";

        $res[] = $this->doDatabaseWork($sel,"sysmaster");



        /* get the completed jobs */
        $sel = " SET ISOLATION TO DIRTY READ ";
        $this->doDatabaseWork($sel,"sysadmin");

        $sel  = "  SELECT ";
        $sel .= " TRIM(SUBSTR(C.cmd_ret_msg,0,1000)) as msg ";
        $sel .= ", J.js_id ";
        $sel .= ", J.js_task ";
        $sel .= ", J.js_start ";
        $sel .= ", J.js_done";
        $sel .= ", J.js_comment ";
        $sel .= ", J.js_command";

        $sel .= " from job_status J, ph_task ,  outer command_history C ";
        $sel .= " where ";
        $sel .= " ABS(js_result)  = cmd_number";
        $sel .= " AND js_task = tk_id";
        $sel .= " AND tk_group = 'COMPRESSION' ";
        //$sel .= " ORDER BY js_id DESC";
        $sel .= " and js_done is NOT NULL ";

        $res[] = $this->doDatabaseWork($sel,"sysadmin");

        //error_log ( var_export ( $res[0] , true ) );

        return $res;
    }

    /**
     * get the list of Databases
     *
     */
    function getDatabaseList()
    {

        $sel  = "SELECT";
        $sel .= " data.name dbname";
        /*
         $sel .= ",dbspace.name as catdbs";
         $sel .= ",CASE is_logging";
         $sel .= " WHEN 1 THEN";
         $sel .= "        CASE is_buff_log";
         $sel .= "        WHEN 1 THEN";
         $sel .= "                'buffered'";
         $sel .= "        ELSE";
         $sel .= "                'unbuffered'";
         $sel .= "        END";
         $sel .= "        ELSE";
         $sel .= "        'nolog'";
         $sel .= " END AS logging";
         */
        $sel .= " FROM";
        $sel .= " sysdatabases data";
        $sel .= " WHERE ";
        $sel .= " data.name NOT IN ('sysmaster','sysutils','syscdr','sysuser','system',";
        $sel .= "'syscdcv1','syscdcv2','syscdcv3','syscdcv4','syscdcv5','syscdcv6','syscdcv7',";
        $sel .= "'syscdcv8','syscdcv9')";
        $sel .= "ORDER BY 1";

        $ret = $this->doDatabaseWork($sel,"sysmaster");
        return $ret;


    } // end getDatabaseList

    /**
     * get the list of DBSpaces
     *
     */
    function getDBSpaceList()
    {

        $sel  = "SELECT";
        $sel .= " dbsnum";
        $sel .= ",name";
        $sel .= ",pagesize";
        $sel .= " FROM sysdbspaces";
        $sel .= " WHERE is_temp = 0";
        $sel .= " ORDER BY dbsnum";

        return $this->doDatabaseWork($sel,"sysmaster");

    } // end getDBSpaceList

    /**
     * get the list of Tables for a particular database.
     *
     */
    function getTableListForDatabase($dbname , $tabname="")
    {

        $extra = "";
        if ( $tabname != "" )
        {
            $extra = " AND tabname='{$tabname}'";
        }
        $sel  = " select sum(ti_nptotal) as nptotal , sum(ti_npdata) as npused ";
        $sel .= ", sum(ti_nrows)  as nrows";
        $sel .= ", count(*) as cnt ";;
        $sel .= ", decode ( (sum ( sysmaster:bitval(ti_flags,'0x08000000') )) , 0 , 'no' , count(*) , 'yes' , 'some' ) as compressed";
        $sel .= ", '{$dbname}' as dbname  , tabname , owner  , pagesize ";
        $sel .= " from sysmaster:systabinfo ,";
        $sel .= "(select hex(decode(partnum , 0 , partn , partnum))  , trim(tabname) as tabname , systables.tabid , trim(owner) as owner , tabtype , pagesize ";
        $sel .= " from systables , outer sysfragments";
        $sel .= " where";
        $sel .= " systables.tabid = sysfragments.tabid";
        $sel .= " {$extra} ";
        $sel .= " and fragtype  = 'T'";
        $sel .= ") as  t2 (partn , tabname , tabid , owner , tabtype , pagesize )";
        $sel .= " where";
        $sel .= " ti_partnum = partn";
        $sel .= " and tabid > 99";
        $sel .= " AND tabtype not in ('P','V','Q','E')";
        $sel .= " group by tabname , owner , pagesize  ";

        $row =  $this->doDatabaseWork($sel,"{$dbname}");

        $runners = $this->getRunningJobs();

        foreach ( $row as $k => $v )
        {
            $comp = trim($v['DBNAME']).":".trim($v['OWNER']).".".trim($v['TABNAME']);

            $isRunning = false;
            if ( isset ($runners[$comp]) )
            {
                $isRunning = true;
            }


            $row[$k]['RUNNING'] = $isRunning;

            if ( $isRunning === false && $v['NROWS'] >= $this->minRows )
            {
                $s = $this->getEstimates($v['DBNAME'],trim($v['OWNER']),$v['TABNAME']);
            }
            else
            {
                $s = array();
            }

            $row[$k]['ESTIMATES'] = $this->parse_estimate($s,$v['TABNAME']);

            if ( $v['CNT'] > 1 )
            {
                $fragments = $this->getFragmentsForTable($v['DBNAME'],$v['OWNER'],$v['TABNAME'] , $isRunning);
                $row[$k]['FRAGMENTS'] = $fragments;
            }
        }

        return $row;

    } // end getTableListForDatabase

    // get an array of running jobs.
    function getRunningJobs()
    {
        $ret = array();

        $sel = "select js_comment from sysadmin:job_status , sysadmin:ph_task where js_done is null and tk_id = js_task and tk_group = 'COMPRESSION' ";
        $rows = $this->doDatabaseWork($sel,"sysadmin");
        foreach ( $rows as $k => $v)
        {
            foreach ( $v as $k2 => $v2)
            {
                $ret[$v2]=true;
            }
        }
        $sel = "select trim(smgr_dbname) as dbname , trim(smgr_owner) as owner ";
        $sel .= ",trim(smgr_tabname) as tabname from sysmaster:sysstoragemgr ";
        $rows = $this->doDatabaseWork($sel,"sysmaster");
        foreach ( $rows as $k => $v )
        {
            $comp = "{$v['DBNAME']}:{$v['OWNER']}.{$v['TABNAME']}";
            $ret[$comp]=true;
        }

        return $ret;
    }

    
    /**
     * check_for_estimates
     * 
     * Check if we have any estimates by checking for the existance of the mon_estimate_compression table
     * in the sysmaster database.
     * 
     * @return boolean . 
     *     true - yes we have some estimates .
     *     false - no estimates yet.
     */
    function check_for_estimates()
    {
    	$sel = "SELECT count(*) AS cnt FROM systables WHERE tabname = 'mon_compression_estimates'";
    	$ret = $this->doDatabaseWork($sel,"sysadmin");
    	if ( isset($ret[0]['CNT']) === true && $ret[0]['CNT'] > 0 )
    	{
    		return true;
    	}
    	return false;
    }
    
    /**
     * 
     * @param $dbname
     * @param $owner
     * @param $table
     * @return unknown_type
     */
    function getEstimates($dbname , $owner , $table)
    {
        $ret = array();
        $sel = " SELECT partnum FROM sysmaster:systabnames WHERE  tabname = '{$table}' AND dbsname = '{$dbname}' AND owner  = '{$owner}' ";
        $rows = $this->doDatabaseWork($sel , "sysadmin");
        foreach ( $rows as $k => $v )
        {
            $row = $this->getEstimatesForPartnum($v['PARTNUM']);
            if ( count($row) > 0 )
            {
                //error_log(" estimate for table {$table}");
                //error_log ( var_export ( $row , true ));
                $ret[] = $row[0];
            }
        }

        return $ret;
    }

    function getEstimatesForPartnum($partnum)
    {
       // first lets check id we have any compression_estimates .. 
        $rows = array();
    	if ( $this->check_for_estimates() === false )
        {
			return $rows;        	
        }

        $sel = " SELECT first 1 est_estimate , est_date FROM mon_compression_estimates ";
        $sel .= " WHERE ";
        $sel .= " est_partnum = {$partnum} ";
        $sel .= " AND est_date = ( SELECT MAX(est_date) FROM mon_compression_estimates WHERE est_partnum = {$partnum} ) ";

        $rows = $this->doDatabaseWork($sel,"sysadmin");
        return $rows;


        //$sel = " select task('fragment estimate_compression','{$partnum}') from sysmaster:sysdual";
        //$rows = $this->doDatabaseWork($sel,"sysadmin");
        //return $rows;
    }

    function getFragmentsForTable($dbname , $owner , $table , $isRunning)
    {
        $sel  = " SELECT ti_nptotal AS nptotal ";
        $sel .= ", ti_npdata AS npused ";
        $sel .= ", ti_nrows  AS nrows";
        $sel .= ", 0 as est ";
        $sel .= ", '{$dbname}' as dbname  ";
        $sel .= ", '{$isRunning}' as running  ";
        $sel .= ", decode ( sysmaster:bitval(ti_flags,'0x08000000') , 1, 'yes' ,'no') as compressed";
        $sel .= ", partnum  ";
        $sel .= ", dbspace  ";
        $sel .= ", tabname  ";
        $sel .= ", owner  ";
        $sel .= " from sysmaster:systabinfo ,";
        $sel .= "(select hex(decode(partnum , 0 , partn , partnum )) , trim(tabname) as tabname , systables.tabid ";
        $sel .= "      , trim(owner) as owner , tabtype , pagesize , trim(partition) as dbspace ";
        $sel .= " from systables , outer sysfragments";
        $sel .= " where";
        $sel .= " systables.tabid = sysfragments.tabid";
        $sel .= " and fragtype  = 'T'";
        $sel .= " and tabname   ='{$table}'";
        $sel .= " and owner     ='{$owner}'";
        $sel .= ") as  t2 (partnum , tabname , tabid , owner , tabtype , pagesize , dbspace)";
        $sel .= " where";
        $sel .= " ti_partnum = t2.partnum";
        $sel .= " and tabid > 99";
        $sel .= " AND tabtype not in ('P','V','Q','E')";

        $rows = $this->doDatabaseWork($sel,$dbname);
        return $rows;
    }

    function estimateCompression($table)
    {
        //error_log( var_export ( $table , true ));
        $fullname = "{$table->dbname}:{$table->owner}.{$table->tabname}";

        if ( $table->partnum == null || $table->partnum == "" )
        {

            $sel = "EXECUTE FUNCTION admin_async_estimates(\"{$table->tabname}\",\"table\",\"{$table->dbname}\",\"{$table->owner}\",\"COMPRESSION\",\"estimate for:{$fullname}\",current,null,null,\"t\",\"t\",\"t\",\"t\",\"t\",\"t\")";
        }
        else
        {
            $sel = "EXECUTE FUNCTION admin_async_estimates(\"{$table->partnum}\",\"fragment\",\"\",\"\",\"COMPRESSION\",\"estimate for:{$fullname}\",current,null,null,\"t\",\"t\",\"t\",\"t\",\"t\",\"t\")";
#            $sel = "EXECUTE FUNCTION admin_async_estimates(\"{$table->partnum}\",\"COMPRESSION\",\"estimate for:{$fullname}\",current,null,null,\"t\",\"t\",\"t\",\"t\",\"t\",\"t\")";
        }
       // error_log ( $sel );
        $res = $this->doDatabaseWork($sel,"sysadmin");
        $res = array( "RESULT" => $res[0][""]  ) ;

        return $res;
    }

    function canBeCompressed($table)
    {
        if ( $table->dbname == "" )
        {
            return "{$this->idsadmin->lang('InvalidDB')} {$table->dbname}";
        }

        if ( $table->tabname == "" )
        {
            return "{$this->idsadmin->lang('NoTable')}";
        }

        if ( $table->owner == "" )
        {
            return "{$this->idsadmin->lang('NoOwner')}";
        }


        if ( $table->partnum == 0 )
        {
            $sel = "SELECT SUM(ti_nrows) AS nrows FROM systabinfo, systabnames WHERE tabname='{$table->tabname}' AND ti_partnum = partnum";
        }
        else
        {
            $sel = "SELECT SUM(ti_nrows) AS nrows FROM systabinfo WHERE ti_partnum = '{$table->partnum}'";
        }

        $val = $this->doDatabaseWork($sel,"sysmaster");
        $actual =  isset( $val[0]['NROWS']) ? $val[0]['NROWS'] : 0;
        if ( $actual < $this->minRows )
        {
            return "{$this->idsadmin->lang('NotEnoughRows')}:  {$this->idsadmin->lang('required')}: {$this->minRows}  {$this->idsadmin->lang('actual')}: {$actual}";
        }
        return NULL;
    }

 
    function isCompressionEnabled()
    {
    	/* This check is still required for pre 11.50.xC6 versions .. With xC6+ there is no need to run enable compression */
        $db = $this->idsadmin->get_database("sysmaster");
        $sel = "SELECT name , value FROM sysmaster:sysshmhdr WHERE number = 6 AND name = 'pz_version'";
        $stmt = $db->query($sel);
        $row  = $stmt->fetch();
        $stmt->closeCursor();

        $compress_enabled = 0;

        if ( isset($row['VALUE']) && $row['VALUE'] > 16 )
        {
            $compress_enabled = 1;
        }

        return $compress_enabled;
    }

    /**
     * compress a table
     */
    function compressTable($table , $options)
    {

        $reason="";

        require_once("lib/compressionOptionsService.php");
        $repack="";
        $truncate="";
        $compress="";

        // lets create our options .

        if ( $options->repack === true )
        {
            if  ( $options->offline === true )
            {
                $repack = "repack_offline";
            }
            else
            {
                $repack = "repack";
            }
        }

        if ( $options->compress == true )
        {
            if ( ( $reason = $this->canBeCompressed( $table ) ) != NULL )
            {
                return array('RESULT' => "FAILED: {$reason}");
            }
            $compress="compress";
        }

        if ( $options->truncate == true )
        {
            $truncate="shrink";
        }

        // is it a whole table or single partnum
        if ( $table->partnum == 0 )
        {
            //$sel = " task('table {$compress} {$repack} {$truncate}' , '{$table->tabname}' , '{$table->dbname}' , '{$table->owner}') ";
            $sel = " 'table {$compress} {$repack} {$truncate}' , '{$table->tabname}' , '{$table->dbname}' , '{$table->owner}' ";
        }
        else
        {
            // $sel = " task('fragment {$compress} {$repack} {$truncate}' , '{$table->partnum}' ) ";
            $sel = " 'fragment {$compress} {$repack} {$truncate}' , '{$table->partnum}'  ";
        }

        // is compression turned on , if not turn it on.
        if ( $this->isCompressionEnabled() == 0 )
        {
            $res = $this->doDatabaseWork("EXECUTE FUNCTION task('enable compression')","sysadmin");

            $res = array( "RESULT" => $res[0][""] );
            if ( substr($res['RESULT'],0,7) == "Unknown" )
            {
                $res['RESULT'] = "FAIL: {$res['RESULT']}";
            }
            if ( substr($res['RESULT'],0,4) == "FAIL" )
            {
                return $res;
            }
        }

        // lets execute our job - using the admin_async which will return us a
        // job number.
        $fullname = "{$table->dbname}:{$table->owner}.{$table->tabname}";
        $sel = "EXECUTE FUNCTION admin_async(\"{$sel}\",\"COMPRESSION\",\"{$fullname}\",current,null,null,\"t\",\"t\",\"t\",\"t\",\"t\",\"t\")";
        $res = $this->doDatabaseWork($sel,"sysadmin");
        $res = array( "RESULT" => $res[0][""]  ) ;

        return $res;
    } // end compressTable


    /**
     * uncompress a table
     */
    function uncompressTable($table,$options)
    {

        require_once("lib/compressionOptionsService.php");
        $repack="";
        $truncate="";
        $compress="";
        // build up our options..
        if ( $options->repack === true )
        {
            if  ( $options->offline === true )
            {
                $repack = "repack_offline";
            }
            else
            {
                $repack = "repack";
            }
        }

        if ( $options->compress == true )
        {
            $compress="uncompress";
            if ( $options->offline === true )
            {
                $compress = "uncompress_offline";
            }
        }

        if ( $options->truncate == true )
        {
            $truncate="shrink";
        }

        if ( $table->partnum == 0 )
        {
            //$sel = "EXECUTE FUNCTION task('table {$compress} {$repack} {$truncate}' , '{$table->tabname}' , '{$table->dbname}' , '{$table->owner}') ";
            $sel = " 'table {$compress} {$repack} {$truncate}' , '{$table->tabname}' , '{$table->dbname}' , '{$table->owner}' ";
        }
        else
        {
            //$sel = "EXECUTE FUNCTION task('fragment {$compress} {$repack} {$truncate}' , '{$table->partnum}' ) ";
            $sel = " 'fragment {$compress} {$repack} {$truncate}' , '{$table->partnum}'  ";
        }


        $fullname = "{$table->dbname}:{$table->owner}.{$table->tabname}";
        $sel = "EXECUTE FUNCTION admin_async(\"{$sel}\",\"COMPRESSION\",\"{$fullname}\",current,null,null,\"t\",\"t\",\"t\",\"t\",\"t\",\"t\")";
        $res = $this->doDatabaseWork($sel,"sysadmin");
        $res = array( "RESULT" => $res[0][""]  ) ;

        return $res;

    }

    function getTableInfoForDatabase($table)
    {
        if ( $table->partnum == 0 )
        {
            $res = $this->getTableListForDatabase($table->dbname , $table->tabname );
        }
        else
        {
            $res = $this->getFragmentInfo($table);
        }
        return $res;

    } // end getTableInfoForDatabase

    // get the minimum number of rows needed for compression.
    function getMinRows()
    {
        $sel = "SELECT env_value FROM sysenv WHERE env_name='IFX_COMPRESSION_MIN_ROWS'";
        $val = $this->doDatabaseWork($sel,"sysmaster");
        // findout the minimum # of rows a table can have.
        $minrows = isset($val[0]['ENV_VALUE']) ? trim($val[0]['ENV_VALUE']) : 2000;
        return $minrows;
    }

    function getTableListForDBSpace($dbsnum)
    {

        $sel  = " select ti_nptotal as nptotal , ti_npdata as npused ";
        $sel .= ", ti_nrows  as nrows";
        $sel .= ", decode (( sysmaster:bitval(ti_flags,'0x08000000') ) , 0 , 'no'  , 'yes') as compressed";
        $sel .= ", dbsname as dbname  , trim(tabname) as tabname , trim(owner) as owner  , ti_pagesize as pagesize , partnum";
        $sel .= " from sysmaster:systabinfo , systabnames ";
        $sel .= " where";
        $sel .= " ti_partnum = partnum";
        $sel .= " and partdbsnum(partnum) = {$dbsnum}";
        $sel .= " and dbsname not in ( select name from sysmaster:sysdbspaces )";
        $sel .= " and bitval(ti_flags,8192) == 0";       // system
        $sel .= " and bitval(ti_flags,4) == 0"    ;      // system catalog
        $sel .= " and bitval(ti_flags,262144) == 0"    ; // index partition
        $sel .= " and tabname NOT IN ( select tabname from systables where tabid < 99 )";
        $sel .= " and ti_npdata != 0";
        $sel .= " AND dbsname NOT IN ('sysmaster','sysutils','syscdr','sysuser','system',";
        $sel .= "'syscdcv1','syscdcv2','syscdcv3','syscdcv4','syscdcv5','syscdcv6','syscdcv7',";
        $sel .= "'syscdcv8','syscdcv9')";

        $res = $this->doDatabaseWork($sel,"sysmaster");

        $runners = $this->getRunningJobs();

        foreach ( $res as $k => $v )
        {
            $comp = trim($v['DBNAME']).":".trim($v['OWNER']).".".trim($v['TABNAME']);

            if ( isset ($runners[$comp]) )
            {
                $isRunning = true;
            }
            else
            {
                $isRunning = false;
            }

            $res[$k]['RUNNING'] = $isRunning;

            if ( $isRunning === false && $v['NROWS'] >= $this->minRows )
            {
                $s = $this->getEstimatesForPartnum($v['PARTNUM']);
            }
            else
            {
                $s = array();
            }
            $res[$k]['ESTIMATES'] = $this->parse_estimate($s,$v['TABNAME']);
        }

        return $res;

    }

    /**
     * getFragmentInfo
     *  Get Info about a fragment / partition.
     */
    function getFragmentInfo($table)
    {
        $sel  = " SELECT ti_nptotal AS nptotal ";
        $sel .= ", ti_npdata AS npused ";
        $sel .= ", ti_nrows  AS nrows";
        $sel .= ", 0 as est ";
        $sel .= ", dbname  ";
        $sel .= ", decode ( sysmaster:bitval(ti_flags,'0x08000000') , 1, 'yes' ,'no') as compressed";
        $sel .= ", partnum  ";
        $sel .= ", dbspace  ";
        $sel .= ", tabname  ";
        $sel .= ", owner  ";
        $sel .= " from sysmaster:systabinfo ,";
        $sel .= "(select hex(decode(partnum , 0 , partn , partnum )) , tabname , systables.tabid ";
        $sel .= "      , owner , tabtype , pagesize , partition as dbspace , dbname ";
        $sel .= " from systables , outer sysfragments";
        $sel .= " where";
        $sel .= " systables.tabid = sysfragments.tabid";
        $sel .= " and fragtype  = 'T'";
        $sel .= ") as  t2 (partnum , tabname , tabid , owner , tabtype , pagesize , dbspace , dbname)";
        $sel .= " where";
        $sel .= " ti_partnum = t2.partnum";
        $sel .= " AND ti_partnum = '{$table->partnum}'";

        $rows = $this->doDatabaseWork($sel,$table->dbname);
        $rows["ESTIMATES"] = $this->parse_estimate($this->getEstimatesForPartnum($table->partnum),$rows['TABNAME']);
        return $rows;
    }

    /**
     * do the database work.
     *
     */
    function doDatabaseWork($sel,$dbname="sysmaster") //,$dirty=true)
    {
        //error_log($dbname." ".$sel);
        $ret = array();

        $db = $this->idsadmin->get_database($dbname);

        while (1 == 1)
        {
            $stmt = $db->query($sel);
            while ($row = $stmt->fetch() )
            {
                $ret[] = $row;
            }
            $err = $db->errorInfo();

            if ( $err[2] == 0 )
            {
                $stmt->closeCursor();
                break;
            }
            else
            {
                $err = "{$this->idsadmin->lang('ErrorF')} {$err[2]} - {$err[1]}";
                $stmt->closeCursor();
                trigger_error($err,E_USER_ERROR);
                continue;
            }
        }

        return $ret;
    }
}

?>
