<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007 , 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */


class dashboardServer {

    public $conndb;  /* PDO handle to the sqlite connections database */
    public $idsadmin = null;  

    function __construct()
    {

        define( "ROOT_PATH","../../");
        define( 'IDSADMIN',  "1" );
        define( 'DEBUG', false);
        define( 'SQLMAXFETNUM' , 100 );

        include_once("../serviceErrorHandler.php");
        set_error_handler("serviceErrorHandler");

        require_once("../../services/idsadmin/clusterdb.php");
        $this->conndb = new clusterdb ();
        
        require_once("../../lib/idsadmin.php");
        $this->idsadmin = new idsadmin(true);
    }

    function getDBSpaces()
    {

        $sel = "select sh_pagesize from sysshmvals";
        $res = $this->doDatabaseWork($sel,"sysmaster");
        $defaultPageSize = $res[0]['SH_PAGESIZE'];

        $sel = "select syschunks.dbsnum as dbsnum, trim(name) as name "
        .",sum(decode(mdsize,-1,nfree,udfree) * {$defaultPageSize}) as nfree "
        .",sum((chksize - decode(mdsize,-1,nfree,udfree)) * {$defaultPageSize}) as nused "
        .",sum(chksize*{$defaultPageSize}) as size "
        ."from syschunks , sysdbspaces "
        ."where syschunks.dbsnum = sysdbspaces.dbsnum "
        ."group by 1,2 "
        ."order by 1 asc" ;

        return $this->doDatabaseWork($sel,"sysmaster");

    } // end getDBSpaces

    function getDBSpacesIO()
    {

        $sel = "select syschunks.dbsnum, trim(name) as name,sum(pagesread) as read, "
        ." sum(pageswritten) as write "
        ." from syschunks , sysdbspaces ,syschkio "
        ." where syschunks.dbsnum = sysdbspaces.dbsnum "
        ." and syschunks.chknum = syschkio.chunknum "
        ." group by 1,2 "
        ." order by 1 asc" ;


        return $this->doDatabaseWork($sel,"sysmaster");

    } // end getDBSpacesIO

    function getChunksByDBS($dbsnum)
    {
        if ( ! $dbsnum )
        {
            triger_error (" getChunksByDBS " , " Param not found ");
        }

        $sel = "select sh_pagesize from sysshmvals";
        $res = $this->doDatabaseWork($sel,"sysmaster");
        $defaultPageSize = $res[0]['SH_PAGESIZE'];

        $sel = "select trim(fname) as name "
        .",((chksize - decode(mdsize,-1,nfree,udfree)) * {$defaultPageSize}) as nused "
        .",decode(mdsize,-1,nfree,udfree) * {$defaultPageSize} as nfree "
        ." from syschktab where dbsnum = {$dbsnum}" ;
        return $this->doDatabaseWork($sel,"sysmaster");

    } // end getChunksByDBS

    function getDBSpaceById($dbsNum=1)
    {
        if ( ! $dbsNum )
        {
            trigger_error(" getDBSPaceById - Param not found ",E_USER_ERROR);
        }

        $sel = "select dbsnum , trim(name) as name, trim(owner) as owner, pagesize "
        ." ,fchunk ,nchunks ,is_mirrored ,is_blobspace "
        ." ,is_sbspace ,is_temp ,flags "
        ." from sysdbspaces where dbsnum = {$dbsNum}" ;

        $row = $this->doDatabaseWork($sel,"sysmaster");
        return new myDBspace_t($row);

    } // end getDBSpacesIO

    function getMemory()
    {

        $sel = "select current hour to second as dt,sum(seg_blkfree) as free "
        .",sum(seg_blkused) as used from syssegments ";

        $row = $this->doDatabaseWork($sel,"sysmaster");
        return $row;

    } // end getMemory

    function getTransac()
    {

        $sel = "select current hour to second as dt, count(*) as num "
        ." from systrans where bitval(tx_flags,2) > 0 ";


        $row = $this->doDatabaseWork($sel,"sysmaster");
        return $row;

    } // end getTransac

    function getLocks()
    {

        $sel = "select current hour to second as dt, count(*) as num "
        ." from syslocks";


        $row = $this->doDatabaseWork($sel,"sysmaster");
        return $row;

    } // end getLocks

    function getTableReads()
    {

        $sel = "select first 10   dbsname , tabname , isreads "
        ." from sysptprof order by isreads desc";

        $row = $this->doDatabaseWork($sel,"sysmaster");
        return $row;

    } // end getTableReads

    function getCpu()
    {

        $sel = "select current hour to second as dt , TRUNC(sum(usecs_user + usecs_sys ),2) as user "
 				. " FROM sysmaster:sysvplst";
 				
         $row = $this->doDatabaseWork($sel,"sysmaster");
        return $row;

    } // end getCpu
    
    function getDashBoards()
    {
        $this->idsadmin->load_lang("home");
        $qry = "select * from dashboards";
        $res = $this->doConnectionsDBWork($qry);
        
        // Localize default "Default" and "Space" dashboard names and descriptions.
        // These are inserted into the connections.db in English at install time.  
        // If user has not edited these, we want to display the names and descriptions 
        // in the current UI language.
        for($i = 0; $i < count($res); $i++)
        {
            if ($res[$i]['dashboard_name'] == "Default")
            {
                $res[$i]['dashboard_name'] = $this->idsadmin->lang("Default");
            } else if ($res[$i]['dashboard_name'] == "Space") {
                $res[$i]['dashboard_name'] = $this->idsadmin->lang("Space");
            }
            if ($res[$i]['dashboard_description'] == "An example dashboard that shows Memory and Transactions")
            {
                $res[$i]['dashboard_description'] = $this->idsadmin->lang("DefaultDashboardDescription");
            } else if ($res[$i]['dashboard_description'] == "An example dashboard that shows Space")
            {
                $res[$i]['dashboard_description'] = $this->idsadmin->lang("SpaceDashboardDescription");
            }
        }
        
        return $res;
    } // end getDashBoards

    function getPanels($dashboardId=0)
    {
        if ( $dashboardId == 0 )
        {
            $qry = "select * from panels";
        }
        else
        {
            $qry = "select * from panels where panel_id not in ( select panelid from dashpanels where dashid = {$dashboardId} )";
        }

        return $this->doConnectionsDBWork($qry);
    } // end getPanels

    function getPanelsForDashBoard($dashboardId)
    {
        if ( !$dashboardId )
        {
            trigger_error("getPanelsForDashBoard - No Dashboard Id Found",E_USER_ERROR);
        }

        $qry = "select * from dashpanels,panels where dashid = {$dashboardId} and panels.panel_id = panelid  order by pos";
        return $this->doConnectionsDBWork($qry);

    }

    function putDashBoardPanels($dashid,$dashboardpanels)
    {

        //error_log("DashBoard ID: {$dashid}");
        //  error_log(var_export($dashboardpanels,true));

        // first delete any dashpanels
        $qry = "delete from dashpanels where dashid = {$dashid}";
        $this->conndb->query($qry);
        $this->checkSQL();
        //now insert the new ones .
        foreach ($dashboardpanels as $k => $v)
        {
             
            if ( is_array ($v) )
            {
                foreach ($v as $p => $c)
                {
                    $qry = "insert into dashpanels values ( {$dashid} ,{$c->panel_id}, {$p} );";
                    $this->conndb->query($qry);
                    $this->checkSQL();
                }
            }
            else
            {
                $qry = "insert into dashpanels values ( {$dashid} ,{$v->panel_id}, 0);";
                $this->conndb->query($qry);
                $this->checkSQL();
            }

            // error_log("panelid = $v->panel_id");
        }
        //error_log(var_export($dashboardpanels,true));
        return 0;
    }

    function putDashBoard($dashboard_id , $dashboard_name,$dashboard_description , $dashboard_refresh=10 )
    {
        if ( $dashboard_refresh > 60 )
        {
            $dashboard_refresh = 60;
        }
        
        if ( $dashboard_id > 0)
        {
            $qry =  "update dashboards  set dashboard_name = '{$dashboard_name}' ";
            $qry .= ",dashboard_description = '{$dashboard_description}' ";
            $qry .= ",dashboard_refresh = '{$dashboard_refresh}' ";
            $qry .= " where dashboard_id = {$dashboard_id};";
           
            $this->conndb->query($qry);
            $this->checkSQL();
            return $dashboard_id;
        }
        else
        {
            $qry =  "insert into dashboards (dashboard_name,dashboard_description,dashboard_refresh) ";
            $qry .= "values ( '{$dashboard_name}','{$dashboard_description}' , '{$dashboard_refresh}' )";
            $this->conndb->query($qry);
             $this->checkSQL();
            $qry="select last_insert_rowid() as status";
            //error_log($qry);
            $res = $this->doConnectionsDBWork($qry);
            //error_log(var_export($res,true));
            return $res[0]['status'];
        }
    }

    function deleteDashBoard($dashboard_id)
    {
        $qry = "delete from dashboards where dashboard_id = {$dashboard_id}";
        $this->conndb->query($qry);
        $this->checkSQL();
        return 0;
    }

    function doConnectionsDBWork($sel)
    {
        $ret = array();
        $stmt = $this->conndb->query($sel);
        while ($row = $stmt->fetch())
        {
            $ret[] = $row;

        }
        return $ret;
    } // end doConnectionsDBWork

    /**
     * Check the sql was successful , if not trigger an error.
     *
     * @return nothing or error out
     */
    function checkSQL()
    {
        $err = $this->conndb->errorInfo();
        //error_log(var_export($err,true));
        if ( $err[0]  != "00000" )
        {
             trigger_error("database: query failed -  Error: {$err[1]} {$err[2]} ",E_USER_ERROR);    
        }
    }
    

    function getOSMemory()
    {
        $sel = "select 	 os_mem_total as total, os_mem_free free, ( os_mem_total - os_mem_free ) as used  "
        ." from sysmachineinfo ";

        $row = $this->doDatabaseWork($sel,"sysmaster");
        return $row;
    }

    function doDatabaseWork($sel,$dbname="sysmaster")
    {

        $db = $this->idsadmin->get_database($dbname);
        $cnt = 0;
        while ($cnt++ <= 5)
        {
            $stmt = $db->query($sel);

            while ($row = $stmt->fetch() )
            {
                $ret[] = $row;
            }
            $err = $db->errorInfo();
            if ( $err[2] == 0 )
            {
                $stmt->closeCursor();
                break;
            }
            else
            {
                //error_log(var_export($err,true));
                $stmt->closeCursor();
                continue;
            }
        }
        if ( $cnt >= 5 )
        {
            trigger_error("too many errors encountered ",E_USER_ERROR);
          // error_log("aborted: {$sel} due to too many errors");
        }
        return $ret;
    }
}

class myDBSpace_t  {
    public $dbsnum        ;
    public $name          ;
    public $owner         ;
    public $pagesize      ;
    public $fchunk        ;
    public $nchunks       ;
    public $is_mirrored   ;
    public $is_blobspace  ;
    public $is_sbspace    ;
    public $is_temp       ;
    public $flags         ;

    function __construct($arr)
    {
        foreach ($arr as $k => $row )
        {
            $this->dbsnum = $row['DBSNUM'];
            $this->name   = $row['NAME'];
            $this->owner  = $row['OWNER'];
            $this->pagesize = $row['PAGESIZE'];
            $this->fchunk   = $row['FCHUNK'];
            $this->nchunks  = $row['NCHUNKS'];
            $this->is_mirrored = $row['IS_MIRRORED'];
            $this->is_blobspace = $row['IS_BLOBSPACE'];
            $this->is_sbspace = $row['IS_SBSPACE'];
            $this->is_temp    = $row['IS_TEMP'];
            $this->flags      = $row['FLAGS'];
        }
    }

}
?>
