<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation, 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */

/* Services for Query By Example (QBE) feature */

class qbeServer {

	var $idsadmin;

	function __construct()
	{
		define ("ROOT_PATH","../../");
		define( 'IDSADMIN',  "1" );
		define( 'DEBUG', false);
		define( 'SQLMAXFETNUM' , 100 );

		include_once("../serviceErrorHandler.php");
		set_error_handler("serviceErrorHandler");

		require_once(ROOT_PATH."lib/idsadmin.php");
		$this->idsadmin = new idsadmin(true);
		
		// Set act=qbe so that when we form database connections, it uses the 
		// SQL Toolbox user/password.
		$this->idsadmin->in = array("act" => "qbe");
	}
	
	/**
	 * Get an XML list of all databases and tables on the server
	 *  - Filter out tables with columns unsupported by QBE
	 *  - Filter out system databases
	 **/
	function getDBTableXMLList()
	{
		$xml="<list>";
		$databases = $this->getDatabases();
		foreach ($databases as $i=>$v)
		{
			$dbname = trim($v['DATABASENAME']);
			if ($this->testForDatabaseConnectPermission($dbname))
			{
    			$xml.="<database name=\"{$dbname}\">";
    			$tables = $this->getTables($dbname);
    			foreach($tables as $i1=>$v1)
    			{
    				$tabname = trim($v1['TABNAME']);
    				$tabid = trim($v1['TABID']);
    				$xml.="<table name=\"{$tabname}\" tabid=\"{$tabid}\" dbname=\"{$dbname}\" />";
    			}
    			$xml.="</database>";
			}
		}
		$xml.="</list>";
		return $xml;
	}

	/**
	 * Get the name of all databases on the server.
	 **/
	private function getDatabases()
	{
		$sql = "SELECT name AS databasename FROM sysdatabases ";
		$sql.= "WHERE name ";
		$sql.= "NOT IN ('sysmaster','sysutils','syscdr','sysuser','system',";
        $sql.= "'syscdcv1','syscdcv2','syscdcv3','syscdcv4','syscdcv5','syscdcv6','syscdcv7',";
        $sql.= "'syscdcv8','syscdcv9')";
		$res = $this->doDatabaseWork($sql,"sysmaster");
		return $res;
	}
	
	/**
	 * Get the name and tabid of all tables for a database. 
	 *  - Filter out tables with columns unsupported by QBE
	 *  - Filter out system tables, with tabid < 99
	 **/
	private function getTables($database)
	{
		/*$sql = "SELECT tabname,tabid FROM systables";*/
		$sql = "SELECT UNIQUE";
		$sql.= " systables.tabname, systables.tabid , systables.owner";
		$sql.= " FROM systables,syscolumns";
		$sql.= " WHERE systables.tabid = syscolumns.tabid";
		$sql.= " AND tabtype != 'Q'"; // This removes sequences
		$sql.= " AND systables.tabid > 99";	// This removes all system tables	
		$sql.= " AND syscolumns.tabid NOT IN (";
		$sql.= " 	SELECT tabid FROM syscolumns";
		$sql.= "	WHERE BITAND(coltype,'0x00ff') IN (11,12,15,16,19,20,21,22)"; // This removes BYTE,TEXT,NCHAR,NVARCHAR,SET,MULTISET,LIST,ROW types
		$sql.= " 	OR BITAND(coltype,'0x0800')==2048"; // This removes DISTINCT TYPES
		$sql.= " 	OR extended_id NOT IN (0,1,5)"; // This removes all UDT types, except boolean, lvarchar		
		$sql.= " 	)";
		$res = $this->doDatabaseWork($sql,$database);
		return $res;
	}
	
	/**
	 * Get the column names, colno, coltype and extended_id for a tabid, database.
	 * Also get the column or set of columns that can be used to uniquely identify 
	 * rows in that table.
	 **/ 
	function getColumnInfo($tabname,$tabid,$database)
	{

		$retval = array();
		$sql = "select tabtype, owner, nrows FROM systables WHERE tabid={$tabid}";
		$res = $this->doDatabaseWork($sql,$database);
		$retval['tabtype'] = trim($res[0]['TABTYPE']);
		$owner = trim( $res[0]['OWNER'] );
		$nrows = $res[0]['NROWS'];
		
		$sql = "SELECT colname, colno, mod(coltype,256) as coltype, extended_id, collength ,";
		$sql.= " DECODE(BITAND(coltype,'0x0100'),256,1,0) AS allownull,";
		$sql.= " DECODE(BITAND(coltype,'0x0200'),512,1,0) AS hvar ";
		$sql.= "FROM syscolumns WHERE tabid={$tabid}";
		$res = $this->doDatabaseWork($sql,$database);
		$retval['database'] = $database;
		$retval['table'] = $tabname;
		$retval['tabid'] = $tabid;
		$retval['owner'] = $owner;
		$retval['nrows'] = $nrows;
		$retval['columns'] = array();
		foreach($res as $i=>$v)
		{
			$colname = trim($v['COLNAME']);
			$colno = trim($v['COLNO']);
			$coltype = trim($v['COLTYPE']);
			$extended_id = trim($v['EXTENDED_ID']);
			$notnull = (trim($v['ALLOWNULL']) == "0");
			$hvar = trim($v['HVAR']);
			$len  = $v['COLLENGTH'];
			$retval['columns'][] = array("colname"=>$colname,"colno"=>$colno,"coltype"=>$coltype,
			"extended_id"=>$extended_id,"notnull"=>$notnull,"hvar"=>$hvar,"collength"=>$len);
		}
		
		$retval['uniqueColSet'] = $this->findUniqueColumn($tabid,$database);
		
		return $retval;
	}
	
	/**
	 * Find the column or set of columns that can be used to uniquely 
 	 * identify rows in this table. 
	 * 
	 * There are 2 possibilities:
	 * (a) rowid - for non-fragemented tables
	 * (b) unique indexes for unique constraint, primary key constraint,
	 *       or unique index
	 * 
	 * @param tabid of the table
	 * @return set of columns that uniquely identify rows in that table
	 **/
	private function findUniqueColumn($tabid, $database) 
	{
		$unique_columns = array();
		
		/* Is there are rowid for this table? */
		$sql ="SELECT tabid, partnum  FROM systables WHERE tabid = " . $tabid . "";
		$res = $this->doDatabaseWork($sql,$database);
		if (trim($res[0]['PARTNUM']) != 0)
		{
			// If partnum != 0, there will be unique rowids for each row in the table
			$unique_columns[] = "rowid";
			return $unique_columns;
		}
		
		/* If not, is there a unqiue index on this table?
		 * Unique indexes are created internally for primary key or unique constraints 
		 * or for unique indexes. */
		
		/* We want to find the unique index on the fewest columns.  So we'll start
		 * by looking for a unique index on a single column (only part1 is non-zero),
		 * then for a unique index on two columns (only part1 and part2 are non-zero), 
		 * etc. searching up to indexes with 16 colums until we find the unique index 
		 * on this table with the fewest columns. */ 
		for( $i=2 ; $i < 17 ; $i++ )
		{
			$sql  = "SELECT * FROM sysindexes WHERE";
			$sql .= " ( ( idxname in  ( " ;
			$sql .= "        select idxname from sysconstraints ";
			$sql .= "        where constrtype in ('U','P') ) )" ;
			$sql .= "    OR idxtype='U'  ) " ;
			$sql .= " AND part" .$i. "=0 " ;
			$sql .= " AND tabid = " . $tabid  ;
			$res = $this->doDatabaseWork($sql,$database);

			// If we found a unique index, find the column names that 
			// make up that index.
			if (count($res) > 0)
			{
				$col_where="";
				foreach( $res[0] as $index => $val )
				{
					if ( (strncmp("PART",$index,4) == 0 ) && $val>0 )
	                {
						if ( $col_where == "" )
						{
							$col_where = " colno IN ( " . abs($val) ;
						} else {
							$col_where .= " , " . abs($val) ;
						}
					}
				}
				$col_where .= " ) ";
				$sql  = "SELECT colname  FROM syscolumns ";
				$sql .= " WHERE tabid = " . $tabid . " AND " . $col_where;
                $res = $this->doDatabaseWork($sql,$database);

                foreach ($res as $row)
                {
                    $unique_columns[] = $row['COLNAME'];
                }
                return $unique_columns;
			}	
		}
		
		return $unique_columns;
	}
	
	/**
	 * Get the data from the table.
	 * 
	 * @param $dbname - database name
	 * @param $owner - table owner
	 * @param $tabname - table name
	 * @param $filters - array of where clause filters
	 * @param $rowid - boolean indicating if we need to return rowids for this table
	 */
	function getDataForTable($dbname,$owner,$tabname,$filters,$rowid=false)
	{
		$filter_array = unserialize($filters);

		$query = " SELECT * ";
		if ($rowid) $query .= ", rowid ";
		$query .= "FROM {$owner}.{$tabname} ";
		$where_clause = "";
		$where_clause_parameters = array();
		$count = 0;
	    foreach ( $filter_array as $column => $columnFilters )
	    {
	    	if (count($columnFilters) > 0)
			{
				$colFilterCount = 0;
				$colFilterStr = "";
	    		foreach ($columnFilters as $colFilter)
	    		{
	    			if ($colFilterCount > 0)
	    			{
	    				$colFilterStr .= "{$colFilter['BOOLEAN_OPERATOR']} ";
	    			}
	    			if ($colFilter['OPERATOR'] == "IS NULL" || $colFilter['OPERATOR'] == "IS NOT NULL")
	    			{
	    				$colFilterStr .= "$column {$colFilter['OPERATOR']} ";
	    			} else {
	    				$colFilterStr .= "$column {$colFilter['OPERATOR']} ? ";
	    				$where_clause_parameters[] = $colFilter['VALUE'];
	    			}
	    			$colFilterCount++; 
	    		}
	    		if ($count > 0)
	    		{
	    			$where_clause .= "AND ";
	    		}
	    		if ($colFilterCount > 1)
	    		{
	    			$where_clause .= "(" . $colFilterStr . ")";
	    		} else {
	    			$where_clause .= $colFilterStr;
	    		}
				$count++;	
			}
	    }
	    
	    if ($count > 0)
	    {
	    	$query .= "WHERE $where_clause";
	    }
	    
	    return $this->doDatabaseWork($query,$dbname,$where_clause_parameters);
	}

	/**
	 * Insert row into table. 
	 * 
	 * @param $dbname database name
	 * @param $owner table owner name
	 * @param $tabname table name
	 * @param $columnData serialized array mapping column name to the value to be inserted 
	 *           e.g.  array('COL1' => '100', 'COL2' => '200');
	 * @return return code/message from executing the insert statement
	 */
	function insertRow($dbname,$owner,$tabname,$columnData)
	{
		$columnData = unserialize($columnData);
		
		$sql = "INSERT INTO {$owner}.{$tabname} (";
		$count = 0;
		$params = array();
		foreach ($columnData as $colname => $value)
		{
			if ($count > 0) $sql .= ", ";
			$sql .= "$colname";
			$params[] = ($value == "null")? null: $value;
			$count++;
		}
		$sql .= ") VALUES (";
		for ($i = 0; $i < $count; $i++)
		{
			if ($i > 0) $sql .= ", ";
			$sql .= "?";
		}
		$sql .= ")";
		
		$db = $this->idsadmin->get_database($dbname, true);
		$stmt = $db->prepare($sql);
		$err = $db->errorInfo();
		if ($err[1] != 0) 
		{
			return array('RETURN_CODE' => $err[1], 'RETURN_MSG' => $err[2]);
		}
		
		$stmt->execute($params);
		$err = $db->errorInfo();
		$result = array('RETURN_CODE' => $err[1], 'RETURN_MSG' => $err[2]);
		return $result; 
	}
	
	
	/**
	 * Update row in the table. 
	 * 
	 * @param $dbname database name
	 * @param $owner table owner name
	 * @param $tabname table name
	 * @param $updateColumns serialized array mapping column name to the value for the 
	 *           columns to be updated in the table.  
	 *           e.g.  array('COL1' => '101', 'COL2' => '201');
	 * @param $uniqueColumns serialized array mapping column name to the value for the 
	 *           where clause of the delete statement.  
	 *           e.g.  array('COL1' => '100', 'COL2' => '200');
	 * @return return code/message from executing the update statement
	 */
	function updateRow($dbname,$owner,$tabname,$updateColumns,$uniqueColumns)
	{
		$updateColumns = unserialize($updateColumns);
		$uniqueColumns = unserialize($uniqueColumns);
		
		if (count($uniqueColumns) == 0)
		{
			return array('RETURN_CODE' => -1, 
				'RETURN_MSG' => 'Cannot udpate row on the database server because there are no unique columns for this table.');
		}
		
		$sql = "UPDATE {$owner}.{$tabname} SET ";
		$count = 0;
		$params = array();
		foreach ($updateColumns as $colname => $value)
		{
			if ($count > 0) $sql .= ", ";
			$sql .= "$colname = ?";
			$params[] = ($value == "null")? null: $value;
			$count++;
		}
		
		$count = 0;
		$where_clause = " WHERE ";
		$where_params = array();
		foreach ($uniqueColumns as $colname => $value)
		{
			if ($count > 0 ) $where_clause .= " AND ";
			$where_clause .= "$colname = ?";
			$where_params[] = $value;
			$count++;
		}
		$sql .= $where_clause;
		$params = array_merge($params,$where_params);
		
		$db = $this->idsadmin->get_database($dbname, true);
		$stmt = $db->prepare($sql);
		$err = $db->errorInfo();
		if ($err[1] != 0) 
		{
			return array('RETURN_CODE' => $err[1], 'RETURN_MSG' => $err[2]);
		}
		
		$stmt->execute($params);
		$err = $db->errorInfo();
		$result = array('RETURN_CODE' => $err[1], 'RETURN_MSG' => $err[2]);
		
		// Requery the server for that row
		$qry  = "SELECT * ";
		$qry .= isset($uniqueColumns['rowid'])? ", rowid ":"";
		$qry .= "FROM {$owner}.{$tabname} ";
		$qry .= $where_clause;
		$result['ROW'] = $this->doDatabaseWork($qry,$dbname,$where_params);
		
		return $result; 
	}
	
	/**
	 * Delete row on the table. 
	 * 
	 * @param $dbname database name
	 * @param $owner table owner name
	 * @param $tabname table name
	 * @param $uniqueColumns serialized array mapping column name to the value for the 
	 *           where clause of the delete statement.  
	 *           e.g.  array('COL1' => '100', 'COL2' => '200');
	 * @return return code/message from executing the delete statement
	 */
	function deleteRow($dbname,$owner,$tabname,$uniqueColumns)
	{
		$uniqueColumns = unserialize($uniqueColumns);
		
		if (count($uniqueColumns) == 0)
		{
			return array('RETURN_CODE' => -1, 
				'RETURN_MSG' => 'Cannot delete row on the database server because there are no unique columns for this table.');
		}
		
		$sql = "DELETE FROM {$owner}.{$tabname} WHERE ";
		$count = 0;
		$params = array();
		foreach ($uniqueColumns as $colname => $value)
		{
			if ($count > 0 ) $sql .= " AND ";
			$sql .= "$colname = ?";
			$params[] = $value;
			$count++;
		}		
		
		$db = $this->idsadmin->get_database($dbname, true);
		$stmt = $db->prepare($sql);
		$err = $db->errorInfo();
		if ($err[1] != 0) 
		{
			return array('RETURN_CODE' => $err[1], 'RETURN_MSG' => $err[2]);
		}
		
		$stmt->execute($params);
		$err = $db->errorInfo();
		$result = array('RETURN_CODE' => $err[1], 'RETURN_MSG' => $err[2]);
		return $result; 
	}
	
    /**
	 * Test if user has connect permission on the database. 
	 * 
	 * @param $dbname
	 * @return true/false indicating if the user has connect permission on the database
	 */
	private function testForDatabaseConnectPermission($dbname)
	{
	    putenv("INFORMIXCONTIME={$this->idsadmin->get_config("INFORMIXCONTIME",20)}");
		putenv("INFORMIXCONRETRY={$this->idsadmin->get_config("INFORMIXCONRETRY",3)}");
		
		$dsn = "informix:host={$this->idsadmin->phpsession->instance->get_host()}";
		$dsn .= ";service={$this->idsadmin->phpsession->instance->get_port()}";
		$dsn .= ";database={$dbname}";
		$dsn .= ";protocol={$this->idsadmin->phpsession->instance->get_idsprotocol()}";
		$dsn .= ";server={$this->idsadmin->phpsession->instance->get_servername()}";
		 
		$informixdir = $this->idsadmin->get_config("INFORMIXDIR");
		if ( substr(PHP_OS,0,3) != "WIN" )
		{
			$libsuffix = (strtoupper(substr(PHP_OS,0,3)) == "DAR")? "dylib":"so";
			$dsn .= ";TRANSLATIONDLL={$informixdir}/lib/esql/igo4a304.".$libsuffix;
			$dsn .= ";Driver={$informixdir}/lib/cli/libifdmr.".$libsuffix.";";
		}

		$locale = $this->idsadmin->get_locale($dbname);
		if ( $locale != "" )
		{
			$dsn .= ";CLIENT_LOCALE={$locale};DB_LOCALE={$locale};";
		}
		
		$user   = (isset($_SESSION['SQLTOOLBOX_USERNAME']))? $_SESSION['SQLTOOLBOX_USERNAME']:$this->idsadmin->phpsession->instance->get_username();
		$passwd = (isset($_SESSION['SQLTOOLBOX_PASSWORD']))? $_SESSION['SQLTOOLBOX_PASSWORD']:$this->idsadmin->phpsession->instance->get_passwd();	

		try {
			$testconn = new PDO($dsn,$user,$passwd);
			$testconnn = null;
		} catch(PDOException $e) {
		    // User could not connect to the databse
		    error_log("QBE: $user cannot connect to database $dbname.  " . $e->getMessage());
		    $testconnn = null;
		    return false;
		}
		
		// If we didn't get exception, the user has connect permission to the database
		return true;
	}
	
	
	/**
	 * do the database work.
	 *
	 */
	function doDatabaseWork($sel,$dbname="sysmaster",$params=array())
	{
		$ret = array();
		
		$db = $this->idsadmin->get_database($dbname, true);

		while (1 == 1)
		{
			if (count($params) == 0)
        		$stmt = $db->query($sel);
			else {
				$stmt = $db->prepare($sel);
				$stmt->execute($params);
			}
			
			while ($row = $stmt->fetch(PDO::FETCH_ASSOC) )
			{
	    		$ret[] = $row;
			}
			$err = $db->errorInfo();
			if ( $err[2] == 0 )
			{
				$stmt->closeCursor();
				break;
			}
			else
			{
				$err = "Error: {$err[2]} - {$err[1]}";
				$stmt->closeCursor();
				trigger_error($err,E_USER_ERROR);
				continue;
			}
		}

		return $ret;
	}
}

?>
