<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007 , 2008.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */


class sessionsServer {

	var $idsadmin;

	function __construct()
	{
		define ("ROOT_PATH","../../");
		define( 'IDSADMIN',  "1" );
		define( 'DEBUG', false);
		define( 'SQLMAXFETNUM' , 100 );


		include_once("../serviceErrorHandler.php");
		set_error_handler("serviceErrorHandler");

		require_once(ROOT_PATH."lib/idsadmin.php");
		$this->idsadmin = new idsadmin(true);
		$this->idsadmin->load_lang("sessions");
	}
	/**
	 * check that session exists.
	 *
	 */
	function checkSession($sessionId)
	{
		$sessionId = intval($sessionId);
		$sel = "select sid from syssessions where sid = {$sessionId} ";
		$res = $this->doDatabaseWork($sel,"sysmaster");
		if ( sizeof($res) == 0 )
		{
			trigger_error("{$this->idsadmin->lang('ErrorNoLongerExists')}",E_USER_WARNING);
		}
	}
	/**
	 * get the list of sessions
	 *
	 */
	function getSessionList()
	{
		if ( $this->idsadmin->isreadonly() )
		{
			$killable = " , 0 as is_killable ";
		}
		else
		{
			$killable = ", case (trim(c.name))  when 'sqlexec' then 1 else 0 end as is_killable ";
		}
		$sel = " SELECT "
		." a.sid AS sid"
		.",a.username"
		.",a.pid"
		.",trim(a.hostname)||decode(length(trim(a.ttyerr)),0,'',':'||a.ttyerr) AS hostname"
		.",DBINFO('UTC_TO_DATETIME', a.connected) AS connected"
		.",memtotal"
		.",iowaittime::decimal(16,3) AS iowaittime"
		.",cpu_time::decimal(16,3) AS cpu_time"
		."{$killable} "
		." FROM sysscblst a, sysrstcb b, systcblst c "
		." WHERE a.address = b.scb "
		." AND a.sid != DBINFO('sessionid') "
		." AND  b.tid = c.tid "
		." ORDER BY a.sid ";

		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionList

	function getSessionInfo($sessionId)
	{
		$this->checkSession($sessionId);

		$sel = "SELECT " .
        " sysscblst.sid, " .
        " trim(sysscblst.username) as user, " .
        " decode(length(hostname),0,'localhost',hostname) as hostname, " .
        " sysscblst.uid as userid, " .
        " sysscblst.gid, " .
        " sysscblst.pid, " .
        " dbinfo('UTC_TO_DATETIME',connected)::DATETIME MONTH TO SECOND as con, " .
        " format_units(memtotal,'b') as mtotal, " .
        " format_units(memused,'b') as mused," .
        " nfiles ".
        " ,cbl_stmt[1,1000] AS curr_stmt ".
        " FROM sysscblst , sysrstcb , outer sysconblock  " .
        " WHERE sysscblst.address = sysrstcb.scb " .
        " AND cbl_sessionid = sysscblst.sid ".
        " AND bitval(sysrstcb.flags,'0x80000')>0 " .
        " AND sysscblst.sid = {$sessionId} ";

		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionInfo

	function getSessionProfile($sessionId)
	{
		$this->checkSession($sessionId);
		$sel = "SELECT " .
        " sysscblst.sid, " .
        " nreads, " .
        " nwrites, " .
        " nfiles, " .
        " upf_rqlock, " .
        " upf_wtlock, " .
        " upf_deadlk, " .
        " upf_lktouts, " .
        " upf_lgrecs, " .
        " upf_isread, " .
        " upf_iswrite, " .
        " upf_isrwrite, " .
        " upf_isdelete, " .
        " upf_iscommit, " .
        " upf_isrollback, " .
        " upf_longtxs, " .
        " upf_bufreads, " .
        " upf_bufwrites, " .
        " format_units(upf_logspuse,'b') as logspuse, " .
        " format_units(upf_logspmax,'b') as logspmax, " .
        " upf_seqscans, " .
        " upf_totsorts, " .
        " upf_dsksorts, " .
        " upf_totsorts - upf_dsksorts as memsorts, " .
        " format_units(upf_srtspmax,'b') as srtspmax, " .
        " nlocks " .
        " FROM sysscblst , sysrstcb " .
        " WHERE sysscblst.address = sysrstcb.scb " .
        " AND bitval(sysrstcb.flags,'0x80000')>0 " .
        " AND sysscblst.sid = {$sessionId} ";

		//error_log($sel);
		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionProfile
	/**
	 * Get network information for a session
	 */
	function getSessionNetwork($sessionId)
	{
		$this->checkSession($sessionId);
		$sel = " SELECT "
		." net_client_name  thread_name "
		.",format_units( net_read_bytes ) RECIEVED_DATA "
		.",format_units( net_write_bytes ) SEND_DATA "
		.",decode ( net_read_cnt , 0 , '0' ,format_units( net_read_bytes/net_read_cnt ) ) AVERAGE_RECV "
		.",decode ( net_write_cnt , 0 , '0' ,format_units( net_write_bytes/net_write_cnt ) ) AVERAGE_SEND "
		.",CURRENT - dbinfo('UTC_TO_DATETIME', net_open_time)  CONNECT_DURATION "
		.",dbinfo('UTC_TO_DATETIME', net_open_time) SESSION_START_TIME "
		.",dbinfo('UTC_TO_DATETIME', net_last_read) as LAST_READ "
		.",dbinfo('UTC_TO_DATETIME', net_last_write) as LAST_WRITE "
		." FROM sysnetworkio "
		." where sid = {$sessionId} ";

		return $this->doDatabaseWork($sel,"sysmaster");
	} // end getSessionNetwork
	/**
	 * get the sql for a session
	 *
	 */
	function getSessionSQL($sessionId)
	{
		$this->checkSession($sessionId);
		$sel = "select  "
		." sql_id as id "
		." ,sql_stmtname as type "
		." ,trim(sql_statement[1,1000]) as statement "
 
		." from syssqltrace a"
		." where a.sql_sid != dbinfo('sessionid') "
		." and a.sql_sid  = {$sessionId} "
		." order by a.sql_sid ";
			
		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionSQL

	function getSessionMem($sessionId)
	{
		$this->checkSession($sessionId);
		$sel = "SELECT "
		." po_name as name "
		." ,format_units(po_usedamt,'B') as used "
		." ,format_units(po_freeamt,'B') as free "
		." FROM syspoollst "
		." WHERE po_name matches '*_{$sessionId}_*' OR po_name = '{$sessionId}' ";
		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionSQL

	function getSessionSQLInfo($id)
	{
		$this->checkSession($sessionId);
		$sel = "select  "
		." * "
		." from syssqltrace a"
		." where a.sql_sid != dbinfo('sessionid') "
		." and a.sql_id  = {$id} ";

		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionSQLInfo

	function getSessionThreads($id)
	{
		$this->checkSession($id);
		$sel = "select  "
		." * "
		." from sysuserthreads a"
		." where a.us_sid != dbinfo('sessionid') "
		." and a.us_sid  = {$id} ";

		$sel = " select "
		." name                NAME "
		.",sysrstcb.tid        THREAD_ID "
		.",wait_reason         WAIT_REASON "
		.",num_sched           NUM_SCHEDULED "
		.",cpu_time            TOTAL_TIME "
		.",cpu_time/num_sched  TIME_SLICE "
		.",vpid                VPID "
		.",priority            THREAD_PRIORITY "
		." from sysrstcb, systcblst "
		." where "
		." systcblst.tid = sysrstcb.tid "
		." AND sysrstcb.sid = {$id} ";

		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionSQL

	function getSessionEnv($id)
	{
		$this->checkSession($id);
		$sel = " SELECT  "
		." 'CLIENT' as location"
		." ,envses_name AS name "
		." ,envses_value AS value "
		." FROM sysenvses a"
		." WHERE a.envses_sid != dbinfo('sessionid') "
		." AND a.envses_sid  = {$id} "
		." UNION "
		." SELECT  "
		." 'SERVER' AS location "
		." ,env_name AS name "
		." ,env_value AS value "
		." FROM sysenv a "
		." WHERE env_name NOT IN ( SELECT envses_name FROM sysenvses WHERE envses_sid  = {$id}  ) "
		."  "
		." order by name ";

		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionSQL

	function getSessionThreadInfo($id)
	{
		$this->checkSession($sessionId);

		$sel = "select  "
		." * "
		." from syssqltrace a"
		." where a.sql_sid != dbinfo('sessionid') "
		." and a.sql_id  = {$id} ";

		return $this->doDatabaseWork($sel,"sysmaster");

	} // end getSessionThreadInfo
	/**
	 * get the locks for the session
	 */
	function getSessionLocks($sessionId)
	{
		$this->checkSession($sessionId);

		$sessionId = intval($sessionId);
		$sel = "select  /*+ORDERED*/ ".
        " trim(dbsname)||':'||trim(b.tabname) AS TABLE_NAME,
		e.txt[1,6]  LOCK_TYPE,
		(CURRENT - DBINFO('UTC_TO_DATETIME', grtime))::INTERVAL HOUR(5) TO SECOND LOCK_HELD,

		decode(a.wtlist,0,'NONE',f.sid||' ') OTHERS_WAITING_FOR_LOCK,
		hex(rowidr) ROWID,
		keynum INDEX_NUMBER,
		hex(rowidn) KEY_ITEM_LOCKED
		from  sysrstcb d
		, systxptab c
		, syslcktab a
		, systabnames b
		, flags_text e
		, outer sysrstcb f
		where a.partnum = b.partnum
		and a.owner = c.address
		and c.owner = d.address
		and a.wtlist = f.address
		and e.tabname = 'syslcktab'
		and e.flags = a.type
		and d.sid = {$sessionId} ";
			
		return $this->doDatabaseWork($sel,"sysmaster");
	}
	/**
	 * killSession - tries to kill a session
	 * @param int - sessionId
	 */
	function killSession($sessiondId=0)
	{
		//error_log("sessionId = {$sessiondId}");
		if ( intval($sessiondId) > 0 )
		{
			$qry = "select task ('onmode' , 'z' "
			. ",'{$sessiondId}') as res from systables where tabid=1";
			return $this->doDatabaseWork($qry,"sysadmin");
		}
		trigger_error("killSession","{$this->idsadmin->lang('ErrorInvalidSessionID')}",E_USER_ERROR);
	}
	/**
	 * do the database work.
	 *
	 */
	function doDatabaseWork($sel,$dbname="sysmaster")
	{
		$ret = array();

		$db = $this->idsadmin->get_database($dbname);

		while (1 == 1)
		{
			$stmt = $db->query($sel);

			while ($row = $stmt->fetch() )
			{
				$ret[] = $row;
			}
			$err = $db->errorInfo();
			if ( $err[2] == 0 )
			{
				$stmt->closeCursor();
				break;
			}
			else
			{
				$err = "{$this->idsadmin->lang('ErrorF')} {$err[2]} - {$err[1]}";
				$stmt->closeCursor();
				trigger_error($err,E_USER_ERROR);
				continue;
			}
		}
		//error_log(var_export($err,true));
		//error_log(var_export($ret,true));

		return $ret;
	}
}

?>